@@ -13,7 +13,7 @@
 # This file is part of libSBML.  Please visit http://sbml.org for more
 # information about SBML, and the latest version of libSBML.
 #
-# Copyright (C) 2009-2011 jointly by the following organizations: 
+# Copyright (C) 2009-2012 jointly by the following organizations: 
 #     1. California Institute of Technology, Pasadena, CA, USA
 #     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
 #  
@@ -58,6 +58,7 @@ libsbmlclasses = ["AlgebraicRule",
                   "FunctionDefinition",
                   "InitialAssignment",
                   "KineticLaw",
+                  "libsbml",
                   "List",
                   "ListOf",
                   "ListOfCompartmentTypes",
@@ -86,6 +87,7 @@ libsbmlclasses = ["AlgebraicRule",
                   "OStream",
                   "OStringStream",
                   "Parameter",
+                  "Priority",
                   "RDFAnnotationParser",
                   "RateRule",
                   "Reaction",
@@ -116,8 +118,8 @@ libsbmlclasses = ["AlgebraicRule",
                   "SBMLWriter",
                   "SBO",
                   "SBase",
-                  "SBaseList",
                   "SBaseExtensionPoint",
+                  "SBaseList",
                   "SBasePlugin",
                   "SBasePluginCreator",
                   "SBasePluginCreatorBase",
@@ -131,11 +133,11 @@ libsbmlclasses = ["AlgebraicRule",
                   "Unit", 
                   "UnitDefinition",
                   "XMLAttributes",
+                  "XMLConstructorException",
                   "XMLError",
                   "XMLErrorLog",
                   "XMLNamespaces",
                   "XMLNode",
-                  "libsbml",
                   "XMLToken",
                   "XMLTriple" ]
 
@@ -871,8 +873,8 @@ def rewriteDocstringForJava (docstring):
   # Fix up for a problem introduced by sanitizeForHTML -- should fix
   # properly some day.
 
-  p = re.compile('(@see\s+[\w ,(]*){@link\s+([^}]+?)}')
-  docstring = p.sub(r'\1\2', docstring)
+  p = re.compile('((@see|@throws)\s+[\w ,(]*){@link\s+([^}]+?)}')
+  docstring = p.sub(r'\1\3', docstring)
 
   # Inside of @see, change double colons to pound signs.
 
@@ -1018,6 +1020,8 @@ def rewriteDocstringForPython (docstring):
   docstring = docstring.replace(r'const std::string', 'string')
   docstring = docstring.replace(r'std::string', 'string')
   docstring = docstring.replace(r'NULL', 'None')
+  docstring = docstring.replace(r'@c true', '@c True')
+  docstring = docstring.replace(r'@c false', '@c False')
 
   # Also use Python syntax instead of "const XMLNode*" etc.
 
@@ -1137,7 +1141,7 @@ def processClassMethods(ostream, cclass):
           continue
 
         newdoc = ' This method has multiple variants that differ in the' + \
-                 ' arguments they accept.  Each is described separately' + \
+                 ' arguments\n they accept.  Each is described separately' + \
                  ' below.\n'
 
         for argVariant in cclass.methodVariants[m.name].values():
@@ -1198,11 +1202,11 @@ def formatMethodDocString (methodname, classname, docstring, isInternal, args=No
     output += classname + '::'
 
   if language == 'perl':
-    output += '%s\n\n%s%s\n\n\n'   % (methodname, docstring, post)
+    output += '%s\n\n%s%s\n\n\n'   % (methodname, docstring.strip(), post)
   elif language == 'python':
-    output += '%s "\n%s%s";\n\n\n' % (methodname, docstring, post)
+    output += '%s "\n %s%s\n";\n\n\n' % (methodname, docstring.strip(), post)
   else:
-    output += '%s%s "\n%s%s";\n\n\n' % (methodname, args, docstring, post)
+    output += '%s%s "\n%s%s\n";\n\n\n' % (methodname, args, docstring.strip(), post)
 
   return output
 
@@ -1224,22 +1228,22 @@ def generateFunctionDocString (methodname, docstring, args, isInternal):
 def generateClassDocString (docstring, classname):
   if language == 'java':
     pre = '%typemap(javaimports) '
-    docstring = rewriteDocstringForJava(docstring)    
-    output = pre + classname + ' "\n' + docstring + '"\n\n'
+    docstring = rewriteDocstringForJava(docstring).strip()
+    output = pre + classname + ' "\n' + docstring + '\n"\n\n\n'
 
   elif language == 'csharp':
     pre = '%typemap(csimports) '
-    docstring = rewriteDocstringForCSharp(docstring)    
-    output = pre + classname + ' "\n using System;\n using System.Runtime.InteropServices;\n\n' + docstring + '"\n\n'    
+    docstring = rewriteDocstringForCSharp(docstring).strip()
+    output = pre + classname + ' "\n using System;\n using System.Runtime.InteropServices;\n\n' + docstring + '\n"\n\n\n'
 
   elif language == 'python':
     pre = '%feature("docstring") '
-    docstring = rewriteDocstringForPython(docstring)
-    output = pre + classname + ' "\n' + docstring + '"\n\n'
+    docstring = rewriteDocstringForPython(docstring).strip()
+    output = pre + classname + ' "\n ' + docstring + '\n";\n\n\n'
 
   elif language == 'perl':
-    docstring = rewriteDocstringForPerl(docstring)
-    output = '=back\n\n=head2 ' + classname + '\n\n' + docstring + '\n\n=over\n\n'
+    docstring = rewriteDocstringForPerl(docstring).strip()
+    output = '=back\n\n=head2 ' + classname + '\n\n' + docstring + '\n\n=over\n\n\n'
 
   return output
 
