@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -314,7 +314,9 @@
 
 #include <sbml/common/extern.h>
 #include <sbml/common/sbmlfwd.h>
+#ifndef LIBSBML_USE_STRICT_INCLUDES
 #include <sbml/annotation/RDFAnnotation.h>
+#endif
 #include <sbml/common/operationReturnValues.h>
 
 #ifdef __cplusplus
@@ -342,6 +344,11 @@ public:
    *
    * @param version an unsigned int, the SBML Version to assign to this
    * Compartment
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the given @p level and @p version combination, or this kind
+   * of SBML object, are either invalid or mismatched with respect to the
+   * parent SBMLDocument object.
    * 
    * @note Upon the addition of a Compartment object to an SBMLDocument
    * (e.g., using Model::addCompartment(@if java Compartment c@endif)), the SBML Level, SBML Version
@@ -377,6 +384,11 @@ public:
    *
    * @param sbmlns an SBMLNamespaces object.
    *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the given @p level and @p version combination, or this kind
+   * of SBML object, are either invalid or mismatched with respect to the
+   * parent SBMLDocument object.
+   *
    * @note Upon the addition of a Compartment object to an SBMLDocument
    * (e.g., using Model::addCompartment(@if java Compartment c@endif)), the SBML XML namespace of the
    * document @em overrides the value used when creating the Compartment
@@ -401,12 +413,21 @@ public:
    * Copy constructor; creates a copy of a Compartment.
    * 
    * @param orig the Compartment instance to copy.
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p orig is @c NULL.
    */
   Compartment(const Compartment& orig);
 
 
   /**
    * Assignment operator for Compartment.
+   *
+   * @param rhs The object whose values are used as the basis of the
+   * assignment.
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p rhs is @c NULL.
    */
   Compartment& operator=(const Compartment& rhs);
 
