@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -407,7 +407,7 @@ CompartmentType::readL2Attributes (const XMLAttributes& attributes)
   //
   // id: SId  { use="required" }  (L2v2 ->)
   //
-  bool assigned = attributes.readInto("id", mId, getErrorLog(), true);
+  bool assigned = attributes.readInto("id", mId, getErrorLog(), true, getLine(), getColumn());
   if (assigned && mId.size() == 0)
   {
     logEmptyString("id", level, version, "<compartmentType>");
@@ -417,7 +417,7 @@ CompartmentType::readL2Attributes (const XMLAttributes& attributes)
   //
   // name: string  { use="optional" }  (L2v2 ->)
   //
-  attributes.readInto("name", mName);
+  attributes.readInto("name", mName, getErrorLog(), false, getLine(), getColumn());
 }
 /** @endcond */
 
@@ -479,6 +479,7 @@ ListOfCompartmentTypes::ListOfCompartmentTypes (unsigned int level, unsigned int
 ListOfCompartmentTypes::ListOfCompartmentTypes (SBMLNamespaces* sbmlns)
   : ListOf(sbmlns)
 {
+  loadPlugins(sbmlns);
 }
 
 
