@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -211,6 +211,11 @@ public:
    *
    * @param version an unsigned int, the SBML Version to assign to this
    * EventAssignment
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the given @p level and @p version combination, or this kind
+   * of SBML object, are either invalid or mismatched with respect to the
+   * parent SBMLDocument object.
    * 
    * @note Upon the addition of a EventAssignment object to an SBMLDocument
    * (e.g., using Event::addEventAssignment(@if java EventAssignment ea@endif)), 
@@ -240,6 +245,11 @@ public:
    *
    * @param sbmlns an SBMLNamespaces object.
    *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the given @p level and @p version combination, or this kind
+   * of SBML object, are either invalid or mismatched with respect to the
+   * parent SBMLDocument object.
+   *
    * @note Upon the addition of a EventAssignment object to an SBMLDocument
    * (e.g., using Event::addEventAssignment(@if java EventAssignment ea@endif)),
    * the SBML XML namespace of the document @em overrides the value used
@@ -263,12 +273,23 @@ public:
 
   /**
    * Copy constructor; creates a copy of this EventAssignment.
+   *
+   * @param orig the object to copy.
+   * 
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p orig is @c NULL.
    */
   EventAssignment (const EventAssignment& orig);
 
 
   /**
    * Assignment operator.
+   *
+   * @param rhs The object whose values are used as the basis of the
+   * assignment.
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p rhs is @c NULL.
    */
   EventAssignment& operator=(const EventAssignment& rhs);
 
@@ -577,6 +598,7 @@ public:
    */
   virtual bool hasRequiredElements() const ;
 
+
   /** @cond doxygen-libsbml-internal */
   /*
    * Return the variable attribute of this object.
@@ -887,6 +909,16 @@ public:
   virtual EventAssignment* remove (const std::string& sid);
 
 
+  /**
+   * Returns the first child element found that has the given id in the model-wide SId namespace, or NULL if no such object is found.  Note that EventAssignments do not actually have IDs, but the libsbml interface pretends that they do:  no event assignment is returned by this function.
+   *
+   * @param id string representing the id of objects to find
+   *
+   * @return pointer to the first element found with the given id.
+   */
+  virtual SBase* getElementBySId(std::string id);
+  
+  
   /** @cond doxygen-libsbml-internal */
 
   /**
