@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -453,6 +453,16 @@ FunctionDefinition::getBody ()
 }
 
 
+/*
+ * @return the true if the body of this FunctionDefinition is set
+ */
+bool
+FunctionDefinition::isSetBody () const
+{
+  return (getBody() == NULL) ? false : true;
+}
+
+
 /*
  * @return the number of arguments (bound variables) that must be passed
  * to this FunctionDefinition.
@@ -680,7 +690,7 @@ FunctionDefinition::readL2Attributes (const XMLAttributes& attributes)
   // id: SId  { use="required" }  (L2v1 ->)
   //
   bool assigned;
-  assigned = attributes.readInto("id", mId, getErrorLog(), true);
+  assigned = attributes.readInto("id", mId, getErrorLog(), true, getLine(), getColumn());
   if (assigned && mId.size() == 0)
   {
     logEmptyString("id", level, version, "<functionDefinition>");
@@ -690,7 +700,7 @@ FunctionDefinition::readL2Attributes (const XMLAttributes& attributes)
   //
   // name: string  { use="optional" }  (L2v1 ->)
   //
-  attributes.readInto("name", mName);
+  attributes.readInto("name", mName, getErrorLog(), false, getLine(), getColumn());
 
   //
   // sboTerm: SBOTerm { use="optional" }  (L2v2 ->)
@@ -719,7 +729,7 @@ FunctionDefinition::readL3Attributes (const XMLAttributes& attributes)
   // id: SId  { use="required" }  (L2v1 ->)
   //
   bool assigned;
-  assigned = attributes.readInto("id", mId, getErrorLog());
+  assigned = attributes.readInto("id", mId, getErrorLog(), false, getLine(), getColumn());
   if (!assigned)
   {
     logError(AllowedAttributesOnFunc, level, version);
@@ -733,7 +743,7 @@ FunctionDefinition::readL3Attributes (const XMLAttributes& attributes)
   //
   // name: string  { use="optional" }  (L2v1 ->)
   //
-  attributes.readInto("name", mName);
+  attributes.readInto("name", mName, getErrorLog(), false, getLine(), getColumn());
 
 }
 /** @endcond */
@@ -824,6 +834,7 @@ ListOfFunctionDefinitions::ListOfFunctionDefinitions (unsigned int level, unsign
 ListOfFunctionDefinitions::ListOfFunctionDefinitions (SBMLNamespaces* sbmlns)
 : ListOf(sbmlns)
 {
+  loadPlugins(sbmlns);
 }
 
 
@@ -1373,6 +1384,25 @@ FunctionDefinition_getBody (const FunctionDefinition_t *fd)
 }
 
 
+/**
+ * Predicate returning @c true or @c false depending on whether the given
+ * FunctionDefinition_t structure's "math" subelement has a body.
+ *
+ * @param fd the FunctionDefinition_t structure
+ *
+ * @return nonzero if the body of this
+ * FunctionDefinition is set, zero (0) otherwise.
+ */
+LIBSBML_EXTERN
+int
+FunctionDefinition_isSetBody (const FunctionDefinition_t *fd)
+{
+  return (fd != NULL) ? static_cast<int>( fd->isSetBody() ) : 0;
+}
+
+
+
+
 /**
  * Get the number of arguments that the given function definition takes.
  *
