@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -167,6 +167,11 @@ public:
    *
    * @param version an unsigned int, the SBML Version to assign to this
    * FunctionDefinition
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the given @p level and @p version combination, or this kind
+   * of SBML object, are either invalid or mismatched with respect to the
+   * parent SBMLDocument object.
    * 
    * @note Upon the addition of a FunctionDefinition object to an
    * SBMLDocument (e.g., using Model::addFunctionDefinition(@if java FunctionDefinition f@endif)), the SBML
@@ -196,6 +201,11 @@ public:
    *
    * @param sbmlns an SBMLNamespaces object.
    *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the given @p level and @p version combination, or this kind
+   * of SBML object, are either invalid or mismatched with respect to the
+   * parent SBMLDocument object.
+   *
    * @note Upon the addition of a FunctionDefinition object to an
    * SBMLDocument (e.g., using Model::addFunctionDefinition(@if java FunctionDefinition f@endif)), the SBML
    * XML namespace of the document @em overrides the value used when
@@ -218,12 +228,23 @@ public:
 
   /**
    * Copy constructor; creates a copy of this FunctionDefinition.
+   *
+   * @param orig the object to copy.
+   * 
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p orig is @c NULL.
    */
   FunctionDefinition (const FunctionDefinition& orig);
 
 
   /**
    * Assignment operator for FunctionDefinition.
+   *
+   * @param rhs The object whose values are used as the basis of the
+   * assignment.
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p rhs is @c NULL.
    */
   FunctionDefinition& operator=(const FunctionDefinition& rhs);
 
@@ -416,6 +437,16 @@ public:
   ASTNode* getBody ();
 
 
+  /**
+   * Predicate returning @c true if the body of this
+   * FunctionDefinition has set.
+   *
+   * @return @c true if the body of this FunctionDefinition is 
+   * set, @c false otherwise.
+   */
+  bool isSetBody () const;
+
+
   /**
    * Get the number of arguments (bound variables) taken by this
    * FunctionDefinition.
@@ -503,6 +534,7 @@ public:
    */
   virtual bool hasRequiredElements() const ;
 
+
   /**
    * Renames all the UnitSIdRef attributes on this element
    */
@@ -923,6 +955,11 @@ const ASTNode_t *
 FunctionDefinition_getBody (const FunctionDefinition_t *fd);
 
 
+LIBSBML_EXTERN
+int
+FunctionDefinition_isSetBody (const FunctionDefinition_t *fd);
+
+
 LIBSBML_EXTERN
 unsigned int
 FunctionDefinition_getNumArguments (const FunctionDefinition_t *fd);
