@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -624,7 +624,7 @@ InitialAssignment::readL2Attributes (const XMLAttributes& attributes)
   //
   // symbol: SId  { use="required" }  (L2v2 -> )
   //
-  bool assigned = attributes.readInto("symbol", mSymbol, getErrorLog(), true);
+  bool assigned = attributes.readInto("symbol", mSymbol, getErrorLog(), true, getLine(), getColumn());
   if (assigned && mSymbol.size() == 0)
   {
     logEmptyString("symbol", level, version, "<initialAssignment>");
@@ -656,7 +656,7 @@ InitialAssignment::readL3Attributes (const XMLAttributes& attributes)
   //
   // symbol: SId  { use="required" }  (L2v2 -> )
   //
-  bool assigned = attributes.readInto("symbol", mSymbol, getErrorLog());
+  bool assigned = attributes.readInto("symbol", mSymbol, getErrorLog(), false, getLine(), getColumn());
   if (!assigned)
   {
     logError(AllowedAttributesOnInitialAssign, level, version);
@@ -723,6 +723,7 @@ ListOfInitialAssignments::ListOfInitialAssignments (unsigned int level, unsigned
 ListOfInitialAssignments::ListOfInitialAssignments (SBMLNamespaces* sbmlns)
   : ListOf(sbmlns)
 {
+  loadPlugins(sbmlns);
 }
 
 
@@ -845,6 +846,20 @@ ListOfInitialAssignments::remove (const std::string& sid)
 }
 
 
+SBase*
+ListOfInitialAssignments::getElementBySId(std::string id)
+{
+  for (unsigned int i = 0; i < size(); i++)
+  {
+    SBase* obj = get(i);
+    //Initial assignments are not in the SId namespace, so don't check 'getId'.  However, their children (through plugins) may have the element we are looking for, so we still need to check all of them.
+    obj = obj->getElementBySId(id);
+    if (obj != NULL) return obj;
+  }
+
+  return getElementFromPluginsBySId(id);
+}
+  
 /** @cond doxygen-libsbml-internal */
 /*
  * @return the ordinal position of the element with respect to its siblings
