@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -171,7 +171,10 @@ bool
 KineticLaw::accept (SBMLVisitor& v) const
 {
   v.visit(*this);
-  mParameters.accept(v);
+  if (getLevel() > 2)
+    mLocalParameters.accept(v);
+  else
+    mParameters.accept(v);
   v.leave(*this);
 
   return true;
@@ -1425,17 +1428,17 @@ KineticLaw::readL1Attributes (const XMLAttributes& attributes)
   //
   // formula: string  { use="required" }  (L1v1->)
   //
-  attributes.readInto("formula", mFormula, getErrorLog(), true);
+  attributes.readInto("formula", mFormula, getErrorLog(), true, getLine(), getColumn());
 
   //
   // timeUnits  { use="optional" }  (L1v1, L1v2, L2v1, L2v2)
   //
-  attributes.readInto("timeUnits", mTimeUnits);
+  attributes.readInto("timeUnits", mTimeUnits, getErrorLog(), false, getLine(), getColumn());
 
   //
   // substanceUnits  { use="optional" }  (L1v1, L1v2, L2v1, L2v2)
   //
-  attributes.readInto("substanceUnits", mSubstanceUnits);
+  attributes.readInto("substanceUnits", mSubstanceUnits, getErrorLog(), false, getLine(), getColumn());
 
 }
 /** @endcond */
@@ -1458,12 +1461,12 @@ KineticLaw::readL2Attributes (const XMLAttributes& attributes)
     //
     // timeUnits  { use="optional" }  (L1v1, L1v2, L2v1)
     //
-    attributes.readInto("timeUnits", mTimeUnits);
+    attributes.readInto("timeUnits", mTimeUnits, getErrorLog(), false, getLine(), getColumn());
 
     //
     // substanceUnits  { use="optional" }  (L1v1, L1v2, L2v1)
     //
-    attributes.readInto("substanceUnits", mSubstanceUnits);
+    attributes.readInto("substanceUnits", mSubstanceUnits, getErrorLog(), false, getLine(), getColumn());
   }
 
   //
