@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -47,6 +47,8 @@ LIBSBML_CPP_NAMESPACE_BEGIN
 ListOf::ListOf (unsigned int level, unsigned int version)
 : SBase(level,version)
 {
+    if (!hasValidLevelVersionNamespaceCombination())
+    throw SBMLConstructorException();
 }
 
 
@@ -56,6 +58,8 @@ ListOf::ListOf (unsigned int level, unsigned int version)
 ListOf::ListOf (SBMLNamespaces* sbmlns)
 : SBase(sbmlns)
 {
+    if (!hasValidLevelVersionNamespaceCombination())
+    throw SBMLConstructorException();
 }
 
 
@@ -132,7 +136,7 @@ ListOf::accept (SBMLVisitor& v) const
 /*
  * @return a (deep) copy of this ListOf items.
  */
-SBase*
+ListOf*
 ListOf::clone () const
 {
   return new ListOf(*this);
@@ -252,7 +256,7 @@ ListOf::getElementBySId(std::string id)
   for (unsigned int i = 0; i < size(); i++)
   {
     SBase* obj = get(i);
-    if (obj->getId() == id)
+    if (obj->isSetId() && obj->getId() == id)
     {
       return obj;
     }
