@@ -1,7 +1,6 @@
 /**
  * @file    Rule.cpp
- * @brief   Implementations of Rule, ListOfRules, AlgebraicRule, AssignmentRule
- *          and RateRule.
+ * @brief   Implementations of Rule and ListOfRules.
  * @author  Ben Bornstein
  *
  *
@@ -9,7 +8,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -45,6 +44,9 @@
 #include <sbml/SBMLDocument.h>
 #include <sbml/Model.h>
 #include <sbml/Rule.h>
+#include <sbml/AlgebraicRule.h>
+#include <sbml/AssignmentRule.h>
+#include <sbml/RateRule.h>
 
 /** @cond doxygen-ignored */
 
@@ -1037,7 +1039,7 @@ Rule::readL1Attributes (const XMLAttributes& attributes)
   //
   // formula: string  { use="required" }  (L1v1, L1v2)
   //
-  attributes.readInto("formula", mFormula, getErrorLog(), true);
+  attributes.readInto("formula", mFormula, getErrorLog(), true, getLine(), getColumn());
 
   //
   // type { use="optional" default="scalar" }  (L1v1, L1v2)
@@ -1052,7 +1054,7 @@ Rule::readL1Attributes (const XMLAttributes& attributes)
     // species: SName   { use="required" }  (L1v2)
     //
     const string s = (level == 1 && version == 1) ? "specie" : "species";
-    bool assigned = attributes.readInto(s, mVariable, getErrorLog(), true);
+    bool assigned = attributes.readInto(s, mVariable, getErrorLog(), true, getLine(), getColumn());
     if (assigned && mVariable.size() == 0)
     {
       logEmptyString(s, level, version, "<rule>");
@@ -1065,7 +1067,7 @@ Rule::readL1Attributes (const XMLAttributes& attributes)
     //
     // compartment: SName  { use="required" }  (L1v1, L1v2)
     //
-    bool assigned = attributes.readInto("compartment", mVariable, getErrorLog(), true);
+    bool assigned = attributes.readInto("compartment", mVariable, getErrorLog(), true, getLine(), getColumn());
     if (assigned && mVariable.size() == 0)
     {
       logEmptyString("compartment", level, version, "<rule>");
@@ -1078,7 +1080,7 @@ Rule::readL1Attributes (const XMLAttributes& attributes)
     //
     // name: SName  { use="required" } (L1v1, L1v2)
     //
-    bool assigned = attributes.readInto("name", mVariable, getErrorLog(), true);
+    bool assigned = attributes.readInto("name", mVariable, getErrorLog(), true, getLine(), getColumn());
     if (assigned && mVariable.size() == 0)
     {
       logEmptyString("name", level, version, "<rule>");
@@ -1089,7 +1091,7 @@ Rule::readL1Attributes (const XMLAttributes& attributes)
     //
     // units  { use="optional" }  (L1v1, L1v2);
     //
-    attributes.readInto("units", mUnits);
+    attributes.readInto("units", mUnits, getErrorLog(), false, getLine(), getColumn());
 
   }
 }
@@ -1113,7 +1115,7 @@ Rule::readL2Attributes (const XMLAttributes& attributes)
     //
     // variable: SId  { use="required" }  (L2v1 ->)
     //
-    bool assigned = attributes.readInto("variable", mVariable, getErrorLog(), true);
+    bool assigned = attributes.readInto("variable", mVariable, getErrorLog(), true, getLine(), getColumn());
     if (assigned && mVariable.size() == 0)
     {
       logEmptyString("variable", level, version, "<rule>");
@@ -1149,7 +1151,7 @@ Rule::readL3Attributes (const XMLAttributes& attributes)
     //
     // variable: SId  { use="required" }  (L2v1 ->)
     //
-    bool assigned = attributes.readInto("variable", mVariable, getErrorLog());
+    bool assigned = attributes.readInto("variable", mVariable, getErrorLog(), false, getLine(), getColumn());
     if (!assigned)
     {
       if (isAssignment())
@@ -1282,246 +1284,6 @@ Rule::setL1TypeCode (int type)
 }
 
 
-
-AlgebraicRule::AlgebraicRule (unsigned int level, unsigned int version) :
-  Rule(SBML_ALGEBRAIC_RULE, level, version)
-{
-  if (!hasValidLevelVersionNamespaceCombination())
-    throw SBMLConstructorException();
-
-  mInternalIdOnly = false;
-}
-
-
-AlgebraicRule::AlgebraicRule (SBMLNamespaces * sbmlns) :
-  Rule(SBML_ALGEBRAIC_RULE, sbmlns)
-{
-  if (!hasValidLevelVersionNamespaceCombination())
-  {
-    throw SBMLConstructorException(getElementName(), sbmlns);
-  }
-
-  mInternalIdOnly = false;
-
-  loadPlugins(sbmlns);
-}
-
-
-/*
- * Destroys this AlgebraicRule.
- */
-AlgebraicRule::~AlgebraicRule ()
-{
-}
-
-/*
- * @return a (deep) copy of this Rule.
- */
-AlgebraicRule*
-AlgebraicRule::clone () const
-{
-  return new AlgebraicRule(*this);
-}
-
-
-/*
- * Accepts the given SBMLVisitor.
- *
- * @return the result of calling <code>v.visit()</code>, which indicates
- * whether or not the Visitor would like to visit the Model's next Rule
- * (if available).
- */
-bool
-AlgebraicRule::accept (SBMLVisitor& v) const
-{
-  return v.visit(*this);
-}
-
-bool 
-AlgebraicRule::hasRequiredAttributes() const
-{
-  bool allPresent = Rule::hasRequiredAttributes();
-
-  return allPresent;
-}
-
-
-
-/** @cond doxygen-libsbml-internal */
-
-/*
- * sets the mInternalIdOnly flag
- */
-void 
-AlgebraicRule::setInternalIdOnly()
-{
-  mInternalIdOnly = true;
-}
-
-/*
- * gets the mInternalIdOnly flag
- */
-bool 
-AlgebraicRule::getInternalIdOnly() const
-{
-  return mInternalIdOnly;
-}
-
-  /** @endcond */
-
-
-
-AssignmentRule::AssignmentRule (unsigned int level, unsigned int version) :
-  Rule(SBML_ASSIGNMENT_RULE, level, version)
-{
-  if (!hasValidLevelVersionNamespaceCombination())
-    throw SBMLConstructorException();
-}
-
-AssignmentRule::AssignmentRule (SBMLNamespaces *sbmlns) :
-  Rule(SBML_ASSIGNMENT_RULE, sbmlns)
-{
-  if (!hasValidLevelVersionNamespaceCombination())
-  {
-    throw SBMLConstructorException(getElementName(), sbmlns);
-  }
-
-  loadPlugins(sbmlns);
-}
-
-
-/*
- * Destroys this AssignmentRule.
- */
-AssignmentRule::~AssignmentRule ()
-{
-}
-
-
-/*
- * @return a (deep) copy of this Rule.
- */
-AssignmentRule*
-AssignmentRule::clone () const
-{
-  return new AssignmentRule(*this);
-}
-
-
-/*
- * Accepts the given SBMLVisitor.
- *
- * @return the result of calling <code>v.visit()</code>, which indicates
- * whether or not the Visitor would like to visit the Model's next Rule
- * (if available).
- */
-bool
-AssignmentRule::accept (SBMLVisitor& v) const
-{
-  return v.visit(*this);
-}
-
-
-bool 
-AssignmentRule::hasRequiredAttributes() const
-{
-  bool allPresent = Rule::hasRequiredAttributes();
-
-  /* required attributes for assignment: variable (comp/species/name in L1) */
-
-  if (!isSetVariable())
-    allPresent = false;
-
-  return allPresent;
-}
-
-
-void
-AssignmentRule::renameSIdRefs(std::string oldid, std::string newid)
-{
-  Rule::renameSIdRefs(oldid, newid);
-  if (isSetVariable()) {
-    if (getVariable()==oldid) {
-      setVariable(newid);
-    }
-  }
-}
-
-RateRule::RateRule (unsigned int level, unsigned int version) :
-  Rule(SBML_RATE_RULE, level, version)
-{
-  if (!hasValidLevelVersionNamespaceCombination())
-    throw SBMLConstructorException();
-}
-
-RateRule::RateRule (SBMLNamespaces *sbmlns) :
-  Rule(SBML_RATE_RULE, sbmlns)
-{
-  if (!hasValidLevelVersionNamespaceCombination())
-  {
-    throw SBMLConstructorException(getElementName(), sbmlns);
-  }
-
-  loadPlugins(sbmlns);
-}
-
-
-/*
- * Destroys this RateRule.
- */
-RateRule::~RateRule ()
-{
-}
-
-/*
- * @return a (deep) copy of this Rule.
- */
-RateRule*
-RateRule::clone () const
-{
-  return new RateRule(*this);
-}
-
-
-bool 
-RateRule::hasRequiredAttributes() const
-{
-  bool allPresent = Rule::hasRequiredAttributes();
-
-  /* required attributes for rateRule: variable (comp/species/name in L1) */
-
-  if (!isSetVariable())
-    allPresent = false;
-
-  return allPresent;
-}
-
-
-/*
- * Accepts the given SBMLVisitor.
- *
- * @return the result of calling <code>v.visit()</code>, which indicates
- * whether or not the Visitor would like to visit the Model's next Rule
- * (if available).
- */
-bool
-RateRule::accept (SBMLVisitor& v) const
-{
-  return v.visit(*this);
-}
-
-
-void
-RateRule::renameSIdRefs(std::string oldid, std::string newid)
-{
-  Rule::renameSIdRefs(oldid, newid);
-  if (isSetVariable()) {
-    if (getVariable()==oldid) {
-      setVariable(newid);
-    }
-  }
-}
-
 /*
  * Creates a new ListOfRules items.
  */
@@ -1537,6 +1299,7 @@ ListOfRules::ListOfRules (unsigned int level, unsigned int version)
 ListOfRules::ListOfRules (SBMLNamespaces* sbmlns)
   : ListOf(sbmlns)
 {
+  loadPlugins(sbmlns);
 }
 
 
@@ -1634,6 +1397,20 @@ ListOfRules::remove (unsigned int n)
    return static_cast<Rule*>(ListOf::remove(n));
 }
 
+SBase* 
+ListOfRules::getElementBySId(std::string id)
+{
+  for (unsigned int i = 0; i < size(); i++)
+  {
+    SBase* obj = get(i);
+    //No rule is in the SId namespace, so don't check 'getId'.  However, their children (through plugins) may have the element we are looking for, so we still need to check all of them.
+    obj = obj->getElementBySId(id);
+    if (obj != NULL) return obj;
+  }
+
+  return getElementFromPluginsBySId(id);
+}
+  
 
 /* Removes item in this list by id */
 Rule*
