@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -31,7 +31,14 @@
 #include <sbml/Model.h>
 #include <sbml/KineticLaw.h>
 #include <sbml/Compartment.h>
+
 #include <sbml/SpeciesReference.h>
+#include <sbml/SimpleSpeciesReference.h>
+#include <sbml/ModifierSpeciesReference.h>
+
+#include <sbml/AlgebraicRule.h>
+#include <sbml/AssignmentRule.h>
+#include <sbml/RateRule.h>
 
 #include <sbml/validator/constraints/IdList.h>
 
@@ -40,6 +47,20 @@ LIBSBML_CPP_NAMESPACE_BEGIN
 
 static const char* ASSIGNED_COMPARTMENT = "AssignedName";
 
+/* functions used in adjusting L3 -> L2 stoichiometryMath */
+
+void createNoValueStoichMath(Model & m, SpeciesReference & sr, 
+                             unsigned int idCount);
+void createParameterAsRateRule(Model &m, SpeciesReference &sr, Rule &rr, 
+                          unsigned int idCount);
+
+void useStoichMath(Model & m, SpeciesReference &sr, bool isRule);
+
+/* functions used in adjusting L1 -> L2 stoichiometryMath */
+
+void dealWithL1Stoichiometry(Model & m, bool l2 = true);
+
+void dealWithAssigningL1Stoichiometry(Model & m, bool l2 = true);
 
 /*
  * Converts the model to a from SBML L1 to L2.  Most of the necessary
@@ -53,6 +74,8 @@ Model::convertL1ToL2 ()
   addModifiers();
 
   addConstantAttribute();
+
+  dealWithL1Stoichiometry(*this);
 }
 
 /* convert from L1 to L3 */
@@ -68,6 +91,8 @@ Model::convertL1ToL3 ()
   addDefinitionsForDefaultUnits();
 
   assignRequiredValues();
+
+  dealWithL1Stoichiometry(*this, false);
 }
 
 
@@ -105,6 +130,7 @@ Model::convertL2ToL1 (bool strict)
 
   }
 
+  dealWithAssigningL1Stoichiometry(*this, true);
   /* make sure underlying model is correct */
   if (strict)
   {
@@ -118,7 +144,33 @@ Model::convertL2ToL1 (bool strict)
 void 
 Model::convertL3ToL1 ()
 {
+  //
+  // Level 3 allows a model to be specified without a Compartment.  However
+  // this is not valid in Level 1.  Thus if a L3 model has no Compartment
+  // one must be included 
+  //
+  if (getNumCompartments() == 0)
+  {
+    createCompartment()->setId(ASSIGNED_COMPARTMENT);
+
+  }
   dealWithModelUnits();
+  
+  dealWithAssigningL1Stoichiometry(*this, false);
+  for (unsigned int i = 0; i < getNumReactions(); i++)
+  {
+    Reaction *r = getReaction(i);
+    if (r->isSetKineticLaw())
+    {
+      KineticLaw *kl = r->getKineticLaw();
+      for (unsigned int j = 0; j < kl->getNumLocalParameters(); j++)
+      {
+        Parameter *lp = new Parameter(getLevel(), getVersion());
+        (*lp) = *(kl->getLocalParameter(j));
+        kl->addParameter(lp);
+      }
+    }
+  }
 }
 
 
@@ -131,6 +183,21 @@ Model::convertL3ToL2 (bool strict)
   dealWithStoichiometry();
 
   dealWithEvents(strict);
+
+  for (unsigned int i = 0; i < getNumReactions(); i++)
+  {
+    Reaction *r = getReaction(i);
+    if (r->isSetKineticLaw())
+    {
+      KineticLaw *kl = r->getKineticLaw();
+      for (unsigned int j = 0; j < kl->getNumLocalParameters(); j++)
+      {
+        Parameter *lp = new Parameter(getLevel(), getVersion());
+        (*lp) = *(kl->getLocalParameter(j));
+        kl->addParameter(lp);
+      }
+    }
+  }
 }
 
 
@@ -492,7 +559,7 @@ Model::removeMetaId()
     {
       getReaction(n)->getReactant(i)->unsetMetaId();
     }
-    for (i = 0; i < getReaction(n)->getNumReactants(); i++)
+    for (i = 0; i < getReaction(n)->getNumProducts(); i++)
     {
       getReaction(n)->getProduct(i)->unsetMetaId();
     }
@@ -1114,12 +1181,11 @@ Model::dealWithModelUnits()
     }
     else
     {
-      Unit *u = new Unit(getSBMLNamespaces());
-      u->initDefaults();
-      u->setKind(UnitKind_forName(time.c_str()));
       ud = new UnitDefinition(getSBMLNamespaces());
       ud->setId("time");
-      ud->addUnit(u);
+      Unit *u = ud->createUnit();
+      u->initDefaults();
+      u->setKind(UnitKind_forName(time.c_str()));
     }
     addUnitDefinition(ud);
   }
@@ -1129,34 +1195,20 @@ void
 Model::dealWithStoichiometry()
 {
   unsigned int idCount = 0;
-  char newid[15];
-  std::string id;
   for (unsigned int i = 0; i < getNumReactions(); i++)
   {
     Reaction *r = getReaction(i);
     unsigned int j;
+
     for (j = 0; j < r->getNumReactants(); j++)
     {
       SpeciesReference *sr = r->getReactant(j);
-      if (!(sr->isSetStoichiometry()))
+      if (sr->isSetStoichiometry() == false)
       {
-        if (!(sr->isSetId()))
+        if (sr->isSetId() == false)
         {
-          // no stoichiometry and no id to set the stoichiometry
-          // replace with stoichiometryMath using a parameter with no value
-          sprintf(newid, "parameterId_%u", idCount);
-          id.assign(newid);
+          createNoValueStoichMath(*this, *sr, idCount);
           idCount++;
-          Parameter *p = createParameter();
-          p->setId(id);
-          p->setConstant(false);
-
-          StoichiometryMath *sm = sr->createStoichiometryMath();
-          if (sm != NULL)
-          {
-            ASTNode *ast = SBML_parseFormula(id.c_str());
-            sm->setMath(ast);
-          }
         }
         else
         {
@@ -1165,51 +1217,25 @@ Model::dealWithStoichiometry()
           // not used
           if (getInitialAssignment(sr->getId()) != NULL)
           {
-            // use stoichiometryMath instead
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
-            {
-              sm->setMath(getInitialAssignment(sr->getId())->getMath());
-              removeInitialAssignment(sr->getId());
-            }
+            useStoichMath(*this, *sr, false);
           }
           else if (getRule(sr->getId()) != NULL)
           {
-            // use stoichiometryMath instead
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
+            //assignmentRule
+            if (getRule(sr->getId())->getTypeCode() == SBML_ASSIGNMENT_RULE)
+            {  
+              useStoichMath(*this, *sr, true);
+            }
+            else if (getRule(sr->getId())->getTypeCode() == SBML_RATE_RULE)
             {
-              sm->setMath(getRule(sr->getId())->getMath());
-              removeRule(sr->getId());
+              createParameterAsRateRule(*this, *sr, *(getRule(sr->getId())), idCount);
+              idCount++;
             }
           }
-          //else if (getRateRule(sr->getId()) != NULL)
-          //{
-          //  // use stoichiometryMath instead
-          //  StoichiometryMath *sm = sr->createStoichiometryMath();
-          //  if (sm != NULL)
-          //  {
-          //    sm->setMath(getRateRule(sr->getId())->getMath());
-          //    removeRateRule(sr->getId());
-          //  }
-          //}
           else
           {
-            // is set but not used 
-            // use StoichiometryMath with parameter with no value
-            sprintf(newid, "parameterId_%u", idCount);
-            id.assign(newid);
+            createNoValueStoichMath(*this, *sr, idCount);
             idCount++;
-            Parameter *p = createParameter();
-            p->setId(id);
-            p->setConstant(false);
-
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
-            {
-              ASTNode *ast = SBML_parseFormula(id.c_str());
-              sm->setMath(ast);
-            }
           }
         }
       }
@@ -1223,34 +1249,21 @@ Model::dealWithStoichiometry()
           // not used
           if (getInitialAssignment(sr->getId()) != NULL)
           {
-            // use stoichiometryMath instead
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
-            {
-              sm->setMath(getInitialAssignment(sr->getId())->getMath());
-              removeInitialAssignment(sr->getId());
-            }
+            useStoichMath(*this, *sr, false);
           }
           else if (getRule(sr->getId()) != NULL)
           {
-            // use stoichiometryMath instead
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
+            //assignmentRule
+            if (getRule(sr->getId())->getTypeCode() == SBML_ASSIGNMENT_RULE)
+            {            
+              useStoichMath(*this, *sr, true);
+            }
+            else if (getRule(sr->getId())->getTypeCode() == SBML_RATE_RULE)
             {
-              sm->setMath(getRule(sr->getId())->getMath());
-              removeRule(sr->getId());
+              createParameterAsRateRule(*this, *sr, *(getRule(sr->getId())), idCount);
+              idCount++;
             }
           }
-          //else if (getRateRule(sr->getId()) != NULL)
-          //{
-          //  // use stoichiometryMath instead
-          //  StoichiometryMath *sm = sr->createStoichiometryMath();
-          //  if (sm != NULL)
-          //  {
-          //    sm->setMath(getRateRule(sr->getId())->getMath());
-          //    removeRateRule(sr->getId());
-          //  }
-          //}
         }
         // no id set - do not need to do anything
       }
@@ -1258,25 +1271,12 @@ Model::dealWithStoichiometry()
     for (j = 0; j < r->getNumProducts(); j++)
     {
       SpeciesReference *sr = r->getProduct(j);
-      if (!(sr->isSetStoichiometry()))
+      if (sr->isSetStoichiometry() == false)
       {
-        if (!(sr->isSetId()))
+        if (sr->isSetId() == false)
         {
-          // no stoichiometry and no id to set the stoichiometry
-          // replace with stoichiometryMath using a parameter with no value
-          sprintf(newid, "parameterId_%u", idCount);
-          id.assign(newid);
+          createNoValueStoichMath(*this, *sr, idCount);
           idCount++;
-          Parameter *p = createParameter();
-          p->setId(id);
-          p->setConstant(false);
-
-          StoichiometryMath *sm = sr->createStoichiometryMath();
-          if (sm != NULL)
-          {
-            ASTNode *ast = SBML_parseFormula(id.c_str());
-            sm->setMath(ast);
-          }
         }
         else
         {
@@ -1285,51 +1285,25 @@ Model::dealWithStoichiometry()
           // not used
           if (getInitialAssignment(sr->getId()) != NULL)
           {
-            // use stoichiometryMath instead
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
-            {
-              sm->setMath(getInitialAssignment(sr->getId())->getMath());
-              removeInitialAssignment(sr->getId());
-            }
+            useStoichMath(*this, *sr, false);
           }
           else if (getRule(sr->getId()) != NULL)
           {
-            // use stoichiometryMath instead
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
+            //assignmentRule
+            if (getRule(sr->getId())->getTypeCode() == SBML_ASSIGNMENT_RULE)
+            {  
+              useStoichMath(*this, *sr, true);
+            }
+            else if (getRule(sr->getId())->getTypeCode() == SBML_RATE_RULE)
             {
-              sm->setMath(getRule(sr->getId())->getMath());
-              removeRule(sr->getId());
+              createParameterAsRateRule(*this, *sr, *(getRule(sr->getId())), idCount);
+              idCount++;
             }
           }
-          //else if (getRateRule(sr->getId()) != NULL)
-          //{
-          //  // use stoichiometryMath instead
-          //  StoichiometryMath *sm = sr->createStoichiometryMath();
-          //  if (sm != NULL)
-          //  {
-          //    sm->setMath(getRateRule(sr->getId())->getMath());
-          //    removeRateRule(sr->getId());
-          //  }
-          //}
           else
           {
-            // is set but not used 
-            // use StoichiometryMath with parameter with no value
-            sprintf(newid, "parameterId_%u", idCount);
-            id.assign(newid);
+            createNoValueStoichMath(*this, *sr, idCount);
             idCount++;
-            Parameter *p = createParameter();
-            p->setId(id);
-            p->setConstant(false);
-
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
-            {
-              ASTNode *ast = SBML_parseFormula(id.c_str());
-              sm->setMath(ast);
-            }
           }
         }
       }
@@ -1343,40 +1317,239 @@ Model::dealWithStoichiometry()
           // not used
           if (getInitialAssignment(sr->getId()) != NULL)
           {
-            // use stoichiometryMath instead
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
-            {
-              sm->setMath(getInitialAssignment(sr->getId())->getMath());
-              removeInitialAssignment(sr->getId());
-            }
+            useStoichMath(*this, *sr, false);
           }
           else if (getRule(sr->getId()) != NULL)
           {
-            // use stoichiometryMath instead
-            StoichiometryMath *sm = sr->createStoichiometryMath();
-            if (sm != NULL)
+            //assignmentRule
+            if (getRule(sr->getId())->getTypeCode() == SBML_ASSIGNMENT_RULE)
+            {            
+              useStoichMath(*this, *sr, true);
+            }
+            else if (getRule(sr->getId())->getTypeCode() == SBML_RATE_RULE)
             {
-              sm->setMath(getRule(sr->getId())->getMath());
-              removeRule(sr->getId());
+              createParameterAsRateRule(*this, *sr, *(getRule(sr->getId())), idCount);
+              idCount++;
             }
           }
-          //else if (getRateRule(sr->getId()) != NULL)
-          //{
-          //  // use stoichiometryMath instead
-          //  StoichiometryMath *sm = sr->createStoichiometryMath();
-          //  if (sm != NULL)
-          //  {
-          //    sm->setMath(getRateRule(sr->getId())->getMath());
-          //    removeRateRule(sr->getId());
-          //  }
-          //}
         }
         // no id set - do not need to do anything
       }
     }
   }
 }
+
+void
+createNoValueStoichMath(Model & m, SpeciesReference & sr, unsigned int idCount)
+{
+  char newid[15];
+  std::string id;
+
+  // no stoichiometry and no id to set the stoichiometry
+  // replace with stoichiometryMath using a parameter with no value
+
+  sprintf(newid, "parameterId_%u", idCount);
+  id.assign(newid);
+
+  Parameter *p = m.createParameter();
+  p->setId(id);
+  p->setConstant(false);
+
+  StoichiometryMath *sm = sr.createStoichiometryMath();
+  if (sm != NULL)
+  {
+    ASTNode *ast = SBML_parseFormula(id.c_str());
+    sm->setMath(ast);
+  }
+}
+
+
+void
+createParameterAsRateRule(Model &m, SpeciesReference &sr, Rule &rr, 
+                          unsigned int idCount)
+{
+  char newid[15];
+  std::string id;
+
+  // create parameter as variable of rate rule 
+  // and use stoichiometryMath to point to this
+  sprintf(newid, "parameterId_%u", idCount);
+  id.assign(newid);
+
+  Parameter *p = m.createParameter();
+  p->setId(id);
+  p->setConstant(false);
+
+  rr.setVariable(id);
+  
+  StoichiometryMath *sm = sr.createStoichiometryMath();
+  if (sm != NULL)
+  {
+    ASTNode *ast = SBML_parseFormula(id.c_str());
+    sm->setMath(ast);
+  }
+}
+
+void
+useStoichMath(Model & m, SpeciesReference &sr, bool isRule)
+{
+  // use stoichiometryMath instead
+  StoichiometryMath *sm = sr.createStoichiometryMath();
+  if (sm != NULL)
+  {
+    if (isRule == true)
+    {
+      sm->setMath(m.getRule(sr.getId())->getMath());
+      m.removeRule(sr.getId());
+    }
+    else
+    {
+      sm->setMath(m.getInitialAssignment(sr.getId())->getMath());
+      m.removeInitialAssignment(sr.getId());
+    }
+  }
+}
+
+void dealWithL1Stoichiometry(Model & m, bool l2)
+{
+  unsigned int idCount = 0;
+  char newid[15];
+  std::string id;
+
+  for (unsigned int i = 0; i < m.getNumReactions(); i++)
+  {
+    Reaction *r = m.getReaction(i);
+    unsigned int j;
+
+    for (j = 0; j < r->getNumReactants(); j++)
+    {
+      SpeciesReference *sr = r->getReactant(j);
+      if (sr->getDenominator() != 1)
+      {
+        long stoich = static_cast<long>(sr->getStoichiometry());
+        int denom = sr->getDenominator();
+        ASTNode *node = new ASTNode();
+        node->setValue(stoich, denom);   
+        if (l2 == true)
+        {
+          StoichiometryMath * sm = sr->createStoichiometryMath();
+          sm->setMath(node);
+        }
+        else
+        {
+          sprintf(newid, "speciesRefId_%u", idCount);
+          id.assign(newid);
+          idCount++;
+          sr->setId(id);
+          InitialAssignment * ar = m.createInitialAssignment();
+          ar->setSymbol(id);
+          ar->setMath(node);
+          sr->unsetStoichiometry();
+        }
+      }
+    }
+    for (j = 0; j < r->getNumProducts(); j++)
+    {
+      SpeciesReference *sr = r->getProduct(j);
+      if (sr->getDenominator() != 1)
+      {
+        long stoich = static_cast<long>(sr->getStoichiometry());
+        int denom = sr->getDenominator();
+        ASTNode *node = new ASTNode();
+        node->setValue(stoich, denom);   
+        if (l2 == true)
+        {
+          StoichiometryMath * sm = sr->createStoichiometryMath();
+          sm->setMath(node);
+        }
+        else
+        {
+          sprintf(newid, "speciesRefId_%u", idCount);
+          id.assign(newid);
+          idCount++;
+          sr->setId(id);
+          InitialAssignment * ar = m.createInitialAssignment();
+          ar->setSymbol(id);
+          ar->setMath(node);
+          sr->unsetStoichiometry();
+        }
+      }
+    }
+  }
+}
+
+void dealWithAssigningL1Stoichiometry(Model & m, bool l2)
+{
+  unsigned int idCount = 0;
+  //char newid[15];
+  std::string id;
+
+  for (unsigned int i = 0; i < m.getNumReactions(); i++)
+  {
+    Reaction *r = m.getReaction(i);
+    unsigned int j;
+
+    for (j = 0; j < r->getNumReactants(); j++)
+    {
+      SpeciesReference *sr = r->getReactant(j);
+      // we do not get here unless the stoichiometryMath is an integer
+      // or a rational 
+      if (l2 == true && sr->isSetStoichiometryMath() == true)
+      {
+        const ASTNode* ast = sr->getStoichiometryMath()->getMath();
+        if (ast->isInteger())
+        {
+          int num = ast->getInteger();
+          sr->setStoichiometry(num);
+          sr->setDenominator(1);
+        }
+        else
+        {
+          int num = ast->getNumerator();
+          int denom = ast->getDenominator();
+          sr->setStoichiometry(num);
+          sr->setDenominator(denom);
+        }
+        sr->unsetStoichiometryMath();
+      }
+      else
+      {
+        sr->setStoichiometry(sr->getStoichiometry());
+        sr->setDenominator(1);
+      }
+    }
+    for (j = 0; j < r->getNumProducts(); j++)
+    {
+      SpeciesReference *sr = r->getProduct(j);
+      // we do not get here unless the stoichiometryMath is an integer
+      // or a rational 
+      if (l2 == true && sr->isSetStoichiometryMath() == true)
+      {
+        const ASTNode* ast = sr->getStoichiometryMath()->getMath();
+        if (ast->isInteger())
+        {
+          int num = ast->getInteger();
+          sr->setStoichiometry(num);
+          sr->setDenominator(1);
+        }
+        else
+        {
+          int num = ast->getNumerator();
+          int denom = ast->getDenominator();
+          sr->setStoichiometry(num);
+          sr->setDenominator(denom);
+        }
+        sr->unsetStoichiometryMath();
+      }
+      else
+      {
+        sr->setStoichiometry(sr->getStoichiometry());
+        sr->setDenominator(1);
+      }
+    }
+  }
+}
+
 /** @endcond **/
 
 LIBSBML_CPP_NAMESPACE_END
