@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -35,25 +35,7 @@
 #include <sbml/xml/XMLOutputStream.h>
 #include <sbml/xml/XMLError.h>
 
-
-#ifdef DONT_USE_VALIDATOR_API
-#include <sbml/validator/ConsistencyValidator.h>
-#include <sbml/validator/IdentifierConsistencyValidator.h>
-#include <sbml/validator/MathMLConsistencyValidator.h>
-#include <sbml/validator/SBOConsistencyValidator.h>
-#include <sbml/validator/UnitConsistencyValidator.h>
-#include <sbml/validator/OverdeterminedValidator.h>
-#include <sbml/validator/ModelingPracticeValidator.h>
-#include <sbml/validator/L1CompatibilityValidator.h>
-#include <sbml/validator/L2v1CompatibilityValidator.h>
-#include <sbml/validator/L2v2CompatibilityValidator.h>
-#include <sbml/validator/L2v3CompatibilityValidator.h>
-#include <sbml/validator/L2v4CompatibilityValidator.h>
-#include <sbml/validator/L3v1CompatibilityValidator.h>
-#include <sbml/validator/InternalConsistencyValidator.h>
-#else
 #include <sbml/validator/SBMLInternalValidator.h>
-#endif
 
 #include <sbml/Model.h>
 #include <sbml/SBMLErrorLog.h>
@@ -72,11 +54,6 @@
 #include <sbml/conversion/SBMLConverterRegistry.h>
 
 
-#ifdef USE_LAYOUT
-  #include <sbml/packages/layout/extension/LayoutExtension.h>
-  #include <sbml/packages/layout/extension/LayoutModelPlugin.h>
-#endif
-
 /** @cond doxygen-ignored */
 
 using namespace std;
@@ -152,16 +129,10 @@ SBMLDocument::SBMLDocument (unsigned int level, unsigned int version) :
  , mModel   ( NULL       )
 {
 
-#ifdef DONT_USE_VALIDATOR_API
-  mApplicableValidators = AllChecksON;
-  mApplicableValidatorsForConversion=AllChecksON;
-#else
   mInternalValidator = new SBMLInternalValidator();
   mInternalValidator->setDocument(this);
   mInternalValidator->setApplicableValidators(AllChecksON);
   mInternalValidator->setConversionValidators(AllChecksON);
-#endif
-  
 
   mSBML = this;
 
@@ -190,16 +161,10 @@ SBMLDocument::SBMLDocument (SBMLNamespaces* sbmlns) :
  , mModel   ( NULL       )
 {
 
-#ifdef DONT_USE_VALIDATOR_API
-  mApplicableValidators = AllChecksON;
-  mApplicableValidatorsForConversion=AllChecksON;
-#else
   mInternalValidator = new SBMLInternalValidator();
   mInternalValidator->setDocument(this);
   mInternalValidator->setApplicableValidators(AllChecksON);
   mInternalValidator->setConversionValidators(AllChecksON);
-#endif
-  
 
   mSBML = this;
   mLevel   = sbmlns->getLevel();
@@ -228,7 +193,6 @@ SBMLDocument::SBMLDocument (SBMLNamespaces* sbmlns) :
 
 /** @cond doxygen-libsbml-internal */
 
-#ifndef DONT_USE_VALIDATOR_API
 
   unsigned int SBMLDocument::getNumValidators() const
   {
@@ -257,8 +221,6 @@ SBMLDocument::SBMLDocument (SBMLNamespaces* sbmlns) :
     return NULL;
   }
 
-
-#endif
 /** @endcond */
 
 
@@ -267,10 +229,8 @@ SBMLDocument::SBMLDocument (SBMLNamespaces* sbmlns) :
  */
 SBMLDocument::~SBMLDocument ()
 {
-#ifndef DONT_USE_VALIDATOR_API
   if (mInternalValidator != NULL)
     delete mInternalValidator;
-#endif
   if (mModel != NULL)
   delete mModel;
 }
@@ -294,16 +254,10 @@ SBMLDocument::SBMLDocument (const SBMLDocument& orig) :
     mLevel                             = orig.mLevel;
     mVersion                           = orig.mVersion;
 
-#ifdef DONT_USE_VALIDATOR_API
-    mApplicableValidators = orig.getApplicableValidators();
-    mApplicableValidatorsForConversion=orig.getConversionValidators();
-#else
     mInternalValidator = new SBMLInternalValidator();
     mInternalValidator->setDocument(this);
     mInternalValidator->setApplicableValidators(orig.getApplicableValidators());
     mInternalValidator->setConversionValidators(orig.getConversionValidators());
-#endif
-  
 
     if (orig.mModel != NULL) 
     {
@@ -340,14 +294,8 @@ SBMLDocument& SBMLDocument::operator=(const SBMLDocument& rhs)
     mLevel                             = rhs.mLevel;
     mVersion                           = rhs.mVersion;
 
-#ifdef DONT_USE_VALIDATOR_API
-    mApplicableValidators = rhs.getApplicableValidators();
-    mApplicableValidatorsForConversion=rhs.getConversionValidators();
-#else
     mInternalValidator = (SBMLInternalValidator*)rhs.mInternalValidator->clone();
     mInternalValidator->setDocument(this);
-#endif
-  
 
     if (rhs.mModel != NULL) 
     {
@@ -453,13 +401,7 @@ SBMLDocument::getAllElements()
 unsigned char
 SBMLDocument::getApplicableValidators() const
 {
-#ifdef DONT_USE_VALIDATOR_API
-  return mApplicableValidators;
-#else
   return mInternalValidator->getApplicableValidators();
-#endif
-  
-
 }
 /** @endcond */
 
@@ -468,11 +410,7 @@ SBMLDocument::getApplicableValidators() const
 unsigned char
 SBMLDocument::getConversionValidators() const
 {
-#ifdef DONT_USE_VALIDATOR_API
-  return mApplicableValidatorsForConversion;
-#else
   return mInternalValidator->getConversionValidators();
-#endif
 }
 /** @endcond */
 
@@ -481,12 +419,7 @@ SBMLDocument::getConversionValidators() const
 void
 SBMLDocument::setApplicableValidators(unsigned char appl)
 {
-#ifdef DONT_USE_VALIDATOR_API
-  mApplicableValidators = appl;
-#else
   return mInternalValidator->setApplicableValidators(appl);
-#endif
-
 }
 /** @endcond */
 
@@ -495,11 +428,7 @@ SBMLDocument::setApplicableValidators(unsigned char appl)
 void
 SBMLDocument::setConversionValidators(unsigned char appl)
 {
-#ifdef DONT_USE_VALIDATOR_API
-  mApplicableValidatorsForConversion = appl;
-#else
   return mInternalValidator->setConversionValidators(appl);
-#endif
 }
 /** @endcond */
 
@@ -554,7 +483,7 @@ SBMLDocument::setLevelAndVersion (unsigned int level, unsigned int version,
 {
   ConversionProperties prop(new SBMLNamespaces(level, version));
   prop.addOption("strict", strict, "should validity be preserved");
-  prop.addOption("setLevelAndVersion", true, "this is checked by matchProperties");
+  prop.addOption("setLevelAndVersion", true, "convert the document to the given level and version");
 
   if (convert(prop) == LIBSBML_OPERATION_SUCCESS)
     return true;
@@ -575,10 +504,10 @@ SBMLDocument::updateSBMLNamespace(const std::string& package, unsigned int level
   std::string currentSBMLCorePrefix = mSBMLNamespaces->getNamespaces()->
     getPrefix(currentSBMLCoreURI);
 
-  bool sbmlDecl = false;
+  //bool sbmlDecl = false;
 
-  if (currentSBMLCorePrefix.empty() == false)
-    sbmlDecl = true;
+  //if (currentSBMLCorePrefix.empty() == false)
+  //  sbmlDecl = true;
 
   mLevel   = level;
   mVersion = version;
@@ -624,11 +553,24 @@ SBMLDocument::updateSBMLNamespace(const std::string& package, unsigned int level
   }
 
 
-  mSBMLNamespaces->getNamespaces()->add(uri);
-  if (sbmlDecl)
+  mSBMLNamespaces->getNamespaces()->remove(currentSBMLCorePrefix);
+  mSBMLNamespaces->getNamespaces()->add(uri, currentSBMLCorePrefix);
+  // it is possible that the ns exists unprefixed as well as prefixed
+  // the code will return the first it encounters
+  // so check if the original ns is still there
+  if (mSBMLNamespaces->getNamespaces()->containsUri(currentSBMLCoreURI) == true)
   {
+    currentSBMLCorePrefix = mSBMLNamespaces->getNamespaces()
+      ->getPrefix(currentSBMLCoreURI);
+    mSBMLNamespaces->getNamespaces()->remove(currentSBMLCorePrefix);
     mSBMLNamespaces->getNamespaces()->add(uri, currentSBMLCorePrefix);
   }
+
+
+  //if (sbmlDecl)
+  //{
+  //  mSBMLNamespaces->getNamespaces()->add(uri, currentSBMLCorePrefix);
+  //}
   mSBMLNamespaces->setLevel(mLevel);
   mSBMLNamespaces->setVersion(mVersion);
   setElementNamespace(uri); // this needs to propagate
@@ -708,100 +650,7 @@ void
 SBMLDocument::setConsistencyChecks(SBMLErrorCategory_t category,
                                    bool apply)
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  switch (category)
-  {
-  case LIBSBML_CAT_IDENTIFIER_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidators |= IdCheckON;
-    }
-    else
-    {
-      mApplicableValidators &= IdCheckOFF;
-    }
-
-    break;
-
-  case LIBSBML_CAT_GENERAL_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidators |= SBMLCheckON;
-    }
-    else
-    {
-      mApplicableValidators &= SBMLCheckOFF;
-    }
-
-    break;
-  
-  case LIBSBML_CAT_SBO_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidators |= SBOCheckON;
-    }
-    else
-    {
-      mApplicableValidators &= SBOCheckOFF;
-    }
-
-    break;
-  
-  case LIBSBML_CAT_MATHML_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidators |= MathCheckON;
-    }
-    else
-    {
-      mApplicableValidators &= MathCheckOFF;
-    }
-
-    break;
-  
-  case LIBSBML_CAT_UNITS_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidators |= UnitsCheckON;
-    }
-    else
-    {
-      mApplicableValidators &= UnitsCheckOFF;
-    }
-
-    break;
-  
-  case LIBSBML_CAT_OVERDETERMINED_MODEL:
-    if (apply)
-    {
-      mApplicableValidators |= OverdeterCheckON;
-    }
-    else
-    {
-      mApplicableValidators &= OverdeterCheckOFF;
-    }
-
-    break;
-
-  case LIBSBML_CAT_MODELING_PRACTICE:
-    if (apply)
-    {
-      mApplicableValidators |= PracticeCheckON;
-    }
-    else
-    {
-      mApplicableValidators &= PracticeCheckOFF;
-    }
-
-    break;
-
-  default:
-    // If it's a category for which we don't have validators, ignore it.
-    break;
-  }
-#else
   return mInternalValidator->setConsistencyChecks(category, apply);
-#endif
 }
 
 
@@ -809,100 +658,7 @@ void
 SBMLDocument::setConsistencyChecksForConversion(SBMLErrorCategory_t category,
                                    bool apply)
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  switch (category)
-  {
-  case LIBSBML_CAT_IDENTIFIER_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidatorsForConversion |= IdCheckON;
-    }
-    else
-    {
-      mApplicableValidatorsForConversion &= IdCheckOFF;
-    }
-
-    break;
-
-  case LIBSBML_CAT_GENERAL_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidatorsForConversion |= SBMLCheckON;
-    }
-    else
-    {
-      mApplicableValidatorsForConversion &= SBMLCheckOFF;
-    }
-
-    break;
-  
-  case LIBSBML_CAT_SBO_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidatorsForConversion |= SBOCheckON;
-    }
-    else
-    {
-      mApplicableValidatorsForConversion &= SBOCheckOFF;
-    }
-
-    break;
-  
-  case LIBSBML_CAT_MATHML_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidatorsForConversion |= MathCheckON;
-    }
-    else
-    {
-      mApplicableValidatorsForConversion &= MathCheckOFF;
-    }
-
-    break;
-  
-  case LIBSBML_CAT_UNITS_CONSISTENCY:
-    if (apply)
-    {
-      mApplicableValidatorsForConversion |= UnitsCheckON;
-    }
-    else
-    {
-      mApplicableValidatorsForConversion &= UnitsCheckOFF;
-    }
-
-    break;
-  
-  case LIBSBML_CAT_OVERDETERMINED_MODEL:
-    if (apply)
-    {
-      mApplicableValidatorsForConversion |= OverdeterCheckON;
-    }
-    else
-    {
-      mApplicableValidatorsForConversion &= OverdeterCheckOFF;
-    }
-
-    break;
-
-  case LIBSBML_CAT_MODELING_PRACTICE:
-    if (apply)
-    {
-      mApplicableValidatorsForConversion |= PracticeCheckON;
-    }
-    else
-    {
-      mApplicableValidatorsForConversion &= PracticeCheckOFF;
-    }
-
-    break;
-
-  default:
-    // If it's a category for which we don't have validators, ignore it.
-    break;
-  }
-#else
   return mInternalValidator->setConsistencyChecksForConversion(category, apply);
-#endif
 }
 
 
@@ -915,164 +671,42 @@ SBMLDocument::setConsistencyChecksForConversion(SBMLErrorCategory_t category,
 unsigned int
 SBMLDocument::checkConsistency ()
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  unsigned int nerrors = 0;
-  unsigned int total_errors = 0;
+  unsigned int numErrors = mInternalValidator->checkConsistency(false);
 
-  //if (getLevel() == 3)
-  //{
-  //  logError(L3NotSupported);
-  //  return 1;
-  //}
-  /* determine which validators to run */
-  bool id    = ((mApplicableValidators & 0x01) == 0x01);
-  bool sbml  = ((mApplicableValidators & 0x02) == 0x02);
-  bool sbo   = ((mApplicableValidators & 0x04) == 0x04);
-  bool math  = ((mApplicableValidators & 0x08) == 0x08);
-  bool units = ((mApplicableValidators & 0x10) == 0x10);
-  bool over  = ((mApplicableValidators & 0x20) == 0x20);
-  bool practice = ((mApplicableValidators & 0x40) == 0x40);
-
-  /* taken the state machine concept out for now
-  if (LibSBMLStateMachine::isActive()) 
-  {
-    units = LibSBMLStateMachine::getUnitState();
-  }
-  */
-
-  IdentifierConsistencyValidator id_validator;
-  ConsistencyValidator validator;
-  SBOConsistencyValidator sbo_validator;
-  MathMLConsistencyValidator math_validator;
-  UnitConsistencyValidator unit_validator;
-  OverdeterminedValidator over_validator;
-  ModelingPracticeValidator practice_validator;
-
-  /* calls each specified validator in turn 
-   * - stopping when errors are encountered */
-
-  if (id)
-  {
-    id_validator.init();
-    nerrors = id_validator.validate(*this);
-    total_errors += nerrors;
-    if (nerrors > 0) 
-    {
-      mErrorLog.add( id_validator.getFailures() );
-      return total_errors;
-    }
-  }
-
-  if (sbml)
-  {
-    validator.init();
-    nerrors = validator.validate(*this);
-    total_errors += nerrors;
-    if (nerrors > 0) 
-    {
-      mErrorLog.add( validator.getFailures() );
-      /* only want to bail if errors not warnings */
-      if (mErrorLog.getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
-        return total_errors;
-    }
-  }
-
-  if (sbo)
-  {
-    sbo_validator.init();
-    nerrors = sbo_validator.validate(*this);
-    total_errors += nerrors;
-    if (nerrors > 0) 
-    {
-      mErrorLog.add( sbo_validator.getFailures() );
-      /* only want to bail if errors not warnings */
-      if (mErrorLog.getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
-        return total_errors;
-    }
-  }
-
-  if (math)
-  {
-    math_validator.init();
-    nerrors = math_validator.validate(*this);
-    total_errors += nerrors;
-    if (nerrors > 0) 
-    {
-      mErrorLog.add( math_validator.getFailures() );
-      /* at this point bail if any problems
-       * unit checks may crash if there have been math errors/warnings
-       */
-      return total_errors;
-    }
-  }
-
-
-  if (units)
-  {
-    unit_validator.init();
-    nerrors = unit_validator.validate(*this);
-    total_errors += nerrors;
-    if (nerrors > 0) 
-    {
-      mErrorLog.add( unit_validator.getFailures() );
-      /* only want to bail if errors not warnings */
-      if (mErrorLog.getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
-        return total_errors;
-    }
-  }
-
-  /* do not even try if there have been unit warnings 
-   * changed this as would have bailed */
-  if (over)
-  {
-    over_validator.init();
-    nerrors = over_validator.validate(*this);
-    total_errors += nerrors;
-    if (nerrors > 0) 
-    {
-      mErrorLog.add( over_validator.getFailures() );
-      /* only want to bail if errors not warnings */
-      if (mErrorLog.getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
-        return total_errors;
-    }
-  }
-
-  if (practice)
+  list<SBMLValidator*>::iterator it;
+  for (it = mValidators.begin(); it != mValidators.end(); it++)
   {
-    practice_validator.init();
-    nerrors = practice_validator.validate(*this);
-    if (nerrors > 0) 
+    long newErrors = (*it)->validate(*this);
+    if (newErrors > 0)
     {
-      unsigned int errorsAdded = 0;
-      const std::list<SBMLError> practiceErrors = practice_validator.getFailures();
-      list<SBMLError>::const_iterator end = practiceErrors.end();
-      list<SBMLError>::const_iterator iter;
-      for (iter = practiceErrors.begin(); iter != end; ++iter)
-      {
-        if (SBMLError(*iter).getErrorId() != 80701)
-        {
-          mErrorLog.add( SBMLError(*iter) );
-          errorsAdded++;
-        }
-        else
-        {
-          if (units) 
-          {
-            mErrorLog.add( SBMLError(*iter) );
-            errorsAdded++;
-          }
-        }
-      }
-      total_errors += errorsAdded;
-
+      mErrorLog.add((*it)->getFailures());
+      numErrors += newErrors;
     }
   }
-
-
-  return total_errors;
-#else
 
-  unsigned int numErrors = mInternalValidator->checkConsistency();
+  return numErrors;
+}
+
+/**
+ * Performs consistency checking and validation on this SBML document.
+ *
+ * If this method returns a nonzero value (meaning, one or more
+ * consistency checks have failed for SBML document), the failures may be
+ * due to warnings @em or errors.  Callers should inspect the severity
+ * flag in the individual SBMLError objects returned by
+ * SBMLDocument::getError(@if java long n@endif) to determine the nature of the failures.
+ *
+ * @note unlike checkConsistency this method will write the document
+ *       in order to determine all errors for the document. This will 
+ *       also clear the error log. 
+ *
+ * @return the number of failed checks (errors) encountered.
+ *
+ * @see SBMLDocument::checkConsistency()
+ */
+unsigned int SBMLDocument::validateSBML ()
+{
+  unsigned int numErrors = mInternalValidator->checkConsistency(true);
 
   list<SBMLValidator*>::iterator it;
   for (it = mValidators.begin(); it != mValidators.end(); it++)
@@ -1086,7 +720,6 @@ SBMLDocument::checkConsistency ()
   }
 
   return numErrors;
-#endif
 }
 
 
@@ -1102,38 +735,7 @@ SBMLDocument::checkConsistency ()
 unsigned int
 SBMLDocument::checkInternalConsistency()
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  unsigned int nerrors = 0;
-  unsigned int totalerrors = 0;
-
-  InternalConsistencyValidator validator;
-
-  validator.init();
-  nerrors = validator.validate(*this);
-  if (nerrors > 0) 
-  {
-    mErrorLog.add( validator.getFailures() );
-  }
-  totalerrors += nerrors;
-  
-  /* hack to catch errors normally caught at read time */
-  char* doc = writeSBMLToString(this);
-  SBMLDocument *d = readSBMLFromString(doc);
-  util_free(doc);
-  nerrors = d->getNumErrors();
-
-  for (unsigned int i = 0; i < nerrors; i++)
-  {
-    mErrorLog.add(*(d->getError(i)));
-  }
-  delete d;
-  totalerrors += nerrors;
-
-
-  return totalerrors;
-#else
   return mInternalValidator->checkInternalConsistency();
-#endif
 }
 
 /*
@@ -1146,19 +748,7 @@ SBMLDocument::checkInternalConsistency()
 unsigned int
 SBMLDocument::checkL1Compatibility ()
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  if (mModel == NULL) return 0;
-
-  L1CompatibilityValidator validator;
-  validator.init();
-
-  unsigned int nerrors = validator.validate(*this);
-  if (nerrors > 0) mErrorLog.add( validator.getFailures() );
-
-  return nerrors;
-#else
   return mInternalValidator->checkL1Compatibility();
-#endif
 }
 
 
@@ -1172,19 +762,7 @@ SBMLDocument::checkL1Compatibility ()
 unsigned int
 SBMLDocument::checkL2v1Compatibility ()
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  if (mModel == NULL) return 0;
-
-  L2v1CompatibilityValidator validator;
-  validator.init();
-
-  unsigned int nerrors = validator.validate(*this);
-  if (nerrors > 0) mErrorLog.add( validator.getFailures() );
-
-  return nerrors;
-#else
   return mInternalValidator->checkL2v1Compatibility();
-#endif
 }
 
 
@@ -1198,19 +776,7 @@ SBMLDocument::checkL2v1Compatibility ()
 unsigned int
 SBMLDocument::checkL2v2Compatibility ()
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  if (mModel == NULL) return 0;
-
-  L2v2CompatibilityValidator validator;
-  validator.init();
-
-  unsigned int nerrors = validator.validate(*this);
-  if (nerrors > 0) mErrorLog.add( validator.getFailures() );
-
-  return nerrors;
-#else
   return mInternalValidator->checkL2v2Compatibility();
-#endif
 }
 
 
@@ -1224,19 +790,7 @@ SBMLDocument::checkL2v2Compatibility ()
 unsigned int
 SBMLDocument::checkL2v3Compatibility ()
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  if (mModel == NULL) return 0;
-
-  L2v3CompatibilityValidator validator;
-  validator.init();
-
-  unsigned int nerrors = validator.validate(*this);
-  if (nerrors > 0) mErrorLog.add( validator.getFailures() );
-
-  return nerrors;
-#else
   return mInternalValidator->checkL2v3Compatibility();
-#endif
 }
 
 
@@ -1250,19 +804,7 @@ SBMLDocument::checkL2v3Compatibility ()
 unsigned int
 SBMLDocument::checkL2v4Compatibility ()
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  if (mModel == NULL) return 0;
-
-  L2v4CompatibilityValidator validator;
-  validator.init();
-
-  unsigned int nerrors = validator.validate(*this);
-  if (nerrors > 0) mErrorLog.add( validator.getFailures() );
-
-  return nerrors;
-#else
   return mInternalValidator->checkL2v4Compatibility();
-#endif
 }
 
 
@@ -1276,19 +818,7 @@ SBMLDocument::checkL2v4Compatibility ()
 unsigned int
 SBMLDocument::checkL3v1Compatibility ()
 {
-#ifdef DONT_USE_VALIDATOR_API  
-  if (mModel == NULL) return 0;
-
-  L3v1CompatibilityValidator validator;
-  validator.init();
-
-  unsigned int nerrors = validator.validate(*this);
-  if (nerrors > 0) mErrorLog.add( validator.getFailures() );
-
-  return nerrors;
-#else
   return mInternalValidator->checkL3v1Compatibility();
-#endif
 }
 
 
@@ -1325,15 +855,7 @@ SBMLDocument::getNumErrors () const
 void
 SBMLDocument::printErrors (std::ostream& stream) const
 {
-  unsigned int numErrors = getNumErrors();
-
-  if (numErrors > 0)
-  {
-    for (unsigned int n = 0; n < numErrors; n++)
-    {
-      stream << *(getError(n));
-    }
-  }
+  getErrorLog()->printErrors(stream);
 }
 
 /** @cond doxygen-libsbml-internal */
@@ -1474,6 +996,16 @@ SBMLDocument::getErrorLog ()
   return &mErrorLog;
 }
 
+/*
+ * @return the SBMLErrorLog used to log errors during while reading and
+ * validating SBML.
+ */
+const SBMLErrorLog*
+SBMLDocument::getErrorLog () const
+{
+  return &mErrorLog;
+}
+
 
 int
 SBMLDocument::enableDefaultNS(const std::string& package, bool flag)
@@ -1589,7 +1121,7 @@ SBMLDocument::getPackageRequired(const std::string& package)
       //
       // (NOTE) objects in mPlugins must be derived
       //        from SBMLDocumentPlugin to invoke the corresponding
-      //        setReuired(), getRequired() functions
+      //        setRequired(), getRequired() functions
       //
       return static_cast<SBMLDocumentPlugin*>(mPlugins[i])->getRequired();
     }
@@ -1615,8 +1147,9 @@ SBMLDocument::isSetPackageRequired(const std::string& package)
 {
   for (size_t i=0; i < mPlugins.size(); i++)
   {
-    std::string uri = mPlugins[i]->getURI();
+    std::string uri = mPlugins[i]->getURI();    
     const SBMLExtension* sbext = SBMLExtensionRegistry::getInstance().getExtensionInternal(uri);
+
     if (   (uri == package)
         || (sbext && (sbext->getName() == package) )
        )
@@ -1700,14 +1233,14 @@ SBMLDocument::readAttributes (const XMLAttributes& attributes,
   // level: positiveInteger  { use="required" fixed="1" }  (L1v1)
   // level: positiveInteger  { use="required" fixed="2" }  (L2v1)
   //
-  bool levelRead = attributes.readInto("level", mLevel, getErrorLog(), false);
+  bool levelRead = attributes.readInto("level", mLevel, getErrorLog(), false, getLine(), getColumn());
 
   //
   // version: positiveInteger  { use="required" fixed="1" }  (L1v1, L2v1)
   // version: positiveInteger  { use="required" fixed="2" }  (L1v2, L2v2)
   // version: positiveInteger  { use="required" fixed="3" }  (L2v3)
   //
-  bool versionRead = attributes.readInto("version", mVersion, getErrorLog(), false);
+  bool versionRead = attributes.readInto("version", mVersion, getErrorLog(), false, getLine(), getColumn());
 
   // 
   // (EXTENSION)  SBMLDocument specific code
@@ -1947,19 +1480,7 @@ SBMLDocument::readAttributes (const XMLAttributes& attributes,
 
   }
 
-#ifdef USE_LAYOUT
-  /* if using layout and reading an L2 model the package
-   * is not enabled by the read since the l3 ns is missing
-   * force the package to be enabled
-   */
-  if (SBMLExtensionRegistry::getInstance().isRegistered("layout"))
-  {
-    if (mLevel == 2)
-    {
-      enablePackage(LayoutExtension::getXmlnsL2(),"layout", true);
-    }
-  }
-#endif
+  SBMLExtensionRegistry::getInstance().enableL2NamespaceForDocument(this);
 
 //   if (getLevel() > 2)
 //   {
@@ -1975,7 +1496,7 @@ SBMLDocument::readAttributes (const XMLAttributes& attributes,
 //       if (sbmlext && sbmlext->isEnabled())
 //       {
 //         XMLTriple triple("required", uri, prefix);
-//         attributes.readInto(triple, reqd, getErrorLog(), true);
+//         attributes.readInto(triple, reqd, getErrorLog(), true, getLine(), getColumn());
 //         mPkgRequiredMap.insert(pair<string,bool>(uri,reqd));
 //         cout << "[DEBUG] SBMLDocument::readAttributes" << uri << endl;
 //       }
@@ -2116,15 +1637,8 @@ SBMLDocument::writeXMLNS (XMLOutputStream& stream) const
   XMLNamespaces * xmlns = thisNs->clone();
   if (xmlns != NULL) 
   {
-#ifdef USE_LAYOUT
-    for (int n = 0; n < xmlns->getNumNamespaces(); n++)
-    {
-      if (xmlns->getURI(n) == LayoutExtension::getXmlnsL2())
-      {
-        xmlns->remove(n);
-      }
-    }
-#endif
+    SBMLExtensionRegistry::getInstance().removeL2Namespaces(xmlns);
+
     stream << *(xmlns);
     delete xmlns;
   }
@@ -2272,7 +1786,9 @@ LIBSBML_EXTERN
 void
 SBMLDocument_free (SBMLDocument_t *d)
 {
-  delete d;
+  if (d != NULL)
+    delete d;
+  d = NULL;
 }
 
 
