@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -163,12 +163,18 @@ public:
    * Copy constructor; creates a copy of a SBMLNamespaces.
    * 
    * @param orig the SBMLNamespaces instance to copy.
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p orig is @c NULL.
    */
   SBMLNamespaces(const SBMLNamespaces& orig);
 
 
   /**
    * Assignment operator for SBMLNamespaces.
+   *
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p rhs is @c NULL.
    */
   SBMLNamespaces& operator=(const SBMLNamespaces& rhs);
 
@@ -190,6 +196,13 @@ public:
    *
    * @return a string representing the SBML namespace that reflects the
    * SBML Level and Version specified.
+   *
+   * @if notclike @note Because this is a @em static method, the
+   * non-C++ language interfaces for libSBML will contain two variants.  One
+   * will be a static method on the class (i.e., SBMLNamespaces), and the
+   * other will be a standalone top-level function with the name
+   * SBMLNamespaces_getSBMLNamespaceURI(). They are functionally
+   * identical. @endif
    */
   static std::string getSBMLNamespaceURI(unsigned int level,
                                          unsigned int version);
@@ -199,9 +212,17 @@ public:
    * libsbml. 
    * 
    * @return a list with supported SBML namespaces. 
+   *
+   * @if notclike @note Because this is a @em static method, the
+   * non-C++ language interfaces for libSBML will contain two variants.  One
+   * will be a static method on the class (i.e., SBMLNamespaces), and the
+   * other will be a standalone top-level function with the name
+   * SBMLNamespaces_getSupportedNamespaces(). They are functionally
+   * identical. @endif
    */
   static const List* getSupportedNamespaces();
 
+
   /**
    * Returns a string representing the SBML XML namespace of this
    * object.
@@ -335,8 +356,10 @@ else
 @endverbatim
    * @endif@if python
 @verbatim
-sbmlDoc = SBMLDocument(3, 1)
-if sbmlDoc == None:
+sbmlDoc = None
+try:
+  sbmlDoc = SBMLDocument(3, 1)
+except ValueError:
   # Do something to handle exceptional situation.  Candidate
   # causes include invalid combinations of SBML Level and Version
   # (impossible if hardwired as given here), running out of memory, and
@@ -344,11 +367,11 @@ if sbmlDoc == None:
 
 namespaces = sbmlDoc.getNamespaces()
 if namespaces == None:
-  # Do something to handle exceptional situation.
+  # Do something to handle case of no namespaces.
 
 status = namespaces.add("http://www.w3.org/1999/xhtml", "html")
 if status != LIBSBML_OPERATION_SUCCESS:
-  # Do something to handle exceptional situation.
+  # Do something to handle failure.
 @endverbatim
    * @endif@if csharp
 @verbatim
@@ -559,6 +582,13 @@ else
    * @param uri the URI of namespace
    *
    * @return @c true if the "uri" is one of SBML namespaces, @c false otherwise.
+   *
+   * @if notclike @note Because this is a @em static method, the
+   * non-C++ language interfaces for libSBML will contain two variants.  One
+   * will be a static method on the class (i.e., SBMLNamespaces), and the
+   * other will be a standalone top-level function with the name
+   * SBMLNamespaces_isSBMLNamespace(). They are functionally
+   * identical. @endif
    */
   static bool isSBMLNamespace(const std::string& uri);
 
