@@ -1,7 +1,6 @@
 /**
  * @file    SpeciesReference.cpp
- * @brief   Implementation of SimpleSpeciesReference, SpeciesReference,
- *          ModifierSpeciesReference, and ListOfSpeciesReferences. 
+ * @brief   Implementation of SpeciesReference and ListOfSpeciesReferences. 
  * @author  Ben Bornstein
  *
  *
@@ -9,7 +8,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -33,6 +32,7 @@
 #include <sbml/xml/XMLInputStream.h>
 #include <sbml/xml/XMLOutputStream.h>
 
+#include <sbml/annotation/RDFAnnotation.h>
 #include <sbml/math/FormulaParser.h>
 #include <sbml/math/MathML.h>
 #include <sbml/math/ASTNode.h>
@@ -43,6 +43,8 @@
 #include <sbml/SBMLDocument.h>
 #include <sbml/Model.h>
 #include <sbml/SpeciesReference.h>
+#include <sbml/SimpleSpeciesReference.h>
+#include <sbml/ModifierSpeciesReference.h>
 #include <sbml/extension/SBasePlugin.h>
 
 /** @cond doxygen-ignored */
@@ -53,555 +55,6 @@ using namespace std;
 
 LIBSBML_CPP_NAMESPACE_BEGIN
 
-SimpleSpeciesReference::SimpleSpeciesReference (unsigned int level, 
-                                                unsigned int version) :
-   SBase ( level, version )
- , mId     ( "" )
- , mName   ( "" )
- , mSpecies( "" )
-{
-}
-
-
-/** @cond doxygen-libsbml-internal */
-SimpleSpeciesReference::SimpleSpeciesReference (SBMLNamespaces *sbmlns) :
-   SBase   (sbmlns  )
- , mId     ( "" )
- , mName   ( "" )
- , mSpecies( "" )
-{
-}
-/** @endcond */
-
-
-/*
- * Destroys this SimpleSpeciesReference.
- */
-SimpleSpeciesReference::~SimpleSpeciesReference ()
-{
-}
-
-
-/*
- * Copy constructor. Creates a copy of this SimpleSpeciesReference.
- */
-SimpleSpeciesReference::SimpleSpeciesReference(const SimpleSpeciesReference& orig) :
-   SBase     ( orig                    )
-{
-  if (&orig == NULL)
-  {
-    throw SBMLConstructorException("Null argument to copy constructor");
-  }
-  else
-  {
-    mId      = orig.mId;
-    mName    = orig.mName;
-    mSpecies = orig.mSpecies;
-  }
-}
-
-
-/*
- * Assignment operator.
- */
-SimpleSpeciesReference& SimpleSpeciesReference::operator=(const SimpleSpeciesReference& rhs)
-{
-  if (&rhs == NULL)
-  {
-    throw SBMLConstructorException("Null argument to assignment operator");
-  }
-  else if(&rhs!=this)
-  {
-    this->SBase::operator =(rhs);
-    mId = rhs.mId;
-    mName = rhs.mName;
-    mSpecies = rhs.mSpecies;
-  }
-
-  return *this;
-}
-
-
-/*
- * Accepts the given SBMLVisitor.
- *
- * @return the result of calling <code>v.visit()</code>, which indicates
- * whether or not the Visitor would like to visit the Reaction's next
- * SimpleSpeciesReference (if available).
- */
-bool
-SimpleSpeciesReference::accept (SBMLVisitor& v) const
-{
-  return v.visit(*this);
-}
-
-
-/*
- * @return the id of this SBML object.
- */
-const string&
-SimpleSpeciesReference::getId () const
-{
-  return mId;
-}
-
-
-/*
- * @return the name of this SBML object.
- */
-const string&
-SimpleSpeciesReference::getName () const
-{
-  return (getLevel() == 1) ? mId : mName;
-}
-
-
-/*
- * @return the species for this SimpleSpeciesReference.
- */
-const string&
-SimpleSpeciesReference::getSpecies () const
-{
-  return mSpecies;
-}
-
-
-/*
- * @return true if the id of this SBML object is set, false
- * otherwise.
- */
-bool
-SimpleSpeciesReference::isSetId () const
-{
-  return (mId.empty() == false);
-}
-
-
-/*
- * @return true if the name of this SBML object is set, false
- * otherwise.
- */
-bool
-SimpleSpeciesReference::isSetName () const
-{
-  return (getLevel() == 1) ? (mId.empty() == false) : 
-                            (mName.empty() == false);
-}
-
-
-/*
- * @return true if the species for this SimpleSpeciesReference is 
- * set, false otherwise.
- */
-bool
-SimpleSpeciesReference::isSetSpecies () const
-{
-  return (mSpecies.empty() == false);
-}
-
-
-/*
- * Sets the species of this SimpleSpeciesReference to a copy of sid.
- */
-int
-SimpleSpeciesReference::setSpecies (const std::string& sid)
-{
-  if (&(sid) == NULL)
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else if (!(SyntaxChecker::isValidInternalSId(sid)))
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else
-  {
-    mSpecies = sid;
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-}
-
-
-/*
- * Sets the id of this SBML object to a copy of sid.
- */
-int
-SimpleSpeciesReference::setId (const std::string& sid)
-{
-  if (&(sid) == NULL)
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else if (getLevel() == 1 ||
-    (getLevel() == 2 && getVersion() == 1))
-  {
-    //
-    // (NOTE)
-    //
-    // The code below is specific to Layout Extension for SBML Level 2 Version 1.
-    // The purpose of this code is not to break existing code based on optional 
-    // Layout Extension implemented in libSBML 4 or before.
-    //
-    // (Basically, this kind of code which is specific to some package extension 
-    //  should not be implemented in source files for SBML Core package. 
-    //  However, only layout extension has been implemented in previours libSBML and 
-    //  thus removing the code below can greately break existing code, which should 
-    //  be avoided.)
-    //
-    bool enabledLayoutL2 = false;
-    const std::string layoutL2URI = "http://projects.eml.org/bcb/sbml/level2";
-    std::vector<SBasePlugin*>::iterator mextIter = mPlugins.begin();
-    while (mextIter != mPlugins.end())
-    { 
-      if ( (*mextIter)->getURI() == layoutL2URI)
-      {
-#if 0
-          std::cout << "[DEBUG] SimpleSpeciesReference::setId() layoutL2 is enabled" << std::endl;
-#endif
-        enabledLayoutL2 = true;
-        break;
-      }
-      ++mextIter;
-    }
-                   
-    if (enabledLayoutL2)
-    {
-      mId = sid;
-      return LIBSBML_OPERATION_SUCCESS;
-    }
-    else
-    {
-      return LIBSBML_UNEXPECTED_ATTRIBUTE;
-    }
-  }
-  else if (!(SyntaxChecker::isValidInternalSId(sid)))
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else
-  {
-    mId = sid;
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-}
-
-
-/*
- * Sets the name of this SBML object to a copy of name.
- */
-int
-SimpleSpeciesReference::setName (const std::string& name)
-{
-  if (&(name) == NULL)
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else if (getLevel() == 1 ||
-    (getLevel() == 2 && getVersion() == 1))
-  {
-    return LIBSBML_UNEXPECTED_ATTRIBUTE;
-  }
-  else if (!(SyntaxChecker::isValidInternalSId(name)))
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else
-  {
-    if (getLevel() == 1) mId = name;
-    else mName = name;
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-}
-
-/*
- * Unsets the id of this SBML object.
- */
-int
-SimpleSpeciesReference::unsetId ()
-{
-  mId.erase();
-
-  if (mId.empty())
-  {
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-  else
-  {
-    return LIBSBML_OPERATION_FAILED;
-  }
-}
-
-
-/*
- * Unsets the name of this SBML object.
- */
-int
-SimpleSpeciesReference::unsetName ()
-{
-  if (getLevel() == 1) 
-  {
-    mId.erase();
-  }
-  else 
-  {
-    mName.erase();
-  }
-
-  if (getLevel() == 1 && mId.empty())
-  {
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-  else if (mName.empty())
-  {
-    return LIBSBML_OPERATION_SUCCESS;
-  }
-  else
-  {
-    return LIBSBML_OPERATION_FAILED;
-  }
-}
-
-
-/*
- * @return true if this SpeciesReference is a ModiferSpeciesReference,
- * false otherwise.
- */
-bool
-SimpleSpeciesReference::isModifier () const
-{
-  return (getTypeCode() == SBML_MODIFIER_SPECIES_REFERENCE);
-}
-
-
-void
-SimpleSpeciesReference::renameSIdRefs(std::string oldid, std::string newid)
-{
-  if (isSetSpecies()) {
-    if (mSpecies==oldid) setSpecies(newid);
-  }
-}
-
-/** @cond doxygen-libsbml-internal */
-bool 
-SimpleSpeciesReference::hasRequiredAttributes() const
-{
-  bool allPresent = true;
-
-  if (!isSetSpecies())
-    allPresent = false;
-
-  return allPresent;
-}
-
-
-/**
- * Subclasses should override this method to get the list of
- * expected attributes.
- * This function is invoked from corresponding readAttributes()
- * function.
- */
-void
-SimpleSpeciesReference::addExpectedAttributes(ExpectedAttributes& attributes)
-{
-  SBase::addExpectedAttributes(attributes);
-
-  const unsigned int level   = getLevel  ();
-  const unsigned int version = getVersion();
-
-  const string s = (level == 1 && version == 1) ? "specie" : "species";
-  attributes.add(s);
-
-  if (level > 1)
-  {
-    if (!(level == 2 && version == 1))
-    {
-      attributes.add("id");
-      attributes.add("name");
-    }
-
-    if (level == 2 && version == 2)
-      attributes.add("sboTerm");
-
-  }
-}
-
-/*
- * Subclasses should override this method to read values from the given
- * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
- */
-void
-SimpleSpeciesReference::readAttributes (const XMLAttributes& attributes,
-                                        const ExpectedAttributes& expectedAttributes)
-{
-  const unsigned int level   = getLevel  ();
-
-  SBase::readAttributes(attributes, expectedAttributes);
-
-  switch (level)
-  {
-  case 1:
-    readL1Attributes(attributes);
-    break;
-  case 2:
-    readL2Attributes(attributes);
-    break;
-  case 3:
-  default:
-    readL3Attributes(attributes);
-    break;
-  }
-}
-
-
-void
-SimpleSpeciesReference::readL1Attributes (const XMLAttributes& attributes)
-{
-  const unsigned int version = getVersion();
-
-  //
-  // specie : SName   { use="required" }  (L1v1)
-  // species: SName   { use="required" }  (L1v2, L2v1->)
-  //
-  const string s = (version == 1) ? "specie" : "species";
-  attributes.readInto(s , mSpecies, getErrorLog(), true);
-}
-
-
-void
-SimpleSpeciesReference::readL2Attributes (const XMLAttributes& attributes)
-{
-  const unsigned int level   = getLevel  ();
-  const unsigned int version = getVersion();
-  //
-  // species: SName   { use="required" }  (L1v2, L2v1->)
-  //
-  attributes.readInto("species" , mSpecies, getErrorLog(), true);
-  
-  if (version > 1)
-  {
-    //
-    // id: SId  { use="optional" }  (L2v2->)
-    //
-    bool assigned = attributes.readInto("id", mId, getErrorLog());
-    if (assigned && mId.size() == 0)
-    {
-      logEmptyString("id", level, version, "<speciesReference>");
-    }
-    if (!SyntaxChecker::isValidInternalSId(mId)) logError(InvalidIdSyntax);
-
-    //
-    // name: string  { use="optional" }  (L2v2->)
-    //
-    attributes.readInto("name" , mName);
-  }
-  if (version == 2)
-  {
-    //
-    // sboTerm: SBOTerm { use="optional" }  (L2v2->)
-    //
-    mSBOTerm = SBO::readTerm(attributes, this->getErrorLog(), level, version,
-				getLine(), getColumn());
-  }
-
-}
-
-
-void
-SimpleSpeciesReference::readL3Attributes (const XMLAttributes& attributes)
-{
-  const unsigned int level   = getLevel  ();
-  const unsigned int version = getVersion();
-  //
-  // species: SName   { use="required" }  (L1v2, L2v1->)
-  //
-  bool assigned = attributes.readInto("species" , mSpecies);
-  if (!assigned)
-  {
-    if (isModifier())
-      logError(AllowedAttributesOnModifier, 
-                                                   level, version);
-    else
-      logError(AllowedAttributesOnSpeciesReference, 
-                                                   level, version);
-  }
- 
-  //
-  // id: SId  { use="optional" }  (L2v2->)
-  //
-  assigned = attributes.readInto("id", mId, getErrorLog());
-  if (assigned && mId.size() == 0)
-  {
-    logEmptyString("id", level, version, "<speciesReference>");
-  }
-  if (!SyntaxChecker::isValidInternalSId(mId)) logError(InvalidIdSyntax);
-
-  //
-  // name: string  { use="optional" }  (L2v2->)
-  //
-  attributes.readInto("name" , mName);
-
-}
-/** @endcond */
-
-
-/** @cond doxygen-libsbml-internal */
-/*
- * Subclasses should override this method to write their XML attributes
- * to the XMLOutputStream.  Be sure to call your parents implementation
- * of this method as well.
- */
-void
-SimpleSpeciesReference::writeAttributes (XMLOutputStream& stream) const
-{
-  SBase::writeAttributes(stream);
-
-  const unsigned int level   = getLevel  ();
-  const unsigned int version = getVersion();
-
-
-  if (level > 1)
-  {
-    if (!(level == 2 && version == 1))
-    {
-      //
-      // sboTerm: SBOTerm { use="optional" }  (L2v2->)
-      //
-      //
-      // sboTerm for L2V3 or later is written in SBase::writeAttributes()
-      //
-      if ( (level == 2) && (version == 2) )
-      {
-        SBO::writeTerm(stream, mSBOTerm);
-      }
-
-      //
-      // id: SId  { use="optional" }  (L2v2->)
-      //
-      stream.writeAttribute("id" , mId);
-
-      //
-      // name: string  { use="optional" }  (L2v2->)
-      //
-      stream.writeAttribute("name" , mName);
-    }
-  }
-
-  //
-  // specie : SName   { use="required" }  (L1v1)
-  // species: SName   { use="required" }  (L1v2, L2v1->)
-  //
-  const string s = (level == 1 && version == 1) ? "specie" : "species";
-  stream.writeAttribute(s , mSpecies);
-
-  //
-  // (EXTENSION)
-  //
-  SBase::writeExtensionAttributes(stream);
-}
-/** @endcond */
-
-
 
 SpeciesReference::SpeciesReference (unsigned int level, unsigned int version) :
    SimpleSpeciesReference( level, version)
@@ -885,7 +338,9 @@ SpeciesReference::setStoichiometryMath (const StoichiometryMath* math)
   else if (mStoichiometryMath == math) 
   {
     mIsSetStoichiometry = false;
+    mExplicitlySetStoichiometry = false;
     mStoichiometry = 1.0;
+    mDenominator = 1;
     return LIBSBML_OPERATION_SUCCESS;
   }
   else if (math == NULL)
@@ -905,6 +360,7 @@ SpeciesReference::setStoichiometryMath (const StoichiometryMath* math)
     mIsSetStoichiometry = false;
     mExplicitlySetStoichiometry = false;
     mStoichiometry      = 1.0;
+    mDenominator        = 1;
 
     delete mStoichiometryMath;
     mStoichiometryMath = static_cast<StoichiometryMath*>(math->clone());
@@ -969,6 +425,7 @@ SpeciesReference::unsetStoichiometryMath ()
     //
     mIsSetStoichiometry = true;
     mStoichiometry = 1.0;
+    mDenominator = 1;
   }
 
   if (mStoichiometryMath == NULL)
@@ -990,7 +447,9 @@ SpeciesReference::unsetStoichiometry ()
   if ( level > 2 )
   {
     mStoichiometry      = numeric_limits<double>::quiet_NaN();
+    mDenominator = 1;
     mIsSetStoichiometry = false;
+    mExplicitlySetStoichiometry = false;
     if (!isSetStoichiometry())
     {
       return LIBSBML_OPERATION_SUCCESS;
@@ -1003,6 +462,7 @@ SpeciesReference::unsetStoichiometry ()
   else
   {
     mStoichiometry      = 1.0;
+    mDenominator = 1;
 
     if ( level == 2 ) 
     {
@@ -1018,6 +478,7 @@ SpeciesReference::unsetStoichiometry ()
       else
       {
         mIsSetStoichiometry = false;
+        mExplicitlySetStoichiometry = false;
       }
     }
     else
@@ -1061,7 +522,9 @@ SpeciesReference::createStoichiometryMath ()
     mStoichiometryMath->connectToParent(this);
     /* this should unset the stoichiometry */
     mStoichiometry = 1.0;
+    mDenominator = 1;
     mIsSetStoichiometry = false;
+    mExplicitlySetStoichiometry = false;
   }
 
   return mStoichiometryMath;
@@ -1379,7 +842,8 @@ SpeciesReference::readOtherXML (XMLInputStream& stream)
       delete mCVTerms;
     }
     mCVTerms = new List();
-    RDFAnnotationParser::parseRDFAnnotation(mAnnotation, mCVTerms);
+    RDFAnnotationParser::parseRDFAnnotation(mAnnotation, mCVTerms, 
+                                            &(stream), getMetaId().c_str());
 //    new_annotation = RDFAnnotationParser::deleteRDFAnnotation(mAnnotation);
 //    delete mAnnotation;
 //    mAnnotation = new_annotation;
@@ -1468,7 +932,7 @@ SpeciesReference::readL1Attributes (const XMLAttributes& attributes)
   // stoichiometry: integer  { use="optional" default="1" }  (L1v1, L1v2)
   // stoichiometry: double   { use="optional" default="1" }  (L2v1->)
   //
-  mIsSetStoichiometry = attributes.readInto("stoichiometry", mStoichiometry);
+  mIsSetStoichiometry = attributes.readInto("stoichiometry", mStoichiometry, getErrorLog(), false, getLine(), getColumn());
   if (!mIsSetStoichiometry)
   {
     //  
@@ -1485,7 +949,7 @@ SpeciesReference::readL1Attributes (const XMLAttributes& attributes)
   //
   // denominator: integer  { use="optional" default="1" }  (L1v1, L1v2)
   //
-  mExplicitlySetDenominator = attributes.readInto("denominator", mDenominator);
+  mExplicitlySetDenominator = attributes.readInto("denominator", mDenominator,getErrorLog(), false, getLine(), getColumn());
 
 }
 /** @endcond */
@@ -1502,7 +966,7 @@ SpeciesReference::readL2Attributes (const XMLAttributes& attributes)
 {
   // stoichiometry: double   { use="optional" default="1" }  (L2v1->)
   //
-  mIsSetStoichiometry = attributes.readInto("stoichiometry", mStoichiometry);
+  mIsSetStoichiometry = attributes.readInto("stoichiometry", mStoichiometry, getErrorLog(), false, getLine(), getColumn());
   mExplicitlySetStoichiometry = mIsSetStoichiometry;
 }
 /** @endcond */
@@ -1522,12 +986,12 @@ SpeciesReference::readL3Attributes (const XMLAttributes& attributes)
   //
   // stoichiometry: double   { use="optional" default="1" }  (L2v1->)
   //
-  mIsSetStoichiometry = attributes.readInto("stoichiometry", mStoichiometry);
+  mIsSetStoichiometry = attributes.readInto("stoichiometry", mStoichiometry, getErrorLog(), false, getLine(), getColumn());
 
   //
   // constant: bool { use="required" } (L3v1 -> )
   //
-  mIsSetConstant = attributes.readInto("constant", mConstant);
+  mIsSetConstant = attributes.readInto("constant", mConstant, getErrorLog(), false, getLine(), getColumn());
   if (!mIsSetConstant && !isModifier())
   {
     logError(AllowedAttributesOnSpeciesReference, level, version);
@@ -1566,8 +1030,8 @@ SpeciesReference::writeAttributes (XMLOutputStream& stream) const
     //
     // stoichiometry: double   { use="optional" default="1" }  (L2v1, L2v2)
     //
-    if ((mDenominator == 1) && (mStoichiometry != 1 || 
-         isExplicitlySetStoichiometry()))
+    if ((mDenominator == 1) && 
+      (mStoichiometry != 1 || isExplicitlySetStoichiometry()))
     {
       stream.writeAttribute("stoichiometry", mStoichiometry);
     }
@@ -1645,91 +1109,6 @@ SpeciesReference::syncAnnotation ()
 /** @endcond */
 
 
-ModifierSpeciesReference::ModifierSpeciesReference (unsigned int level, 
-                          unsigned int version) :
-  SimpleSpeciesReference(level, version)
-{
-  if (!hasValidLevelVersionNamespaceCombination())
-    throw SBMLConstructorException();
-}
-
-
-ModifierSpeciesReference::ModifierSpeciesReference (SBMLNamespaces *sbmlns) :
-  SimpleSpeciesReference(sbmlns)
-{
-  if (!hasValidLevelVersionNamespaceCombination())
-    throw SBMLConstructorException();
-
-  loadPlugins(sbmlns);
-}
-
-
-/*
- * Destroys this ModifierSpeciesReference.
- */
-ModifierSpeciesReference::~ModifierSpeciesReference ()
-{
-}
-
-
-/*
- * Accepts the given SBMLVisitor.
- *
- * @return the result of calling <code>v.visit()</code>, which indicates
- * whether or not the Visitor would like to visit the Reaction's next
- * ModifierSpeciesReference (if available).
- */
-bool
-ModifierSpeciesReference::accept (SBMLVisitor& v) const
-{
-  return v.visit(*this);
-}
-
-
-/*
- * @return a (deep) copy of this ModifierSpeciesReference.
- */
-ModifierSpeciesReference*
-ModifierSpeciesReference::clone () const
-{
-  return new ModifierSpeciesReference(*this);
-}
-
-
-/*
- * @return the typecode (int) of SBML objects contained in this ListOf or
- * (default).
- *
- * @see getElementName()
- */
-int
-ModifierSpeciesReference::getTypeCode () const
-{
-  return SBML_MODIFIER_SPECIES_REFERENCE;
-}
-
-
-/*
- * @return the name of this element ie "modifierSpeciesReference".
- 
- */
-const string&
-ModifierSpeciesReference::getElementName () const
-{
-  static const string name = "modifierSpeciesReference";
-  return name;
-}
-
-
-bool 
-ModifierSpeciesReference::hasRequiredAttributes() const
-{
-  bool allPresent = SimpleSpeciesReference::hasRequiredAttributes();
-
-  return allPresent;
-}
-
-
 /*
  * Creates a new ListOfSpeciesReferences items.
  */
@@ -1747,17 +1126,10 @@ ListOfSpeciesReferences::ListOfSpeciesReferences (SBMLNamespaces* sbmlns)
   : ListOf(sbmlns)
  , mType(Unknown)
 {
+  loadPlugins(sbmlns);
 }
 
 
-/*
- * Creates a new ListOfSpeciesReferences.
- */
-// ListOfSpeciesReferences::ListOfSpeciesReferences () : mType(Unknown)
-// {
-// }
-
-
 /*
  * @return a (deep) copy of this ListOfSpeciesReferences.
  */
