@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -563,7 +563,7 @@ Trigger::readL3Attributes (const XMLAttributes& attributes)
   // initailValue { use="required"}  (L3v1 ->)
   //
   mIsSetInitialValue = attributes.readInto("initialValue", 
-                        mInitialValue, getErrorLog());
+                        mInitialValue, getErrorLog(), false, getLine(), getColumn());
 
   if (!mIsSetInitialValue)
   {
@@ -574,7 +574,7 @@ Trigger::readL3Attributes (const XMLAttributes& attributes)
   // persistent { use="required"}  (L3v1 ->)
   //
   mIsSetPersistent = attributes.readInto("persistent", 
-                        mPersistent, getErrorLog());
+                        mPersistent, getErrorLog(), false, getLine(), getColumn());
 
   if (!mIsSetPersistent)
   {
