@@ -3,7 +3,7 @@
  * @brief   RDFAnnotation I/O
  * @author  Sarah Keating
  * 
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -39,6 +39,8 @@
 #include <sbml/util/List.h>
 
 #include <sbml/annotation/ModelHistory.h>
+#include <sbml/annotation/ModelCreator.h>
+#include <sbml/annotation/Date.h>
 #include <sbml/annotation/RDFAnnotation.h>
 
 /** @cond doxygen-ignored */
@@ -49,16 +51,63 @@ using namespace std;
 
 LIBSBML_CPP_NAMESPACE_BEGIN
 
+/**
+ * logs the given erroron the error log of the stream.
+ * 
+ * @param stream the stream to log the error on
+ * @param element the element to log the error for
+ * @param code the error code to log
+ * @param msg optional message
+ */
+static void
+logError (XMLInputStream* stream, const XMLToken& element, SBMLErrorCode_t code,
+          const std::string& msg = "")
+{
+  if (&element == NULL || &stream == NULL) return;
+
+  SBMLNamespaces* ns = stream->getSBMLNamespaces();
+  if (ns != NULL)
+  {
+    static_cast <SBMLErrorLog*>
+      (stream->getErrorLog())->logError(
+      code,
+      ns->getLevel(), 
+      ns->getVersion(),
+      msg, 
+      element.getLine(), 
+      element.getColumn());
+  }
+  else
+  {
+    static_cast <SBMLErrorLog*>
+      (stream->getErrorLog())->logError(
+      code, 
+      SBML_DEFAULT_LEVEL, 
+      SBML_DEFAULT_VERSION, 
+      msg, 
+      element.getLine(), 
+      element.getColumn());
+  }
+}
+
 /*
  * takes an annotation that has been read into the model
  * identifies the RDF elements
  * and creates a List of CVTerms from the annotation
  */
 void 
-RDFAnnotationParser::parseRDFAnnotation(const XMLNode * annotation, List * CVTerms)
+RDFAnnotationParser::parseRDFAnnotation(
+     const XMLNode * annotation, 
+     List * CVTerms, 
+     XMLInputStream* stream /*= NULL*/, 
+     const char* metaId /*= NULL*/)
 {
   if (annotation == NULL) return;
 
+  const XMLTriple rdfAbout(
+                "about", 
+                "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
+                "rdf");
   const string&  name = annotation->getName();
   const XMLNode*  RDFTop = NULL;
   unsigned int n = 0;
@@ -76,10 +125,68 @@ RDFAnnotationParser::parseRDFAnnotation(const XMLNode * annotation, List * CVTer
       {
 	      if (annotation->getChild(n).getNumChildren() > 0)
 	      {
-          if (annotation->getChild(n).getChild(0).getName() == "Description")
+          const XMLNode& current = annotation->getChild(n).getChild(0);
+          if (current.getName() == "Description")
           {
-	          RDFTop = &(annotation->getChild(n).getChild(0));
-	          break;
+            if (current.hasAttr(rdfAbout))
+            {
+              string about = current.getAttrValue(rdfAbout);
+              if (!about.empty())
+              {
+	              if (metaId == NULL || about.find(metaId) != string::npos)
+                {
+                  RDFTop = &current;
+                  break;
+                }
+                else
+                {
+                  if (stream != NULL)
+                  {
+	                  logError(stream, current, RDFAboutTagNotMetaid);
+                  }
+                }
+              }
+              else
+              {
+                if (stream != NULL)
+                {
+                  logError(stream, current, RDFEmptyAboutTag);
+                }
+              }
+            }
+            else if (current.hasAttr("rdf:about"))
+            {
+              string about = current.getAttrValue("rdf:about");
+              if (!about.empty())
+              {
+	              if (metaId == NULL || about.find(metaId) != string::npos)
+                {
+                  RDFTop = &current;
+                  break;
+                }
+                else
+                {
+                  if (stream != NULL)
+                  {
+	                  logError(stream, current, RDFAboutTagNotMetaid);
+                  }
+                }
+              }
+              else
+              {
+                if (stream != NULL)
+                {
+                  logError(stream, current, RDFEmptyAboutTag);
+                }
+              }
+            }
+            else
+            {
+              if (stream != NULL)
+              {
+                logError(stream, current, RDFMissingAboutTag);
+              }
+            }
           }
 	      }
       }
@@ -204,10 +311,16 @@ RDFAnnotationParser::deleteRDFAnnotation(const XMLNode * annotation)
  */
 
 ModelHistory*
-RDFAnnotationParser::parseRDFAnnotation(const XMLNode * annotation)
+RDFAnnotationParser::parseRDFAnnotation(
+     const XMLNode * annotation, 
+     XMLInputStream* stream /*= NULL*/, 
+     const char* metaId /*= NULL*/)
 {
   if (annotation == NULL) return NULL;
-
+  const XMLTriple rdfAbout(
+                "about", 
+                "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
+                "rdf");
   const string&  name = annotation->getName();
   const XMLNode*  RDFTop = NULL;
   ModelHistory * history = NULL;
@@ -228,11 +341,69 @@ RDFAnnotationParser::parseRDFAnnotation(const XMLNode * annotation)
 	      {
 	        if (name1 == "RDF")
 	        {
-	          if (annotation->getChild(n).getNumChildren() > 0)
-	          {
-	            RDFTop = &(annotation->getChild(n).getChild(0));
-	            break;
-	          }
+	          const XMLNode& current = annotation->getChild(n).getChild(0);
+            if (current.getName() == "Description")
+            {
+              if (current.hasAttr(rdfAbout))
+              {
+                string about = current.getAttrValue(rdfAbout);
+                if (!about.empty())
+                {
+	                if (metaId == NULL || about.find(metaId) != string::npos)
+                  {
+                    RDFTop = &current;
+                    break;
+                  }
+                  else
+                  {
+                    if (stream != NULL)
+                    {
+	                    logError(stream, current, RDFAboutTagNotMetaid);
+                    }
+                  }
+                }
+                else
+                {
+                  if (stream != NULL)
+                  {
+                    logError(stream, current, RDFEmptyAboutTag);
+                  }
+                }
+              }
+              else if (current.hasAttr("rdf:about"))
+              {
+                string about = current.getAttrValue("rdf:about");
+                if (!about.empty())
+                {
+	                if (metaId == NULL || about.find(metaId) != string::npos)
+                  {
+                    RDFTop = &current;
+                    break;
+                  }
+                  else
+                  {
+                    if (stream != NULL)
+                    {
+	                    logError(stream, current, RDFAboutTagNotMetaid);
+                    }
+                  }
+                }
+                else
+                {
+                  if (stream != NULL)
+                  {
+                    logError(stream, current, RDFEmptyAboutTag);
+                  }
+                 }
+              }
+              else
+              {
+                if (stream != NULL)
+                {
+                  logError(stream, current, RDFMissingAboutTag);
+                }
+              }
+            }
 	        }
 	      }
 	      n++;
@@ -358,7 +529,8 @@ RDFAnnotationParser::parseCVTerms(const SBase * object)
 
   if (object == NULL || 
 	  object->getCVTerms() == NULL || 
-	  object->getCVTerms()->getSize() == 0)
+	  object->getCVTerms()->getSize() == 0 ||
+    !object->isSetMetaId())
   {
     return NULL;
   }
@@ -481,7 +653,8 @@ XMLNode *
 RDFAnnotationParser::parseModelHistory(const SBase *object)
 {
   if (object == NULL  || 
-		(object->getLevel() < 3 && object->getTypeCode() != SBML_MODEL))
+		(object->getLevel() < 3 && object->getTypeCode() != SBML_MODEL) ||
+    !object->isSetMetaId())
   {
     return NULL;
   }
@@ -868,9 +1041,18 @@ RDFAnnotationParser::hasHistoryRDFAnnotation(const XMLNode *annotation)
 
   // check whether the annotation relates to Model History
   ModelHistory *temp = parseRDFAnnotation(annotation);
-  if (temp && temp->getNumCreators() > 0)
+  /* ok I relaxed the test so that an invalid model could be stored
+   * but need to check that it resembles an model history in some 
+   * way otherwise any RDF could be a model history
+   */
+  if (temp != NULL) // && temp->getNumCreators() > 0)
   {
-    hasHistoryRDF = true;
+    if (temp->getNumCreators() > 0 
+      || temp->isSetCreatedDate() == true
+      || temp->isSetModifiedDate() == true )
+    {
+      hasHistoryRDF = true;
+    }
   }
   delete temp;
 
