@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -338,7 +338,7 @@ START_TEST (test_RDFAnnotation_recreate)
   Compartment* c = m->getCompartment(1);
 
   const char * expected =
-    "<compartment id=\"A\">\n"
+    "<compartment metaid=\"_000003\" id=\"A\">\n"
     "  <annotation>\n"
 		"    <jd2:JDesignerLayout version=\"2.0\" MajorVersion=\"2\" MinorVersion=\"0\" BuildVersion=\"41\">\n"
 		"      <jd2:header>\n"
@@ -348,7 +348,7 @@ START_TEST (test_RDFAnnotation_recreate)
 		"      </jd2:header>\n"
 		"    </jd2:JDesignerLayout>\n"
 		"    <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:vCard=\"http://www.w3.org/2001/vcard-rdf/3.0#\" xmlns:bqbiol=\"http://biomodels.net/biology-qualifiers/\" xmlns:bqmodel=\"http://biomodels.net/model-qualifiers/\">\n"
-		"      <rdf:Description rdf:about=\"#\">\n"
+		"      <rdf:Description rdf:about=\"#_000003\">\n"
 		"        <bqbiol:is>\n"
 		"          <rdf:Bag>\n"
 		"            <rdf:li rdf:resource=\"http://www.geneontology.org/#GO:0007274\"/>\n"
@@ -368,10 +368,10 @@ START_TEST (test_RDFAnnotation_recreateFromEmpty)
   Compartment* c = m->getCompartment(3);
 
   const char * expected =
-    "<compartment id=\"C\">\n"
+    "<compartment metaid=\"_000004\" id=\"C\">\n"
     "  <annotation>\n"
 		"    <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:vCard=\"http://www.w3.org/2001/vcard-rdf/3.0#\" xmlns:bqbiol=\"http://biomodels.net/biology-qualifiers/\" xmlns:bqmodel=\"http://biomodels.net/model-qualifiers/\">\n"
-		"      <rdf:Description rdf:about=\"#\">\n"
+		"      <rdf:Description rdf:about=\"#_000004\">\n"
 		"        <bqbiol:is>\n"
 		"          <rdf:Bag>\n"
 		"            <rdf:li rdf:resource=\"http://www.geneontology.org/#GO:0007274\"/>\n"
@@ -434,6 +434,160 @@ START_TEST (test_RDFAnnotation_recreateWithOutOther)
 }
 END_TEST
 
+#include <sbml/SBMLTypes.h>
+
+START_TEST (test_RDFAnnotation_testAnnotationForMetaId)
+{
+
+  SBMLDocument doc(3, 1);
+  Model* model = doc.createModel();
+  fail_unless(model != NULL);
+  
+  model->setId("test1");
+    
+  CVTerm term (MODEL_QUALIFIER);
+  term.addResource("testResource");
+  term.setModelQualifierType(BQM_IS);
+  
+  model->setMetaId("t1");
+  model->addCVTerm(&term);
+  
+  // unset metaid ... now we have potentially dangling RDF
+  model->setMetaId("");
+  std::string test = model->toSBML();
+
+  // this should be the test
+  fail_unless(test == "<model id=\"test1\"/>");   
+
+}
+END_TEST
+
+#include <sbml/xml/XMLInputStream.h>
+  
+START_TEST (test_RDFAnnotation_testMissingAbout)
+{
+
+ const char * withAbout =
+    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+    "  <annotation>\n"
+		"    <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:vCard=\"http://www.w3.org/2001/vcard-rdf/3.0#\" xmlns:bqbiol=\"http://biomodels.net/biology-qualifiers/\" xmlns:bqmodel=\"http://biomodels.net/model-qualifiers/\">\n"
+		"      <rdf:Description rdf:about=\"#_000004\">\n"
+		"        <bqbiol:is>\n"
+		"          <rdf:Bag>\n"
+		"            <rdf:li rdf:resource=\"http://www.geneontology.org/#GO:0007274\"/>\n"
+		"          </rdf:Bag>\n"
+		"        </bqbiol:is>\n"
+		"      </rdf:Description>\n"
+		"    </rdf:RDF>\n"
+    "  </annotation>";
+
+  const char * emptyAbout =
+    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+    "  <annotation>\n"
+		"    <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:vCard=\"http://www.w3.org/2001/vcard-rdf/3.0#\" xmlns:bqbiol=\"http://biomodels.net/biology-qualifiers/\" xmlns:bqmodel=\"http://biomodels.net/model-qualifiers/\">\n"
+		"      <rdf:Description rdf:about=\"\">\n"
+		"        <bqbiol:is>\n"
+		"          <rdf:Bag>\n"
+		"            <rdf:li rdf:resource=\"http://www.geneontology.org/#GO:0007274\"/>\n"
+		"          </rdf:Bag>\n"
+		"        </bqbiol:is>\n"
+		"      </rdf:Description>\n"
+		"    </rdf:RDF>\n"
+    "  </annotation>";
+
+  const char * noAbout =
+    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+    "  <annotation>\n"
+		"    <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:vCard=\"http://www.w3.org/2001/vcard-rdf/3.0#\" xmlns:bqbiol=\"http://biomodels.net/biology-qualifiers/\" xmlns:bqmodel=\"http://biomodels.net/model-qualifiers/\">\n"
+		"      <rdf:Description>\n"
+		"        <bqbiol:is>\n"
+		"          <rdf:Bag>\n"
+		"            <rdf:li rdf:resource=\"http://www.geneontology.org/#GO:0007274\"/>\n"
+		"          </rdf:Bag>\n"
+		"        </bqbiol:is>\n"
+		"      </rdf:Description>\n"
+		"    </rdf:RDF>\n"
+    "  </annotation>";
+
+ List *cvTerms = new List();
+ XMLInputStream stream(withAbout,false);
+ XMLNode node(stream);
+ RDFAnnotationParser::parseRDFAnnotation( &node, cvTerms );
+
+ // regular parsing
+ fail_unless( cvTerms->getSize() == 1 );
+ 
+ // test parsing for specific meta id
+ cvTerms = new List();
+ RDFAnnotationParser::parseRDFAnnotation( &node, cvTerms, NULL, "_000004" );
+ fail_unless( cvTerms->getSize() == 1 );
+
+ // test parsing for a non-existing meta id
+ delete cvTerms;
+ cvTerms=NULL;
+
+ cvTerms = new List();
+ RDFAnnotationParser::parseRDFAnnotation( &node, cvTerms, NULL, "badMetaId" );
+ fail_unless( cvTerms->getSize() == 0 );
+
+
+ // now the test with empty about 
+ delete cvTerms;
+ cvTerms=NULL;
+
+ cvTerms = new List();
+
+ XMLInputStream stream1(emptyAbout,false);
+ XMLNode node1(stream1);
+ RDFAnnotationParser::parseRDFAnnotation( &node1, cvTerms );
+
+ fail_unless( cvTerms->getSize() == 0 );
+ 
+ // now the test with empty about 
+ delete cvTerms;
+ cvTerms=NULL;
+
+ cvTerms = new List();
+
+ XMLInputStream stream2(noAbout,false);
+ XMLNode node2(stream2);
+ RDFAnnotationParser::parseRDFAnnotation( &node2, cvTerms );
+
+ fail_unless( cvTerms->getSize() == 0 );
+
+ delete cvTerms;
+ cvTerms=NULL;
+
+}
+END_TEST
+
+
+START_TEST (test_RDFAnnotation_testMissingMetaId)
+{
+
+  SBMLDocument doc(3, 1);
+  Model* model = doc.createModel();
+  fail_unless(model != NULL);
+  
+  model->setId("test1");
+    
+  CVTerm term (MODEL_QUALIFIER);
+  term.addResource("testResource");
+  term.setModelQualifierType(BQM_IS);
+  
+  model->setMetaId("t1");
+  model->addCVTerm(&term);
+  
+  // unset metaid ... now we have potentially dangling RDF
+  model->setMetaId("");
+  std::string test = model->toSBML();
+
+  // this should be the test
+  fail_unless(test == "<model id=\"test1\"/>");   
+
+}
+END_TEST
+
 Suite *
 create_suite_RDFAnnotation (void)
 {
@@ -453,6 +607,9 @@ create_suite_RDFAnnotation (void)
   tcase_add_test(tcase, test_RDFAnnotation_recreateFromEmpty );
   tcase_add_test(tcase, test_RDFAnnotation_deleteWithOutOther );
   tcase_add_test(tcase, test_RDFAnnotation_recreateWithOutOther );
+  tcase_add_test(tcase, test_RDFAnnotation_testMissingMetaId );
+  tcase_add_test(tcase, test_RDFAnnotation_testMissingAbout );
+  tcase_add_test(tcase, test_RDFAnnotation_testAnnotationForMetaId );
   suite_add_tcase(suite, tcase);
 
   return suite;
