@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -28,16 +28,14 @@
 
 #include <sbml/conversion/SBMLLevelVersionConverter.h>
 #include <sbml/conversion/SBMLConverterRegistry.h>
-#include <sbml/conversion/SBMLConverterRegister.h>
+#include <sbml/extension/SBMLExtensionRegistry.h>
+
+#include <sbml/SBMLTypes.h>
 
 #ifdef __cplusplus
 
 #include <algorithm>
 #include <string>
-#ifdef USE_LAYOUT
-  #include <sbml/packages/layout/extension/LayoutExtension.h>
-  #include <sbml/packages/layout/extension/LayoutModelPlugin.h>
-#endif
 
 using namespace std;
 LIBSBML_CPP_NAMESPACE_BEGIN
@@ -211,52 +209,21 @@ SBMLLevelVersionConverter::convert()
 
   bool conversion = false;
 
-  /* if model has extensions we cannot convert */
-  if (mDocument->getNumPlugins() > 0)
-  {
-    /* if it is layout extension and we are converting between l2 targetVersions
-     * that is okay
-     */
-    if (currentLevel == 2 && mDocument->getPlugin("layout") != NULL)
-    {
-      bool problem = false;
-      /* if there is no layout - no problem ! */
-#ifdef USE_LAYOUT
-      if (currentModel != NULL)
-      {
-        LayoutModelPlugin * mplugin = static_cast<LayoutModelPlugin*>
-                      (currentModel->getPlugin("layout"));
-      
-        if (mplugin != NULL && mplugin->getNumLayouts() > 0 && targetLevel != 2)
-        {
-          problem = true;
-        }
-        else if (mDocument->getNumPlugins() > 1)
-        {
-          problem = true;
-        }
-        else if (targetLevel == 3)
-        {
-          /* disable the L2 extension as leaving it enabled means you
-           * cannot enable the L3 extension later
-           */
-          mDocument->enablePackage(LayoutExtension::getXmlnsL2(),"layout", false);
+  bool ignorePackages = getProperties()->getBoolValue("ignorePackages");
 
-        }
-      }
-#endif      
-      if (problem == true)
-      {
-        mDocument->getErrorLog()->logError(PackageConversionNotSupported, 
-                                           currentLevel, currentVersion);
-        return LIBSBML_CONV_PKG_CONVERSION_NOT_AVAILABLE;
-      }
-    }
-    else
+  /* if model has extensions we cannot convert */
+  if (!ignorePackages && mDocument->getNumPlugins() > 0)
+  {
+
+    // disable all unused packages
+    SBMLExtensionRegistry::getInstance().disableUnusedPackages(mDocument);
+    if (mDocument->getNumPlugins() > 0)
     {
+      // if there are still plugins enabled fail
       mDocument->getErrorLog()->logError(PackageConversionNotSupported, 
                                          currentLevel, currentVersion);
       return LIBSBML_CONV_PKG_CONVERSION_NOT_AVAILABLE;
+
     }
   }
 
@@ -281,7 +248,7 @@ SBMLLevelVersionConverter::convert()
    */
   /* see whether the unit validator is on */
   //bool strictSBO   = ((convValidators & 0x04) == 0x04);
-  bool strictUnits = ((convValidators & 0x10) == 0x10);
+  bool strictUnits = strict && ((convValidators & 0x10) == 0x10);
   
   if (strict == true)
   {
@@ -333,14 +300,13 @@ SBMLLevelVersionConverter::convert()
 
   if (currentModel != NULL)
   {
-    bool doConversion = false;
     unsigned int origLevel;
     unsigned int origVersion;
     Model *origModel;
     if (strict)
     {
       /* here we are strict and only want to do
-       * contargetVersion if output will be valid
+       * conversion if output will be valid
        *
        * save a copy of the model so it can be restored
        */
@@ -348,419 +314,8 @@ SBMLLevelVersionConverter::convert()
       origVersion = currentVersion;
       origModel = currentModel->clone();
     }
-      
-    if (currentLevel == 1)
-    {
-      switch (targetLevel)
-      {
-      case 1:
-        switch (targetVersion)
-        {
-        case 1:
-          mDocument->getErrorLog()->logError(CannotConvertToL1V1);
-          break;
-        case 2:
-          mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-          conversion = true;
-          break;
-        default:
-          mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-          break;
-        }
-        break;
-      case 2:    
-        switch (targetVersion)
-        {
-        case 1:
-          if (!conversion_errors(mDocument->checkL2v1Compatibility()))
-          {
-            doConversion = true;
-          }
-          break;
-        case 2:
-          if (!conversion_errors(mDocument->checkL2v2Compatibility()))
-          {
-            doConversion = true;
-          }
-          break;
-        case 3:
-          if (!conversion_errors(mDocument->checkL2v3Compatibility()))
-          {
-            doConversion = true;
-          }
-          break;
-        case 4:
-          if (!conversion_errors(mDocument->checkL2v4Compatibility()))
-          {
-            doConversion = true;
-          }
-          break;
-        default:
-          mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-          break;
-        }
-        if (doConversion == true)
-        {
-          currentModel->removeParameterRuleUnits(strict);
-          mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-          currentModel->convertL1ToL2();
-          conversion = true;
-        }
-        break;
-      case 3:
-        switch (targetVersion)
-        {
-        case 1:
-          if (!conversion_errors(mDocument->checkL3v1Compatibility()))
-          {
-            doConversion = true;
-          }
-          break;
-        default:
-          mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-          break;
-        }
-        if (doConversion == true)
-        {
-          currentModel->removeParameterRuleUnits(strict);
-          currentModel->convertParametersToLocals(targetLevel, targetVersion);
-          mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-          currentModel->convertL1ToL3();
-          conversion = true;
-        }
-        break;
-      default:
-        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-        break;
-      }
-    }
-    else if (currentLevel == 2)
-    {
-      switch (targetLevel)
-      {
-      case 1:
-        switch (targetVersion)
-        {
-        case 1:
-          mDocument->getErrorLog()->logError(CannotConvertToL1V1);
-          break;
-        case 2:
-          if (!conversion_errors(mDocument->checkL1Compatibility()))
-          {
-            doConversion = true;
-            /* if existing model is L2V4 need to mDocument->check that
-            * units are strict
-            */
-            if (currentVersion == 4 && !hasStrictUnits())
-            {
-              if (strict == false)
-              {
-                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL1);
-              }
-              else
-              {
-                if (strictUnits == true)
-                {
-                  mDocument->getErrorLog()->logError(StrictUnitsRequiredInL1);
-                  doConversion = false;
-                }
-              }
-            }
-            else
-            {
-              doConversion = true;
-            }
-          }
-          break;
-        default:
-          mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-          break;
-        }
-        if (doConversion == true)
-        {
-          currentModel->convertL2ToL1(strict);
-          mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-          conversion = true;
-        }
-        break;
-      case 2:
-        switch (targetVersion)
-        {
-        case 1:
-          if (!conversion_errors(mDocument->checkL2v1Compatibility()))
-          {
-            doConversion = true;
-            /* if existing model is L2V4 need to mDocument->check that
-            * units are strict
-            */
-            if (currentVersion == 4 && !hasStrictUnits())
-            {
-              if (strict == false)
-              {
-                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v1);
-              }
-              else
-              {
-                if (strictUnits == true)
-                {
-                  mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v1);
-                  doConversion = false;
-                }
-              }
-            }
-            else
-            {
-              doConversion = true;
-            }
-          }
-          break;
-        case 2:
-          if (!conversion_errors(mDocument->checkL2v2Compatibility()))
-          {
-            doConversion = true;
-            /* if existing model is L2V4 need to mDocument->check that
-            * units are strict
-            */
-            if (currentVersion == 4 && !hasStrictUnits())
-            {
-              if (strict == false)
-              {
-                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v2);
-              }
-              else
-              {
-                if (strictUnits == true)
-                {
-                  mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v2);
-                  doConversion = false;
-                }
-              }
-            }
-            
-            if (currentVersion == 4 && !hasStrictSBO())
-            {
-              if (strict == false)
-              {
-                mDocument->getErrorLog()->logError(StrictSBORequiredInL2v2);
-              }
-              else
-              {
-                if (strictUnits == true)
-                {
-                  mDocument->getErrorLog()->logError(StrictSBORequiredInL2v2);
-                  doConversion = false;
-                }
-              }
-            }
-            // look for duplicate top targetLevel annotations
-            for (i = 0; i < mDocument->getErrorLog()->getNumErrors(); i++)
-            {
-              if (mDocument->getErrorLog()->getError(i)->getErrorId() 
-                                  == DuplicateAnnotationInvalidInL2v2)
-                duplicateAnn = true;
-            }
-           }
-          break;
-        case 3:
-          if (!conversion_errors(mDocument->checkL2v3Compatibility()))
-          {
-            doConversion = true;
-            /* if existing model is L2V4 need to mDocument->check that
-            * units are strict
-            */
-            if (currentVersion == 4 && !hasStrictUnits())
-            {
-              if (strict == false)
-              {
-                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v3);
-              }
-              else
-              {
-                if (strictUnits == true)
-                {
-                  mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v3);
-                  doConversion = false;
-                }
-              }
-            }
-            
-            if (currentVersion == 4 && !hasStrictSBO())
-            {
-              if (strict == false)
-              {
-                mDocument->getErrorLog()->logError(StrictSBORequiredInL2v3);
-              }
-              else
-              {
-                if (strictUnits == true)
-                {
-                  mDocument->getErrorLog()->logError(StrictSBORequiredInL2v3);
-                  doConversion = false;
-                }
-              }
-            }
-            // look for duplicate top targetLevel annotations
-            for (i = 0; i < mDocument->getErrorLog()->getNumErrors(); i++)
-            {
-              if (mDocument->getErrorLog()->getError(i)->getErrorId() 
-                              == DuplicateAnnotationInvalidInL2v3)
-                duplicateAnn = true;
-            }
-           }
-          break;
-        case 4:
-          if (!conversion_errors(mDocument->checkL2v4Compatibility()))
-          {
-            doConversion = true;
-            // look for duplicate top targetLevel annotations
-            for (i = 0; i < mDocument->getErrorLog()->getNumErrors(); i++)
-            {
-              if (mDocument->getErrorLog()->getError(i)->getErrorId() 
-                              == DuplicateAnnotationInvalidInL2v4)
-                duplicateAnn = true;
-            }
-          }
-          break;
-        default:
-          mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-          break;
-        }
-        if (doConversion == true)
-        {
-          if (duplicateAnn == true)
-          {
-            mDocument->removeDuplicateAnnotations();
-            currentModel->removeDuplicateTopLevelAnnotations();
-          }
-          if (targetVersion == 1)
-          {
-            currentModel->removeSBOTerms(strict);
-          }
-          else if (targetVersion == 2)
-          {
-            currentModel->removeSBOTermsNotInL2V2(strict);
-          }
-          mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-          conversion = true;
-        }
-        break;
-      case 3:
-        switch (targetVersion)
-        {
-        case 1:
-          if (!conversion_errors(mDocument->checkL3v1Compatibility()))
-          {
-            doConversion = true;
-            // look for duplicate top targetLevel annotations
-            for (i = 0; i < mDocument->getErrorLog()->getNumErrors(); i++)
-            {
-              if (mDocument->getErrorLog()->getError(i)->getErrorId() 
-                              == DuplicateAnnotationInvalidInL2v4)
-                duplicateAnn = true;
-            }
-          }
-          break;
-        default:
-          mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-          break;
-        }
-        if (doConversion == true)
-        {
-          if (duplicateAnn == true)
-          {
-            mDocument->removeDuplicateAnnotations();
-            currentModel->removeDuplicateTopLevelAnnotations();
-          }
-          currentModel->convertParametersToLocals(targetLevel, targetVersion);
-          mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-          currentModel->convertL2ToL3();
-          conversion = true;
-        }
-        break;
-      default:
-        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-        break;
-      }      
-    }
-    else if (currentLevel == 3)
-      {
-        switch (targetLevel)
-        {
-        case 1:
-          switch (targetVersion)
-          {
-          case 1:
-            mDocument->getErrorLog()->logError(CannotConvertToL1V1);
-            break;
-          case 2:
-            if (!conversion_errors(mDocument->checkL1Compatibility(), strictUnits))
-            {
-              mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-              currentModel->convertL3ToL1();
-              conversion = true;
-            }
-            break;
-          default:
-            mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-            break;
-          }
-          break;
-        case 2:
-          switch (targetVersion)
-          {
-          case 1:
-            if (!conversion_errors(mDocument->checkL2v1Compatibility(), strictUnits))
-            {
-              doConversion = true;
-            }
-            break;
-          case 2:
-            if (!conversion_errors(mDocument->checkL2v2Compatibility(), strictUnits))
-            {
-              doConversion = true;
-            }
-            break;
-          case 3:
-            if (!conversion_errors(mDocument->checkL2v3Compatibility(), strictUnits))
-            {
-              doConversion = true;
-            }
-            break;
-          case 4:
-            if (!conversion_errors(mDocument->checkL2v4Compatibility(), strictUnits))
-            {
-              doConversion = true;
-            }
-            break;
-          default:
-            mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-            break;
-          }
-          if (doConversion == true)
-          {
-            mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-            currentModel->convertL3ToL2(strict);
-            conversion = true;
-          }
-          break;
-        case 3:
-          switch (targetVersion)
-          {
-          case 1:
-            conversion = true;
-            break;
-          default:
-            mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-            break;
-          }
-          break;
-        default:
-          mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
-          break;
-        }
-
-      }
 
+    conversion = performConversion(strict, strictUnits, duplicateAnn);
       
     if (conversion == false)
     {
@@ -779,7 +334,7 @@ SBMLLevelVersionConverter::convert()
       {
         /* now we want to mDocument->check whether the resulting model is valid
          */
-        mDocument->checkConsistency();
+        mDocument->validateSBML();
         unsigned int errors = 
            mDocument->getErrorLog()->getNumFailsWithSeverity(LIBSBML_SEV_ERROR);
         if (errors > 0)
@@ -815,7 +370,545 @@ SBMLLevelVersionConverter::convert()
   else
     return LIBSBML_OPERATION_FAILED;
 }
+ 
+
+/** @cond doxygen-libsbml-internal */
+bool
+SBMLLevelVersionConverter::performConversion(bool strict, bool strictUnits, 
+                                        bool duplicateAnn)
+{
+  bool conversion = false;
+ 
+  bool doConversion = false;
   
+  unsigned int currentLevel = mDocument->getLevel();
+  unsigned int currentVersion = mDocument->getVersion();
+  unsigned int targetLevel = getTargetLevel(); 
+  unsigned int targetVersion = getTargetVersion();
+  Model * currentModel = mDocument->getModel();
+
+  unsigned int i = 0;
+
+  if (currentLevel == 1)
+  {
+    switch (targetLevel)
+    {
+    case 1:
+      switch (targetVersion)
+      {
+      case 1:
+        mDocument->getErrorLog()->logError(CannotConvertToL1V1);
+        break;
+      case 2:
+        mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
+        conversion = true;
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      break;
+    case 2:    
+      switch (targetVersion)
+      {
+      case 1:
+        if (!conversion_errors(mDocument->checkL2v1Compatibility()))
+        {
+          doConversion = true;
+        }
+        break;
+      case 2:
+        if (!conversion_errors(mDocument->checkL2v2Compatibility()))
+        {
+          doConversion = true;
+        }
+        break;
+      case 3:
+        if (!conversion_errors(mDocument->checkL2v3Compatibility()))
+        {
+          doConversion = true;
+        }
+        break;
+      case 4:
+        if (!conversion_errors(mDocument->checkL2v4Compatibility()))
+        {
+          doConversion = true;
+        }
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      if (doConversion == true)
+      {
+        currentModel->removeParameterRuleUnits(strict);
+        mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
+        currentModel->convertL1ToL2();
+        conversion = true;
+      }
+      break;
+    case 3:
+      switch (targetVersion)
+      {
+      case 1:
+        if (!conversion_errors(mDocument->checkL3v1Compatibility()))
+        {
+          doConversion = true;
+        }
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      if (doConversion == true)
+      {
+        currentModel->removeParameterRuleUnits(strict);
+        currentModel->convertParametersToLocals(targetLevel, targetVersion);
+        mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
+        currentModel->convertL1ToL3();
+        conversion = true;
+      }
+      break;
+    default:
+      mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+      break;
+    }
+  }
+  else if (currentLevel == 2)
+  {
+    switch (targetLevel)
+    {
+    case 1:
+      switch (targetVersion)
+      {
+      case 1:
+        mDocument->getErrorLog()->logError(CannotConvertToL1V1);
+        break;
+      case 2:
+        if (!conversion_errors(mDocument->checkL1Compatibility()))
+        {
+          doConversion = true;
+          /* if existing model is L2V4 need to mDocument->check that
+          * units are strict
+          */
+          if (currentVersion == 4 && !hasStrictUnits())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictUnitsRequiredInL1);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL1);
+                doConversion = false;
+              }
+            }
+          }
+          else
+          {
+            doConversion = true;
+          }
+        }
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      if (doConversion == true)
+      {
+        mDocument->expandFunctionDefinitions();
+        mDocument->expandInitialAssignments();
+        currentModel->convertL2ToL1(strict);
+        mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
+        conversion = true;
+      }
+      break;
+    case 2:
+      switch (targetVersion)
+      {
+      case 1:
+        if (!conversion_errors(mDocument->checkL2v1Compatibility()))
+        {
+          doConversion = true;
+          /* if existing model is L2V4 need to mDocument->check that
+          * units are strict
+          */
+          if (currentVersion == 4 && !hasStrictUnits())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v1);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v1);
+                doConversion = false;
+              }
+            }
+          }
+          else
+          {
+            doConversion = true;
+          }
+        }
+        break;
+      case 2:
+        if (!conversion_errors(mDocument->checkL2v2Compatibility()))
+        {
+          doConversion = true;
+          /* if existing model is L2V4 need to mDocument->check that
+          * units are strict
+          */
+          if (currentVersion == 4 && !hasStrictUnits())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v2);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v2);
+                doConversion = false;
+              }
+            }
+          }
+          
+          if (currentVersion == 4 && !hasStrictSBO())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictSBORequiredInL2v2);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictSBORequiredInL2v2);
+                doConversion = false;
+              }
+            }
+          }
+          // look for duplicate top targetLevel annotations
+          for (i = 0; i < mDocument->getErrorLog()->getNumErrors(); i++)
+          {
+            if (mDocument->getErrorLog()->getError(i)->getErrorId() 
+                                == DuplicateAnnotationInvalidInL2v2)
+              duplicateAnn = true;
+          }
+         }
+        break;
+      case 3:
+        if (!conversion_errors(mDocument->checkL2v3Compatibility()))
+        {
+          doConversion = true;
+          /* if existing model is L2V4 need to mDocument->check that
+          * units are strict
+          */
+          if (currentVersion == 4 && !hasStrictUnits())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v3);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v3);
+                doConversion = false;
+              }
+            }
+          }
+          
+          if (currentVersion == 4 && !hasStrictSBO())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictSBORequiredInL2v3);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictSBORequiredInL2v3);
+                doConversion = false;
+              }
+            }
+          }
+          // look for duplicate top targetLevel annotations
+          for (i = 0; i < mDocument->getErrorLog()->getNumErrors(); i++)
+          {
+            if (mDocument->getErrorLog()->getError(i)->getErrorId() 
+                            == DuplicateAnnotationInvalidInL2v3)
+              duplicateAnn = true;
+          }
+         }
+        break;
+      case 4:
+        if (!conversion_errors(mDocument->checkL2v4Compatibility()))
+        {
+          doConversion = true;
+          // look for duplicate top targetLevel annotations
+          for (i = 0; i < mDocument->getErrorLog()->getNumErrors(); i++)
+          {
+            if (mDocument->getErrorLog()->getError(i)->getErrorId() 
+                            == DuplicateAnnotationInvalidInL2v4)
+              duplicateAnn = true;
+          }
+        }
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      if (doConversion == true)
+      {
+        if (duplicateAnn == true)
+        {
+          mDocument->removeDuplicateAnnotations();
+          currentModel->removeDuplicateTopLevelAnnotations();
+        }
+        if (targetVersion == 1)
+        {
+          currentModel->removeSBOTerms(strict);
+          mDocument->expandInitialAssignments();
+        }
+        else if (targetVersion == 2)
+        {
+          currentModel->removeSBOTermsNotInL2V2(strict);
+        }
+        mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
+        conversion = true;
+      }
+      break;
+    case 3:
+      switch (targetVersion)
+      {
+      case 1:
+        if (!conversion_errors(mDocument->checkL3v1Compatibility()))
+        {
+          doConversion = true;
+          // look for duplicate top targetLevel annotations
+          for (i = 0; i < mDocument->getErrorLog()->getNumErrors(); i++)
+          {
+            if (mDocument->getErrorLog()->getError(i)->getErrorId() 
+                            == DuplicateAnnotationInvalidInL2v4)
+              duplicateAnn = true;
+          }
+        }
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      if (doConversion == true)
+      {
+        if (duplicateAnn == true)
+        {
+          mDocument->removeDuplicateAnnotations();
+          currentModel->removeDuplicateTopLevelAnnotations();
+        }
+        currentModel->convertParametersToLocals(targetLevel, targetVersion);
+        mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
+        currentModel->convertL2ToL3();
+        conversion = true;
+      }
+      break;
+    default:
+      mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+      break;
+    }      
+  }
+  else if (currentLevel == 3)
+  {
+    switch (targetLevel)
+    {
+    case 1:
+      switch (targetVersion)
+      {
+      case 1:
+        mDocument->getErrorLog()->logError(CannotConvertToL1V1);
+        break;
+      case 2:
+        if (!conversion_errors(mDocument->checkL1Compatibility(), strictUnits))
+        {
+          doConversion = true;
+          if (!hasStrictUnits())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictUnitsRequiredInL1);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL1);
+                doConversion = false;
+              }
+            }
+          }
+          if (doConversion == true)
+          {
+            mDocument->expandFunctionDefinitions();
+            mDocument->expandInitialAssignments();
+            mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
+            currentModel->convertL3ToL1();
+            conversion = true;
+          }
+        }
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      break;
+    case 2:
+      switch (targetVersion)
+      {
+      case 1:
+        if (!conversion_errors(mDocument->checkL2v1Compatibility(), strictUnits))
+        {
+          doConversion = true;
+           if (!hasStrictUnits())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v1);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v1);
+                doConversion = false;
+              }
+            }
+          }
+       }
+        break;
+      case 2:
+        if (!conversion_errors(mDocument->checkL2v2Compatibility(), strictUnits))
+        {
+          doConversion = true;
+          if (!hasStrictUnits())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v2);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v2);
+                doConversion = false;
+              }
+            }
+          }
+          if (!hasStrictSBO())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictSBORequiredInL2v2);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictSBORequiredInL2v2);
+                doConversion = false;
+              }
+            }
+          }
+       }
+        break;
+      case 3:
+        if (!conversion_errors(mDocument->checkL2v3Compatibility(), strictUnits))
+        {
+          doConversion = true;
+          if (!hasStrictUnits())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v3);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictUnitsRequiredInL2v3);
+                doConversion = false;
+              }
+            }
+          }
+          if (!hasStrictSBO())
+          {
+            if (strict == false)
+            {
+              mDocument->getErrorLog()->logError(StrictSBORequiredInL2v3);
+            }
+            else
+            {
+              if (strictUnits == true)
+              {
+                mDocument->getErrorLog()->logError(StrictSBORequiredInL2v3);
+                doConversion = false;
+              }
+            }
+          }
+        }
+        break;
+      case 4:
+        if (!conversion_errors(mDocument->checkL2v4Compatibility(), strictUnits))
+        {
+          doConversion = true;
+        }
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      if (doConversion == true)
+      {
+        if (targetVersion == 1)
+          mDocument->expandInitialAssignments();
+        mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
+        currentModel->convertL3ToL2(strict);
+        conversion = true;
+      }
+      break;
+    case 3:
+      switch (targetVersion)
+      {
+      case 1:
+        conversion = true;
+        break;
+      default:
+        mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+        break;
+      }
+      break;
+    default:
+      mDocument->getErrorLog()->logError(InvalidTargetLevelVersion, currentLevel, currentVersion);
+      break;
+    }
+
+  }
+
+  return conversion;
+
+}
+
+/** @endcond */
+
+
 /** @cond doxygen-libsbml-internal */
 /*
  * Predicate returning true if the errors encountered are not ignorable.
@@ -837,6 +930,7 @@ SBMLLevelVersionConverter::conversion_errors(unsigned int errors, bool strictUni
         mDocument->getErrorLog()->remove(IntegerSpatialDimensions);
       }
     }
+    mDocument->getErrorLog()->remove(GlobalUnitsNotDeclared);
   }
   /** 
    * changed this code in line with the rest of the validation 
