@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -31,6 +31,8 @@
 #include <sbml/conversion/SBMLConverterRegister.h>
 #include <sbml/extension/SBasePlugin.h>
 
+#include <sbml/SBMLTypes.h>
+
 #ifdef __cplusplus
 
 #include <algorithm>
@@ -485,8 +487,8 @@ SBMLUnitsConverter::convertUnits(SBase &sb, Model &m,
  
   UnitDefinition *siud = UnitDefinition::convertToSI(ud);
   /* catch in case things have gone wrong */
-  //if (siud->getNumUnits() == 0)
-  //  return false;
+  if (ud == NULL || siud == NULL)
+    return false;
 
   /* if we are dealing in substance only then we need to use substance
    * units when calculating the newvalue 
