@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -834,6 +834,44 @@ public:
    */
   bool isEnabled() const;
 
+  /**
+   * Removes the L2 Namespaces. 
+   *
+   * This method should be overridden by all extensions that want to serialize
+   * to an L2 annotation.
+   */
+  virtual void removeL2Namespaces(XMLNamespaces* xmlns)  const;
+
+  /**
+   * adds all L2 Extension namespaces to the namespace list. 
+   * 
+   * This method should be overridden by all extensions that want to serialize
+   * to an L2 annotation.
+   */
+  virtual void addL2Namespaces(XMLNamespaces *xmlns) const;
+
+  /**
+   * Adds the L2 Namespace to the document and enables the extension.
+   *
+   * If the extension supports serialization to SBML L2 Annotations, this 
+   * method should be overrridden, so it will be activated.
+   */
+  virtual void enableL2NamespaceForDocument(SBMLDocument* doc)  const;
+
+  /** 
+   * Determines whether this extension is being used by the given SBMLDocument
+   *
+   * The default implementation returns true. This means that when a document
+   * had this extension enabled, it will not be possible to convert it to L2
+   * as we cannot make sure that the extension can be converted.
+   * 
+   * @param doc the sbml document to test. 
+   * 
+   * @return a boolean indicating whether the extension is actually being used
+   *         byy the document. 
+   */
+  virtual bool isInUse(SBMLDocument *doc) const;
+
 protected:
   /** @cond doxygen-libsbml-internal */
 
