@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -29,10 +29,15 @@
  */
 
 #include <sbml/extension/SBMLExtensionRegistry.h>
+#include <sbml/SBMLDocument.h>
+#include <sbml/extension/SBasePlugin.h>
 #include <algorithm>
 #include <iostream>
 #include <string>
 
+#include "RegisterExtensions.h"
+
+
 #ifdef __cplusplus
 
 #include <sbml/validator/constraints/IdList.h>
@@ -41,6 +46,7 @@ using namespace std;
 
 LIBSBML_CPP_NAMESPACE_BEGIN
 
+bool SBMLExtensionRegistry::registered = false;
 
 /*
  *
@@ -49,9 +55,19 @@ SBMLExtensionRegistry&
 SBMLExtensionRegistry::getInstance()
 {
   static SBMLExtensionRegistry singletonObj;
+  if (!registered)
+  {
+    registered = true;
+    #include "RegisterExtensions.cxx"
+  }
   return singletonObj;
 }
 
+SBMLExtensionRegistry::SBMLExtensionRegistry()
+{
+}
+
+
 SBMLExtensionRegistry::SBMLExtensionRegistry(const SBMLExtensionRegistry& orig)
 {
   if (&orig != NULL)
@@ -275,6 +291,46 @@ SBMLExtensionRegistry::setEnabled(const std::string& uri, bool isEnabled)
   return (sbmlext) ? sbmlext->mIsEnabled = isEnabled : false;
 }
 
+void
+SBMLExtensionRegistry::removeL2Namespaces(XMLNamespaces *xmlns)  const
+{
+  SBMLExtensionMap::const_iterator it = mSBMLExtensionMap.begin();
+  while (it != mSBMLExtensionMap.end())
+  {
+    it->second->removeL2Namespaces(xmlns);
+    it++;
+  }
+}
+
+/**
+ * adds all L2 Extension namespaces to the namespace list. This will call all 
+ * overriden SBMLExtension::addL2Namespaces methods.
+ */
+void
+SBMLExtensionRegistry::addL2Namespaces(XMLNamespaces *xmlns) const
+{
+  SBMLExtensionMap::const_iterator it = mSBMLExtensionMap.begin();
+  while (it != mSBMLExtensionMap.end())
+  {
+    it->second->addL2Namespaces(xmlns);
+    it++;
+  }
+}
+
+/**
+ * Enables all extensions that support serialization / deserialization with
+ * SBML Annotations.
+ */
+void 
+SBMLExtensionRegistry::enableL2NamespaceForDocument(SBMLDocument* doc)  const
+{
+  SBMLExtensionMap::const_iterator it = mSBMLExtensionMap.begin();
+  while (it != mSBMLExtensionMap.end())
+  {
+    it->second->enableL2NamespaceForDocument(doc);
+    it++;
+  }
+}
 
 /*
  * Checks if the extension with the given URI is enabled (true) or disabled (false)
@@ -330,14 +386,19 @@ SBMLExtensionRegistry::getRegisteredPackageName(unsigned int index)
 {
   char * name = (char *) (getRegisteredPackageNames()->get(index));
   return string(name);
-  //if (index > packages->getSize()
-  //while (it != instance.mSBMLExtensionMap.end())
-  //{   
-  //  if (count == index) return (*it).second->getName();
-  //  count++;
-  //  it++;
-  //}
-  //return NULL;
+}
+
+void 
+SBMLExtensionRegistry::disableUnusedPackages(SBMLDocument *doc)
+{
+  for (unsigned int i = doc->getNumPlugins(); i > 0; i--)
+  {
+    SBasePlugin *plugin = doc->getPlugin(i-1);
+    if (plugin == NULL) continue;
+    const SBMLExtension *ext = getExtensionInternal(plugin->getURI());
+    if (!ext->isInUse(doc))
+      doc->disablePackage(plugin->getURI(), plugin->getPrefix());
+  }
 }
 
 
