@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -184,6 +184,28 @@ SBasePlugin::connectToParent (SBase* sbase)
 /** @endcond */
 
 
+
+/**
+ *
+ * (Extension)
+ *
+ * Sets the XML namespace to which this element belogns to.
+ * For example, all elements that belong to SBML Level 3 Version 1 Core
+ * must set the namespace to "http://www.sbml.org/sbml/level3/version1/core";
+ * all elements that belong to Layout Extension Version 1 for SBML Level 3
+ * Version 1 Core must set the namespace to
+ * "http://www.sbml.org/sbml/level3/version1/layout/version1/"
+ *
+ */
+int
+SBasePlugin::setElementNamespace(const std::string &uri)
+{
+//  cout << "[DEBUG] SBasePlugin::setElementNamespace() " << uri << endl;
+  mURI = uri;
+
+  return LIBSBML_OPERATION_SUCCESS;
+}
+
 /*
  * Returns the parent element.
  */
@@ -234,7 +256,7 @@ SBasePlugin::stripPackage(const std::string& pkgPrefix, bool flag)
 unsigned int 
 SBasePlugin::getLevel() const
 {
-  return mSBMLExt->getLevel(mURI);
+  return mSBMLExt->getLevel(getURI());
 }
 
 
@@ -246,10 +268,24 @@ SBasePlugin::getLevel() const
 unsigned int 
 SBasePlugin::getVersion() const
 {
-  return mSBMLExt->getVersion(mURI);
+  return mSBMLExt->getVersion(getURI());
 }
 
 
+unsigned int 
+SBasePlugin::getLine() const
+{
+  if (mParent == NULL) return 0;
+  return mParent->getLine();
+}
+
+unsigned int 
+SBasePlugin::getColumn() const
+{
+  if (mParent == NULL) return 0;
+  return mParent->getColumn();
+}
+
 /*
  * Returns the package version of this plugin object.
  *
@@ -258,7 +294,7 @@ SBasePlugin::getVersion() const
 unsigned int 
 SBasePlugin::getPackageVersion() const
 {
-  return mSBMLExt->getPackageVersion(mURI);
+  return mSBMLExt->getPackageVersion(getURI());
 }
 
 
@@ -303,11 +339,34 @@ SBasePlugin::prependStringToAllIdentifiers(const std::string& prefix)
  * Returns the namespace URI of this element.
  */
 const std::string& 
-SBasePlugin::getURI() const
+SBasePlugin::getElementNamespace() const
 {
   return mURI;  
 }
 
+std::string 
+SBasePlugin::getURI() const
+{
+  const std::string &package = mSBMLExt->getName();
+  const SBMLDocument* doc = getSBMLDocument();
+
+  if (doc == NULL)
+    return getElementNamespace();
+  
+  SBMLNamespaces* sbmlns = doc->getSBMLNamespaces();
+
+  if (sbmlns == NULL)
+    return getElementNamespace();
+
+  if (package == "" || package == "core")
+    return sbmlns->getURI();
+
+  std::string packageURI = sbmlns->getNamespaces()->getURI(package);
+  if (!packageURI.empty())
+    return packageURI;
+
+  return getElementNamespace();
+}
 
 /*
  * Returns the prefix bound to this element.
@@ -315,6 +374,7 @@ SBasePlugin::getURI() const
 const std::string& 
 SBasePlugin::getPrefix() const
 {
+  
   return mPrefix;
 }
 
@@ -371,6 +431,32 @@ SBasePlugin::readOtherXML (SBase* parentObject, XMLInputStream& stream)
 }
 /** @endcond */
 
+/** @cond doxygen-libsbml-internal */
+/**
+ * Synchronizes the annotation of this SBML object.
+ *
+ * Annotation element (XMLNode* mAnnotation) is synchronized with the 
+ * current CVTerm objects (List* mCVTerm).
+ * Currently, this method is called in getAnnotation, isSetAnnotation,
+ * and writeElements methods.
+ */
+void 
+SBasePlugin::syncAnnotation(SBase* parentObject, XMLNode *annotation)
+{
+
+}
+
+/** 
+ * Parse L2 annotation if supported
+ *
+ */
+void 
+SBasePlugin::parseAnnotation(SBase *parentObject, XMLNode *annotation)
+{
+
+}
+
+/** @endcond */
 
 /** @cond doxygen-libsbml-internal */
 /* default for components that have no required elements */
@@ -629,7 +715,7 @@ const char*
 SBasePlugin_getURI(SBasePlugin_t* plugin)
 {
   if (plugin == NULL) return NULL;
-  return plugin->getURI().c_str();
+  return plugin->getElementNamespace().c_str();
 }
 
 /**
