@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -278,7 +278,7 @@ public:
    *
    * @return the URI of the package extension of this plugin object.
    */
-  const std::string& getURI() const;
+  const std::string& getElementNamespace() const;
 
 
   /**
@@ -371,6 +371,23 @@ public:
    */
   virtual bool readOtherXML (SBase* parentObject, XMLInputStream& stream);
 
+  
+  /**
+   * Synchronizes the annotation of this SBML object.
+   *
+   * Annotation element (XMLNode* mAnnotation) is synchronized with the 
+   * current CVTerm objects (List* mCVTerm).
+   * Currently, this method is called in getAnnotation, isSetAnnotation,
+   * and writeElements methods.
+   */
+  virtual void syncAnnotation(SBase* parentObject, XMLNode *annotation);
+
+
+  /** 
+   * Parse L2 annotation if supported
+   *
+   */
+  virtual void parseAnnotation(SBase *parentObject, XMLNode *annotation);
 
   /**
    * Subclasses must override this method to write out their contained
@@ -536,6 +553,27 @@ public:
    */
   const SBMLDocument* getSBMLDocument () const;
 
+  /**
+   * Gets the URI to which this element belongs to.
+   * For example, all elements that belong to SBML Level 3 Version 1 Core
+   * must would have the URI "http://www.sbml.org/sbml/level3/version1/core"; 
+   * all elements that belong to Layout Extension Version 1 for SBML Level 3
+   * Version 1 Core must would have the URI
+   * "http://www.sbml.org/sbml/level3/version1/layout/version1/"
+   *
+   * Unlike getElementNamespace, this function first returns the URI for this 
+   * element by looking into the SBMLNamespaces object of the document with 
+   * the its package name. if not found it will return the result of 
+   * getElementNamespace
+   *
+   * @return the URI this elements  
+   *
+   * @see getPackageName
+   * @see getElementNamespace
+   * @see SBMLDocument::getSBMLNamespaces
+   * @see getSBMLDocument
+   */
+  std::string getURI() const;
 
   /**
    * Returns the parent SBase object to which this plugin 
@@ -556,6 +594,23 @@ public:
    */
   const SBase* getParentSBMLObject () const;
 
+  
+  /**
+   * Sets the XML namespace to which this element belongs to.
+   * For example, all elements that belong to SBML Level 3 Version 1 Core
+   * must set the namespace to "http://www.sbml.org/sbml/level3/version1/core"; 
+   * all elements that belong to Layout Extension Version 1 for SBML Level 3
+   * Version 1 Core must set the namespace to 
+   * "http://www.sbml.org/sbml/level3/version1/layout/version1/"
+   *
+   * @return integer value indicating success/failure of the
+   * function.  @if clike The value is drawn from the
+   * enumeration #OperationReturnValues_t. @endif The possible values
+   * returned by this function are:
+   * @li @link OperationReturnValues_t#LIBSBML_OPERATION_SUCCESS LIBSBML_OPERATION_SUCCESS @endlink
+   * @li @link OperationReturnValues_t#LIBSBML_INVALID_ATTRIBUTE_VALUE LIBSBML_INVALID_ATTRIBUTE_VALUE @endlink
+   */
+  int setElementNamespace(const std::string &uri);
 
   /**
    * Returns the SBML level of the package extension of 
@@ -620,7 +675,59 @@ public:
   virtual int prependStringToAllIdentifiers(const std::string& prefix);
   /** @endcond */
   
+  /** @cond doxygen-libsbml-internal */
+  /**
+   * Returns the line number on which this object first appears in the XML
+   * representation of the SBML document.
+   * 
+   * @return the line number of the underlying SBML object.
+   *
+   * @note The line number for each construct in an SBML model is set upon
+   * reading the model.  The accuracy of the line number depends on the
+   * correctness of the XML representation of the model, and on the
+   * particular XML parser library being used.  The former limitation
+   * relates to the following problem: if the model is actually invalid
+   * XML, then the parser may not be able to interpret the data correctly
+   * and consequently may not be able to establish the real line number.
+   * The latter limitation is simply that different parsers seem to have
+   * their own accuracy limitations, and out of all the parsers supported
+   * by libSBML, none have been 100% accurate in all situations. (At this
+   * time, libSBML supports the use of <a target="_blank"
+   * href="http://xmlsoft.org">libxml2</a>, <a target="_blank"
+   * href="http://expat.sourceforge.net/">Expat</a> and <a target="_blank"
+   * href="http://xerces.apache.org/xerces-c/">Xerces</a>.)
+   *
+   * @see getColumn()
+   */
+  unsigned int getLine() const;
+  /** @endcond */
 
+  /** @cond doxygen-libsbml-internal */
+  /**
+   * Returns the column number on which this object first appears in the XML
+   * representation of the SBML document.
+   * 
+   * @return the column number of the underlying SBML object.
+   * 
+   * @note The column number for each construct in an SBML model is set
+   * upon reading the model.  The accuracy of the column number depends on
+   * the correctness of the XML representation of the model, and on the
+   * particular XML parser library being used.  The former limitation
+   * relates to the following problem: if the model is actually invalid
+   * XML, then the parser may not be able to interpret the data correctly
+   * and consequently may not be able to establish the real column number.
+   * The latter limitation is simply that different parsers seem to have
+   * their own accuracy limitations, and out of all the parsers supported
+   * by libSBML, none have been 100% accurate in all situations. (At this
+   * time, libSBML supports the use of <a target="_blank"
+   * href="http://xmlsoft.org">libxml2</a>, <a target="_blank"
+   * href="http://expat.sourceforge.net/">Expat</a> and <a target="_blank"
+   * href="http://xerces.apache.org/xerces-c/">Xerces</a>.)
+   * 
+   * @see getLine()
+   */
+  unsigned int getColumn() const;
+  /** @endcond */
 
 protected:
   /** @cond doxygen-libsbml-internal */
