@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -36,6 +36,7 @@
 #include <sbml/math/ASTNode.h>
 #include <sbml/xml/XMLAttributes.h>
 #include <sbml/xml/XMLNode.h>
+#include <sbml/Model.h>
 
 /** @cond doxygen-ignored */
 
@@ -48,6 +49,9 @@ LIBSBML_CPP_NAMESPACE_BEGIN
 /**
  * ASTNodeType predicates
  */
+#define ASTNodeType_isAvogadro(t) \
+  (t == AST_NAME_AVOGADRO)
+
 #define ASTNodeType_isConstant(t) \
   (((t >= AST_CONSTANT_E) && (t <= AST_CONSTANT_TRUE)) || t == AST_NAME_AVOGADRO)
 
@@ -196,9 +200,11 @@ ASTNode::ASTNode (ASTNodeType_t type)
 
 
 /*
- * Creates a new ASTNode from the given Token.
- *
- * The ASTNode will contain the same data as the Token.
+ * Creates a new ASTNode from the given Token_t structure.                   
+ *                                                                           
+ * The resulting ASTNode will contain the same data as the Token_t           
+ * object.  Please refer to the documentation for Token_t to learn           
+ * about the possible contents.                                              
  */
 LIBSBML_EXTERN
 ASTNode::ASTNode (Token_t* token)
@@ -1212,6 +1218,18 @@ ASTNode::getUnits() const
   return mUnits;
 }
 
+/*
+ * @return true if this ASTNode is a boolean (a logical operator, a
+ * relational operator, or the constants true or false), false otherwise.
+ */
+LIBSBML_EXTERN
+bool
+ASTNode::isAvogadro () const
+{
+  return ASTNodeType_isAvogadro(mType);
+}
+
+
 /*
  * @return true if this ASTNode is a boolean (a logical operator, a
  * relational operator, or the constants true or false), false otherwise.
@@ -1228,6 +1246,61 @@ ASTNode::isBoolean () const
 }
 
 
+LIBSBML_EXTERN
+bool
+ASTNode::returnsBoolean (const Model* givenModel /*=NULL*/) const
+{   
+
+  if (isBoolean() == true)
+  {
+    return true;
+  }
+
+  const Model* model = givenModel;
+  if (givenModel == NULL && getParentSBMLObject() != NULL)
+  {
+    model = getParentSBMLObject()->getModel();
+  }
+
+  if (getType() == AST_FUNCTION)
+  {
+    if (model == NULL)
+    {
+      return false;
+    }
+    else
+    {
+      const FunctionDefinition* fd = model->getFunctionDefinition( getName() );
+
+      if (fd != NULL && fd->isSetMath())
+      {
+        return fd->getMath()->getRightChild()->returnsBoolean();
+      }
+      else
+      {
+        return false;
+      }
+    }
+  }
+
+  else if (getType() == AST_FUNCTION_PIECEWISE)
+  {
+    for (unsigned int c = 0; c < getNumChildren(); c += 2)
+    {
+      if ( getChild(c)->returnsBoolean() == false ) 
+        return false;
+    }
+
+    return true;
+  }
+
+  // add explicit return value in case we overlooked something
+  return false;
+}
+
+
+
+
 /*
  * @return true if this ASTNode is a MathML constant (true,
  * false, pi, exponentiale), false otherwise.
@@ -2103,6 +2176,9 @@ ASTNode::hasCorrectNumberArguments() const
     case AST_LOGICAL_AND:
     case AST_LOGICAL_OR:
     case AST_LOGICAL_XOR:
+      correctNum = true;
+      break;
+
     case AST_RELATIONAL_EQ:
     case AST_RELATIONAL_GEQ:
     case AST_RELATIONAL_GT:
@@ -2772,6 +2848,19 @@ ASTNode_getUnits(const ASTNode_t * node)
 }
 
 
+/**
+ * @return true (non-zero) if this ASTNode is the special symbol
+ * avogadro, false (0) otherwise.
+ */
+LIBSBML_EXTERN
+int
+ASTNode_isAvogadro (const ASTNode_t *node)
+{
+  if (node == NULL) return (int) false;
+  return (int) static_cast<const ASTNode*>(node)->isAvogadro();
+}
+
+
 /**
  * @return true (non-zero) if this ASTNode is a boolean (a logical
  * operator, a relational operator, or the constants true or false), false
@@ -2786,6 +2875,27 @@ ASTNode_isBoolean (const ASTNode_t *node)
 }
 
 
+LIBSBML_EXTERN
+int
+ASTNode_returnsBoolean (const ASTNode_t *node)
+{
+  if (node == NULL) return (int) false;
+  return (int) static_cast<const ASTNode*>(node)->returnsBoolean();
+}
+
+/**
+ * @return true (non-zero) if this ASTNode returns a boolean, false (0) otherwise.
+ */
+LIBSBML_EXTERN
+int
+ASTNode_returnsBooleanForModel (const ASTNode_t *node, const Model_t* model)
+{
+  if (node == NULL) return (int) false;
+  return (int) static_cast<const ASTNode*>(node)->returnsBoolean(model);
+}
+
+
+
 /**
  * @return true (non-zero) if this ASTNode is a MathML constant (true,
  * false, pi, exponentiale), false (0) otherwise.
