@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -956,6 +956,16 @@ public:
   std::string getUnits () const;
 
 
+  /**
+   * Predicate returning @c true (non-zero) if this node is the special 
+   * symbol @c avogadro.  The predicate returns @c false (zero) otherwise.
+   * 
+   * @return @c true if this ASTNode is the special symbol avogadro.
+   */
+  LIBSBML_EXTERN
+  bool isAvogadro () const;
+
+
   /**
    * Predicate returning @c true (non-zero) if this node has a boolean type
    * (a logical operator, a relational operator, or the constants @c true
@@ -967,6 +977,26 @@ public:
   bool isBoolean () const;
 
 
+  /**
+   * Predicate returning @c true (non-zero) if this node returns a boolean type
+   * or @c false (zero) otherwise.
+   *
+   * This function looks at the whole ASTNode rather than just the top 
+   * level of the ASTNode. Thus it will consider return values from
+   * piecewise statements.  In addition, if this ASTNode uses a function
+   * call, the return value of the functionDefinition will be determined.
+   * Note that this is only possible where the ASTNode can trace its parent
+   * Model, that is, the ASTNode must represent the math element of some
+   * SBML object that has already been added to an instance of an SBMLDocument.
+   *
+   * @see isBoolean()
+   *
+   * @return true if this ASTNode returns a boolean, false otherwise.
+   */
+  LIBSBML_EXTERN
+  bool returnsBoolean (const Model* model=NULL) const;
+
+
   /**
    * Predicate returning @c true (non-zero) if this node represents a MathML
    * constant (e.g., @c true, @c Pi).
@@ -2085,6 +2115,11 @@ const char *
 ASTNode_getUnits(const ASTNode_t * node);
 
 
+LIBSBML_EXTERN
+int
+ASTNode_isAvogadro (const ASTNode_t * node);
+
+
 /**
  * @return true (non-zero) if this ASTNode is a boolean (a logical
  * operator, a relational operator, or the constants true or false), false
@@ -2092,7 +2127,23 @@ ASTNode_getUnits(const ASTNode_t * node);
  */
 LIBSBML_EXTERN
 int
-ASTNode_isBoolean (const ASTNode_t *node);
+ASTNode_isBoolean (const ASTNode_t * node);
+
+/**
+ * @return true (non-zero) if this ASTNode returns a boolean, false (0) otherwise.
+ */
+LIBSBML_EXTERN
+int
+ASTNode_returnsBoolean (const ASTNode_t *node);
+
+
+/**
+ * @return true (non-zero) if this ASTNode returns a boolean, false (0) otherwise.
+ */
+LIBSBML_EXTERN
+int
+ASTNode_returnsBooleanForModel (const ASTNode_t *node, const Model_t* model);
+
 
 /**
  * @return true (non-zero) if this ASTNode is a MathML constant (true,
@@ -2100,7 +2151,7 @@ ASTNode_isBoolean (const ASTNode_t *node);
  */
 LIBSBML_EXTERN
 int
-ASTNode_isConstant (const ASTNode_t *node);
+ASTNode_isConstant (const ASTNode_t * node);
 
 /**
  * @return true (non-zero) if this ASTNode is a function in SBML L1, L2
@@ -2109,7 +2160,7 @@ ASTNode_isConstant (const ASTNode_t *node);
  */
 LIBSBML_EXTERN
 int
-ASTNode_isFunction (const ASTNode_t *node);
+ASTNode_isFunction (const ASTNode_t * node);
 
 /**
  * @return true if this ASTNode is the special IEEE 754 value infinity,
