@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -381,12 +381,24 @@ setTypeCI (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
     string url;
     element.getAttributes().readInto("definitionURL", url);
 
-    if ( url == URL_DELAY ) node.setType(AST_FUNCTION_DELAY);
-    else if ( url == URL_TIME  ) node.setType(AST_NAME_TIME);
-    else if ( url == URL_AVOGADRO  ) node.setType(AST_NAME_AVOGADRO);
-    else 
+    if (stream.getSBMLNamespaces()->getLevel() < 3)
     {
-      logError(stream, element, BadCsymbolDefinitionURLValue);      
+      if ( url == URL_DELAY ) node.setType(AST_FUNCTION_DELAY);
+      else if ( url == URL_TIME  ) node.setType(AST_NAME_TIME);
+      else 
+      {
+        logError(stream, element, BadCsymbolDefinitionURLValue);      
+      }
+    }
+    else
+    {
+      if ( url == URL_DELAY ) node.setType(AST_FUNCTION_DELAY);
+      else if ( url == URL_TIME  ) node.setType(AST_NAME_TIME);
+      else if ( url == URL_AVOGADRO  ) node.setType(AST_NAME_AVOGADRO);
+      else 
+      {
+        logError(stream, element, BadCsymbolDefinitionURLValue);      
+      }
     }
   }
   else if (element.getName() == "ci")
@@ -791,12 +803,13 @@ readMathML (ASTNode& node, XMLInputStream& stream, std::string reqd_prefix)
     else
     {
       /* catch case where there is no otherwise
+       * BUT do not return if we are dealing with <piecewise/>
        */
-      if (elem.isEnd()) 
-      {
-        //node = NULL;
-        return;
-      }
+      //if (elem.isEnd()) 
+      //{
+      //  //node = NULL;
+      //  return;
+      //}
       node.setType(AST_FUNCTION_PIECEWISE);
     }
 
@@ -1301,8 +1314,18 @@ writeLambda (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmln
 {
   if (&node == NULL || &stream == NULL) return;
 
+  bool bodyPresent = true;
   unsigned int bvars = node.getNumChildren() - 1;
-  unsigned int n;
+  unsigned int n = bvars;
+
+  /* look for the case where the element is missing a body- 
+   * not valid I know but it messes up roundtripping
+   */
+  if (node.getChild(n)->isBvar() == true)
+  {
+    bvars = bvars + 1;
+    bodyPresent = false;
+  }
 
   stream.startElement("lambda");
 
@@ -1312,8 +1335,10 @@ writeLambda (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmln
     writeNode(*node.getChild(n), stream, sbmlns);
     stream.endElement("bvar");
   }
-
-  writeNode( *node.getChild(n), stream, sbmlns);
+  if (bodyPresent == true)
+  {
+    writeNode( *node.getChild(n), stream, sbmlns);
+  }
 
   stream.endElement("lambda");
 }
