@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -31,6 +31,8 @@
 #include <sbml/math/ASTNode.h>
 #include <sbml/math/FormulaParser.h>
 #include <sbml/EventAssignment.h>
+#include <sbml/Model.h>
+#include <sbml/SBMLDocument.h>
 #include <sbml/xml/XMLNode.h>
 
 #include <limits.h>
@@ -2258,6 +2260,7 @@ START_TEST (test_ASTNode_accessWithNULL)
   fail_unless( ASTNode_hasUnits (NULL) == 0);
   fail_unless( ASTNode_insertChild (NULL, 0, NULL) == LIBSBML_INVALID_OBJECT);
   fail_unless( ASTNode_isBoolean (NULL) == 0);
+  fail_unless( ASTNode_returnsBoolean (NULL) == 0);
   fail_unless( ASTNode_isConstant (NULL) == 0);
   fail_unless( ASTNode_isFunction (NULL) == 0);
   fail_unless( ASTNode_isInfinity (NULL) == 0);
@@ -2302,6 +2305,470 @@ START_TEST (test_ASTNode_accessWithNULL)
 }
 END_TEST
 
+START_TEST (test_ASTNode_isBoolean)
+{
+  ASTNode_t *n;// = ASTNode_createWithType(AST_FUNCTION);
+  n = ASTNode_createWithType(AST_PLUS);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_MINUS);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_TIMES);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_DIVIDE);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_POWER);
+  fail_unless(! ASTNode_isBoolean(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_INTEGER);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_REAL);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_REAL_E);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RATIONAL);
+  fail_unless(! ASTNode_isBoolean(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_NAME);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_NAME_AVOGADRO);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_NAME_TIME);
+  fail_unless(! ASTNode_isBoolean(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_CONSTANT_E);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_CONSTANT_FALSE);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_CONSTANT_PI);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_CONSTANT_TRUE);
+  fail_unless( ASTNode_isBoolean(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LAMBDA);
+  fail_unless(! ASTNode_isBoolean(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ABS);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCOS);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCOSH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCOT);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCOTH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCSC);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCSCH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCSEC);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCSECH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCSIN);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCSINH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCTAN);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCTANH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_CEILING);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_COS);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_COSH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_COT);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_COTH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_CSC);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_CSCH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_DELAY);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_EXP);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_FACTORIAL);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_FLOOR);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_LN);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_LOG);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_PIECEWISE);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_POWER);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ROOT);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_SEC);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_SECH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_SIN);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_SINH);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_TAN);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_TANH);
+  fail_unless(! ASTNode_isBoolean(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LOGICAL_AND);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LOGICAL_NOT);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LOGICAL_OR);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LOGICAL_XOR);
+  fail_unless( ASTNode_isBoolean(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_EQ);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_GEQ);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_GT);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_LEQ);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_LT);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_NEQ);
+  fail_unless( ASTNode_isBoolean(n));
+  ASTNode_free(n);
+
+  n = ASTNode_createWithType(AST_UNKNOWN);
+  fail_unless(! ASTNode_isBoolean(n));
+  ASTNode_free(n);
+}
+END_TEST
+
+START_TEST (test_ASTNode_returnsBoolean)
+{
+
+  ASTNode_t *n = ASTNode_create();
+  const ASTNode_t *math;
+
+  // boolean function
+  n = SBML_parseFormula("geq(a,b)");
+  fail_unless(ASTNode_returnsBoolean(n) == 1);
+
+  // not boolean function
+  n = SBML_parseFormula("times(a,b)");
+  fail_unless(ASTNode_returnsBoolean(n) == 0);
+
+  // piecewise with bool
+  n = SBML_parseFormula("piecewise(true, geq(X, T), false)");
+  fail_unless(ASTNode_returnsBoolean(n) == 1);
+
+  // piecewise no boolean
+  n = SBML_parseFormula("piecewise(true, geq(X, T), 5)");
+  fail_unless(ASTNode_returnsBoolean(n) == 0);
+
+  // func with no model
+  n = SBML_parseFormula("func1(X)");
+  fail_unless(ASTNode_returnsBoolean(n) == 0);
+
+  // func with model that does not contain that func
+  SBMLDocument_t *doc = SBMLDocument_createWithLevelAndVersion(3,1);
+  Model_t* model = SBMLDocument_createModel(doc);
+  Constraint_t *c = Model_createConstraint(model);
+  Constraint_setMath(c, n);
+
+  math = Constraint_getMath(c);
+  fail_unless(ASTNode_returnsBoolean(math) == 0);
+
+  // func with model but func has no math
+  FunctionDefinition_t* fd = Model_createFunctionDefinition(model);
+  FunctionDefinition_setId(fd, "func1");
+  fail_unless(ASTNode_returnsBoolean(math) == 0);
+
+  // func with model func returns boolean
+  FunctionDefinition_setMath(fd, SBML_parseFormula("lambda(x, true)"));
+  fail_unless(ASTNode_returnsBoolean(math) == 1);
+
+  // func with model func returns number
+  FunctionDefinition_setMath(fd, SBML_parseFormula("lambda(x, 6)"));
+  fail_unless(ASTNode_returnsBoolean(math) == 0);
+  ASTNode_free(n);
+  SBMLDocument_free(doc);
+}
+END_TEST
+
+START_TEST (test_ASTNode_isAvogadro)
+{
+  ASTNode_t *n;// = ASTNode_createWithType(AST_FUNCTION);
+  n = ASTNode_createWithType(AST_PLUS);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_MINUS);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_TIMES);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_DIVIDE);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_POWER);
+  fail_unless(! ASTNode_isAvogadro(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_INTEGER);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_REAL);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_REAL_E);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RATIONAL);
+  fail_unless(! ASTNode_isAvogadro(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_NAME);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_NAME_AVOGADRO);
+  fail_unless( ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_NAME_TIME);
+  fail_unless(! ASTNode_isAvogadro(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_CONSTANT_E);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_CONSTANT_FALSE);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_CONSTANT_PI);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_CONSTANT_TRUE);
+  fail_unless( ! ASTNode_isAvogadro(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LAMBDA);
+  fail_unless(! ASTNode_isAvogadro(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ABS);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCOS);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCOSH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCOT);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCOTH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCSC);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCCSCH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCSEC);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCSECH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCSIN);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCSINH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCTAN);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ARCTANH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_CEILING);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_COS);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_COSH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_COT);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_COTH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_CSC);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_CSCH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_DELAY);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_EXP);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_FACTORIAL);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_FLOOR);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_LN);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_LOG);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_PIECEWISE);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_POWER);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_ROOT);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_SEC);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_SECH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_SIN);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_SINH);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_TAN);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_FUNCTION_TANH);
+  fail_unless(! ASTNode_isAvogadro(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LOGICAL_AND);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LOGICAL_NOT);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LOGICAL_OR);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_LOGICAL_XOR);
+  fail_unless( !ASTNode_isAvogadro(n));
+
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_EQ);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_GEQ);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_GT);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_LEQ);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_LT);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+  n = ASTNode_createWithType(AST_RELATIONAL_NEQ);
+  fail_unless( !ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+
+  n = ASTNode_createWithType(AST_UNKNOWN);
+  fail_unless(! ASTNode_isAvogadro(n));
+  ASTNode_free(n);
+}
+END_TEST
+
 Suite *
 create_suite_ASTNode (void) 
 { 
@@ -2354,6 +2821,9 @@ create_suite_ASTNode (void)
   tcase_add_test( tcase, test_ASTNode_avogadro                );
   tcase_add_test( tcase, test_ASTNode_avogadro_bug            );
   tcase_add_test( tcase, test_ASTNode_accessWithNULL          );
+  tcase_add_test( tcase, test_ASTNode_isBoolean               );
+  tcase_add_test( tcase, test_ASTNode_returnsBoolean          );
+  tcase_add_test( tcase, test_ASTNode_isAvogadro              );
 
   suite_add_tcase(suite, tcase);
 
