@@ -163,6 +163,15 @@ Layout::Layout(const XMLNode& node, unsigned int l2version)
   ,mTextGlyphs(2,l2version)
   ,mAdditionalGraphicalObjects(2,l2version)
 {
+
+  LayoutPkgNamespaces *layoutNS = new LayoutPkgNamespaces(2,l2version);
+
+  setSBMLNamespacesAndOwn(layoutNS);  
+    
+  // load plugins
+  loadPlugins(mSBMLNamespaces);
+
+
     const XMLAttributes& attributes=node.getAttributes();
     const XMLNode* child;
     //ExpectedAttributes ea(getElementName());
@@ -334,10 +343,10 @@ Layout::Layout(const XMLNode& node, unsigned int l2version)
         ++n;
     }
 
-  setSBMLNamespacesAndOwn(new LayoutPkgNamespaces(2,l2version));  
 
   // connect child elements to this element.
   connectToChild();
+
 }
 
 /**
@@ -909,7 +918,7 @@ Layout::getCompartmentGlyph (const std::string& id) const
 
 /**
  * Returns the species glyph that has the given id, or NULL if no
- * compartment glyph has the id.
+ * species glyph has the id.
  */
 const SpeciesGlyph*
 Layout::getSpeciesGlyph (const std::string& id) const
@@ -919,7 +928,7 @@ Layout::getSpeciesGlyph (const std::string& id) const
 
 /**
  * Returns the species glyph that has the given id, or NULL if no
- * compartment glyph has the id.
+ * species glyph has the id.
  */
 SpeciesGlyph*
 Layout::getSpeciesGlyph (const std::string& id) 
@@ -930,7 +939,7 @@ Layout::getSpeciesGlyph (const std::string& id)
 
 /**
  * Returns the reaction glyph that has the given id, or NULL if no
- * compartment glyph has the id.
+ * reaction glyph has the id.
  */
 const ReactionGlyph*
 Layout::getReactionGlyph (const std::string& id) const
@@ -940,7 +949,7 @@ Layout::getReactionGlyph (const std::string& id) const
 
 /**
  * Returns the reaction glyph that has the given id, or NULL if no
- * compartment glyph has the id.
+ * reaction glyph has the id.
  */
 ReactionGlyph*
 Layout::getReactionGlyph (const std::string& id) 
@@ -1303,7 +1312,7 @@ Layout::readAttributes (const XMLAttributes& attributes,
   const unsigned int sbmlLevel   = getLevel  ();
   const unsigned int sbmlVersion = getVersion();
 
-  bool assigned = attributes.readInto("id", mId, getErrorLog(), true);
+  bool assigned = attributes.readInto("id", mId, getErrorLog(), true, getLine(), getColumn());
   if (assigned && mId.empty())
   {
     logEmptyString(mId, sbmlLevel, sbmlVersion, "<layout>");
@@ -1379,7 +1388,14 @@ XMLNode Layout::toXML() const
   XMLNode layout_node(layout_token);
   // add the notes and annotations
   if(this->mNotes) layout_node.addChild(*this->mNotes);
-  if(this->mAnnotation) layout_node.addChild(*this->mAnnotation);
+  
+  Layout *self = const_cast<Layout*>(this);
+  XMLNode *annot = self->getAnnotation();
+  if(annot)
+  {
+    layout_node.addChild(*annot);
+  }
+  
   // add the dimensions
   layout_node.addChild(this->mDimensions.toXML());
   // add the list of compartment glyphs
@@ -1421,7 +1437,7 @@ Layout::getTypeCode () const
 }
 
 
-SBase*
+Layout*
 Layout::clone() const
 {
     return new Layout(*this);
@@ -1510,6 +1526,9 @@ ListOfLayouts::ListOfLayouts(LayoutPkgNamespaces* layoutns)
   // set the element namespace of this object
   //
   setElementNamespace(layoutns->getURI());
+
+  loadPlugins(layoutns);
+
 }
 
 
@@ -1520,13 +1539,15 @@ ListOfLayouts::ListOfLayouts(unsigned int level, unsigned int version, unsigned
  : ListOf(level,version)
 {
   setSBMLNamespacesAndOwn(new LayoutPkgNamespaces(level,version,pkgVersion));
+
+  
 };
 
 
 /**
  * @return a (deep) copy of this ListOfUnitDefinitions.
  */
-SBase*
+ListOfLayouts*
 ListOfLayouts::clone () const
 {
   return new ListOfLayouts(*this);
@@ -1641,7 +1662,19 @@ ListOfLayouts::createObject (XMLInputStream& stream)
 
   if (name == "layout")
   {
-    object = new Layout(static_cast<LayoutPkgNamespaces*>(mSBMLNamespaces));
+    SBMLNamespaces* sbmlns = this->getSBMLNamespaces();
+    XMLNamespaces* xmlns = sbmlns->getNamespaces();
+    LayoutPkgNamespaces* layoutns1 = dynamic_cast<LayoutPkgNamespaces*>(sbmlns);
+    if (layoutns1 == NULL)
+    {
+       layoutns1 = new LayoutPkgNamespaces(sbmlns->getLevel(), sbmlns->getVersion());
+       for (int i = 0; i < xmlns->getNumNamespaces(); i++)
+       {         
+         if (!layoutns1->getNamespaces()->hasURI(xmlns->getURI(i)))
+           layoutns1->getNamespaces()->add(xmlns->getURI(i), xmlns->getPrefix(i));
+       }
+    }
+    object = new Layout(layoutns1);
     appendAndOwn(object);
     //mItems.push_back(object);
   }
@@ -1689,9 +1722,12 @@ XMLNode ListOfLayouts::toXML() const
       node.addChild(*this->mNotes);
       end=false;
   }
-  if(this->mAnnotation)
+  ListOfLayouts *self = const_cast<ListOfLayouts*>(this);
+  XMLNode *annot = self->getAnnotation();
+  if(annot)
   {
-      node.addChild(*this->mAnnotation);
+    
+    node.addChild(*annot);
       end=false;
   }
   unsigned int i,iMax=this->size();
@@ -1733,7 +1769,7 @@ ListOfCompartmentGlyphs::ListOfCompartmentGlyphs(unsigned int level, unsigned in
 /**
  * @return a (deep) copy of this ListOfUnitDefinitions.
  */
-SBase*
+ListOfCompartmentGlyphs*
 ListOfCompartmentGlyphs::clone () const
 {
   return new ListOfCompartmentGlyphs(*this);
@@ -1914,7 +1950,7 @@ ListOfSpeciesGlyphs::ListOfSpeciesGlyphs(unsigned int level, unsigned int versio
 /**
  * @return a (deep) copy of this ListOfUnitDefinitions.
  */
-SBase*
+ListOfSpeciesGlyphs*
 ListOfSpeciesGlyphs::clone () const
 {
   return new ListOfSpeciesGlyphs(*this);
@@ -2092,7 +2128,7 @@ ListOfReactionGlyphs::ListOfReactionGlyphs(unsigned int level, unsigned int vers
 /**
  * @return a (deep) copy of this ListOfUnitDefinitions.
  */
-SBase*
+ListOfReactionGlyphs*
 ListOfReactionGlyphs::clone () const
 {
   return new ListOfReactionGlyphs(*this);
@@ -2270,7 +2306,7 @@ ListOfTextGlyphs::ListOfTextGlyphs(unsigned int level, unsigned int version, uns
 /**
  * @return a (deep) copy of this ListOfUnitDefinitions.
  */
-SBase*
+ListOfTextGlyphs*
 ListOfTextGlyphs::clone () const
 {
   return new ListOfTextGlyphs(*this);
@@ -2448,7 +2484,7 @@ ListOfGraphicalObjects::ListOfGraphicalObjects(unsigned int level, unsigned int
 /**
  * @return a (deep) copy of this ListOfUnitDefinitions.
  */
-SBase*
+ListOfGraphicalObjects*
 ListOfGraphicalObjects::clone () const
 {
   return new ListOfGraphicalObjects(*this);
@@ -2554,7 +2590,20 @@ ListOfGraphicalObjects::createObject (XMLInputStream& stream)
 
   if (name == "graphicalObject")
   {
-    object = new GraphicalObject();
+    SBMLNamespaces* sbmlns = this->getSBMLNamespaces();
+    XMLNamespaces* xmlns = sbmlns->getNamespaces();
+    LayoutPkgNamespaces* layoutns1 = dynamic_cast<LayoutPkgNamespaces*>(sbmlns);
+    if (layoutns1 == NULL)
+    {
+       layoutns1 = new LayoutPkgNamespaces(sbmlns->getLevel(), sbmlns->getVersion());
+       for (int i = 0; i < xmlns->getNumNamespaces(); i++)
+       {         
+         if (!layoutns1->getNamespaces()->hasURI(xmlns->getURI(i)))
+           layoutns1->getNamespaces()->add(xmlns->getURI(i), xmlns->getPrefix(i));
+       }
+    }
+
+    object = new GraphicalObject(layoutns1);
     appendAndOwn(object);
 //    mItems.push_back(object);
   }
