@@ -556,7 +556,7 @@ const std::string& ReactionGlyph::getElementName () const
 /**
  * @return a (deep) copy of this Model.
  */
-SBase* 
+ReactionGlyph* 
 ReactionGlyph::clone () const
 {
     return new ReactionGlyph(*this);
@@ -619,7 +619,7 @@ void ReactionGlyph::readAttributes (const XMLAttributes& attributes,
   const unsigned int sbmlLevel   = getLevel  ();
   const unsigned int sbmlVersion = getVersion();
 
-  bool assigned = attributes.readInto("reaction", mReaction);
+  bool assigned = attributes.readInto("reaction", mReaction, getErrorLog(), false, getLine(), getColumn());
   if (assigned && mReaction.empty())
   {
     logEmptyString(mReaction, sbmlLevel, sbmlVersion, "<" + getElementName() + ">");
@@ -759,7 +759,7 @@ ListOfSpeciesReferenceGlyphs::ListOfSpeciesReferenceGlyphs(LayoutPkgNamespaces*
 /**
  * @return a (deep) copy of this ListOfUnitDefinitions.
  */
-SBase*
+ListOfSpeciesReferenceGlyphs*
 ListOfSpeciesReferenceGlyphs::clone () const
 {
   return new ListOfSpeciesReferenceGlyphs(*this);
