@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -278,7 +278,7 @@ SBMLInternalValidator::setConsistencyChecksForConversion(SBMLErrorCategory_t cat
  * @return the number of failed checks (errors) encountered.
  */
 unsigned int
-SBMLInternalValidator::checkConsistency ()
+SBMLInternalValidator::checkConsistency (bool writeDocument)
 {
   unsigned int nerrors = 0;
   unsigned int total_errors = 0;
@@ -312,60 +312,129 @@ SBMLInternalValidator::checkConsistency ()
   OverdeterminedValidator over_validator;
   ModelingPracticeValidator practice_validator;
 
+  SBMLDocument *doc;
+  SBMLErrorLog *log = getErrorLog();
+  
+  if (writeDocument)
+  {
+    char* sbmlString = writeSBMLToString(getDocument());
+    log->clearLog();
+    doc = readSBMLFromString(sbmlString);
+    free (sbmlString);  
+  }
+  else
+  {
+    doc = getDocument();
+  }
+
   /* calls each specified validator in turn 
    * - stopping when errors are encountered */
 
   if (id)
   {
     id_validator.init();
-    nerrors = id_validator.validate(*getDocument());
-    total_errors += nerrors;
+    nerrors = id_validator.validate(*doc);
     if (nerrors > 0) 
     {
-      getErrorLog()->add( id_validator.getFailures() );
-      return total_errors;
+      unsigned int origNum = log->getNumErrors();
+      log->add( id_validator.getFailures() );
+
+      if (origNum > 0 && log->contains(InvalidUnitIdSyntax) == true)
+      {
+        /* do not log dangling ref */
+        while (log->contains(DanglingUnitSIdRef) == true)
+        {
+          log->remove(DanglingUnitSIdRef);
+          nerrors--;
+        }
+        
+        total_errors += nerrors;
+        if (nerrors > 0)
+        {
+          if (writeDocument)
+            SBMLDocument_free(doc);
+          return total_errors;
+        }
+      }
+      else if (log->contains(DanglingUnitSIdRef) == false)
+      {
+        total_errors += nerrors;
+        if (writeDocument)
+          SBMLDocument_free(doc);
+        return total_errors;
+      }
+      else
+      {
+        bool onlyDangRef = true;
+        for (unsigned int a = 0; a < log->getNumErrors(); a++)
+        {
+          if (log->getError(a)->getErrorId() != DanglingUnitSIdRef)
+          {
+            onlyDangRef = false;
+            break;
+          }
+        }
+        total_errors += nerrors;
+
+        if (onlyDangRef == false)
+        {
+          if (writeDocument)
+            SBMLDocument_free(doc);
+          return total_errors;
+        }
+      }
     }
   }
 
   if (sbml)
   {
     validator.init();
-    nerrors = validator.validate(*getDocument());
+    nerrors = validator.validate(*doc);
     total_errors += nerrors;
     if (nerrors > 0) 
     {
-      getErrorLog()->add( validator.getFailures() );
+      log->add( validator.getFailures() );
       /* only want to bail if errors not warnings */
-      if (getErrorLog()->getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
+      if (log->getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
+      {
+        if (writeDocument)
+          SBMLDocument_free(doc);
         return total_errors;
+      }
     }
   }
 
   if (sbo)
   {
     sbo_validator.init();
-    nerrors = sbo_validator.validate(*getDocument());
+    nerrors = sbo_validator.validate(*doc);
     total_errors += nerrors;
     if (nerrors > 0) 
     {
-      getErrorLog()->add( sbo_validator.getFailures() );
+      log->add( sbo_validator.getFailures() );
       /* only want to bail if errors not warnings */
-      if (getErrorLog()->getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
+      if (log->getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
+      {
+        if (writeDocument)
+          SBMLDocument_free(doc);
         return total_errors;
+      }
     }
   }
 
   if (math)
   {
     math_validator.init();
-    nerrors = math_validator.validate(*getDocument());
+    nerrors = math_validator.validate(*doc);
     total_errors += nerrors;
     if (nerrors > 0) 
     {
-      getErrorLog()->add( math_validator.getFailures() );
+      log->add( math_validator.getFailures() );
       /* at this point bail if any problems
        * unit checks may crash if there have been math errors/warnings
        */
+      if (writeDocument)
+        SBMLDocument_free(doc);
       return total_errors;
     }
   }
@@ -374,14 +443,18 @@ SBMLInternalValidator::checkConsistency ()
   if (units)
   {
     unit_validator.init();
-    nerrors = unit_validator.validate(*getDocument());
+    nerrors = unit_validator.validate(*doc);
     total_errors += nerrors;
     if (nerrors > 0) 
     {
-      getErrorLog()->add( unit_validator.getFailures() );
+      log->add( unit_validator.getFailures() );
       /* only want to bail if errors not warnings */
-      if (getErrorLog()->getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
+      if (log->getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
+      {
+        if (writeDocument)
+          SBMLDocument_free(doc);
         return total_errors;
+      }
     }
   }
 
@@ -390,21 +463,25 @@ SBMLInternalValidator::checkConsistency ()
   if (over)
   {
     over_validator.init();
-    nerrors = over_validator.validate(*getDocument());
+    nerrors = over_validator.validate(*doc);
     total_errors += nerrors;
     if (nerrors > 0) 
     {
-      getErrorLog()->add( over_validator.getFailures() );
+      log->add( over_validator.getFailures() );
       /* only want to bail if errors not warnings */
-      if (getErrorLog()->getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
+      if (log->getNumFailsWithSeverity(LIBSBML_SEV_ERROR) > 0)
+      {
+        if (writeDocument)
+          SBMLDocument_free(doc);
         return total_errors;
+      }
     }
   }
 
   if (practice)
   {
     practice_validator.init();
-    nerrors = practice_validator.validate(*getDocument());
+    nerrors = practice_validator.validate(*doc);
     if (nerrors > 0) 
     {
       unsigned int errorsAdded = 0;
@@ -415,14 +492,14 @@ SBMLInternalValidator::checkConsistency ()
       {
         if (SBMLError(*iter).getErrorId() != 80701)
         {
-          getErrorLog()->add( SBMLError(*iter) );
+          log->add( SBMLError(*iter) );
           errorsAdded++;
         }
         else
         {
           if (units) 
           {
-            getErrorLog()->add( SBMLError(*iter) );
+            log->add( SBMLError(*iter) );
             errorsAdded++;
           }
         }
@@ -432,7 +509,8 @@ SBMLInternalValidator::checkConsistency ()
     }
   }
 
-
+  if (writeDocument)
+    SBMLDocument_free(doc);
   return total_errors;
 }
 
