@@ -1,31 +1,55 @@
 /**
-* @file    SBMLInternalValidator.h
-* @brief   Definition of SBMLInternalValidator, the validator for all internal validation performed by libSBML.
-* @author  Frank Bergmann
-* 
-* <!--------------------------------------------------------------------------
-* This file is part of libSBML.  Please visit http://sbml.org for more
-* information about SBML, and the latest version of libSBML.
-*
-* Copyright (C) 2009-2011 jointly by the following organizations: 
-*     1. California Institute of Technology, Pasadena, CA, USA
-*     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
-*  
-* Copyright (C) 2006-2008 by the California Institute of Technology,
-*     Pasadena, CA, USA 
-*  
-* Copyright (C) 2002-2005 jointly by the following organizations: 
-*     1. California Institute of Technology, Pasadena, CA, USA
-*     2. Japan Science and Technology Agency, Japan
-* 
-* This library is free software; you can redistribute it and/or modify it
-* under the terms of the GNU Lesser General Public License as published by
-* the Free Software Foundation.  A copy of the license agreement is provided
-* in the file named "LICENSE.txt" included with this software distribution
-* and also available online as http://sbml.org/software/libsbml/license.html
-* ------------------------------------------------------------------------ -->
-*/
-
+ * @file    SBMLInternalValidator.h
+ * @brief   Definition of SBMLInternalValidator, the validator for all internal validation performed by libSBML.
+ * @author  Frank Bergmann
+ * 
+ * <!--------------------------------------------------------------------------
+ * This file is part of libSBML.  Please visit http://sbml.org for more
+ * information about SBML, and the latest version of libSBML.
+ *
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
+ *  
+ * Copyright (C) 2006-2008 by the California Institute of Technology,
+ *     Pasadena, CA, USA 
+ *  
+ * Copyright (C) 2002-2005 jointly by the following organizations: 
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. Japan Science and Technology Agency, Japan
+ * 
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.  A copy of the license agreement is provided
+ * in the file named "LICENSE.txt" included with this software distribution
+ * and also available online as http://sbml.org/software/libsbml/license.html
+ * ------------------------------------------------------------------------ -->
+ *
+ * @class SBMLInternalValidator
+ * @brief Implementation of basic SBML consistency checks and other validations.
+ *
+ * @htmlinclude not-sbml-warning.html
+ *
+ * LibSBML implements facilities for verifying that a given SBML document is
+ * valid according to the SBML specifications; it also exposes the validation
+ * interface so that user programs and SBML Level&nbsp;3 package authors may
+ * use the facilities to implement new validators.  The entry point for this
+ * is the SBMLValidator class.
+ *
+ * The subclass SBMLInternalValidator embodies the implementation of the
+ * consistency-checking methods defined on SBMLDocument.  The methods
+ * SBMLDocument::setConsistencyChecks(), SBMLDocument::checkConsistency(),
+ * SBMLDocument::checkInternalConsistency() and other method of that sort
+ * are in fact implemented by SBMLInternalValidator.  These validations
+ * are all performed on the internal (in-memory) representation of an SBML
+ * model.
+ *
+ * Users should not need to call SBMLInternalValidator methods directly,
+ * since the interface is already provided on SBMLDocument.  However, this
+ * class is exposed in case users would like to implement new or additional
+ * validations by extending this class (SBMLInternalValidator) or using
+ * this class as an example of how to implement such validators.
+ */
 
 #ifndef SBMLInternalValidator_h
 #define SBMLInternalValidator_h
@@ -151,6 +175,7 @@ public:
    */
   void setConsistencyChecks(SBMLErrorCategory_t category, bool apply);
 
+
   /**
    * Controls the consistency checks that are performed when
    * SBMLDocument::setLevelAndVersion(@if java long lev, long ver, boolean strict@endif) is called.
@@ -259,6 +284,7 @@ public:
   void setConsistencyChecksForConversion(SBMLErrorCategory_t category, 
                                          bool apply);
 
+
   /**
    * Performs consistency checking and validation on this SBML document.
    *
@@ -268,11 +294,17 @@ public:
    * flag in the individual SBMLError objects returned by
    * SBMLDocument::getError(@if java long n@endif) to determine the nature of the failures.
    *
+   * @param writeDocument by default checkConsistency will write the document
+   *                      in order to determine all errors for the document. 
+   *                      This will also clear the error log. Setting this
+   *                      parameter to false will skip this additional step
+   *                      but might not find all errors.
+   *
    * @return the number of failed checks (errors) encountered.
    *
    * @see SBMLDocument::checkInternalConsistency()
    */
-  unsigned int checkConsistency ();
+  unsigned int checkConsistency (bool writeDocument=false);
 
   
   /**
@@ -374,39 +406,35 @@ public:
    */
   unsigned int checkL3v1Compatibility ();
 
+
   /** 
    * @return the current list of selected validators
    */
   unsigned char getApplicableValidators() const;
 
+
   /** 
    * @return the current list of selected validators for conversion
    */
   unsigned char getConversionValidators() const;
 
+
   /** 
    * Set the current list of validators to be applied
    *
    * @param appl the mask of validators to be applied
-   * 
    */  
   void setApplicableValidators(unsigned char appl);
 
+
   /** 
    * Set the current list of conversion validators to be applied
    *
    * @param appl the mask of validators to be applied
-   *
    */
   void setConversionValidators(unsigned char appl);
 
 
-  /** @cond doxygen-libsbml-internal */
-  
-
-  /** @endcond */
-
-
   /**
    * Constructor.
    */
@@ -414,9 +442,15 @@ public:
 
 
   /**
-   * Copy constructor.
+   * Copy constructor; creates a copy of an SBMLInternalValidator object.
+   *
+   * @param orig the object to copy.
+   * 
+   * @throws @if python ValueError @else SBMLConstructorException @endif
+   * Thrown if the argument @p orig is @c NULL.
    */
-  SBMLInternalValidator(const SBMLInternalValidator&);
+  SBMLInternalValidator(const SBMLInternalValidator& orig);
+
 
   /**
    * Creates and returns a deep copy of this validator.
@@ -425,18 +459,27 @@ public:
    */
   virtual SBMLValidator* clone() const;
 
+
   /** 
-   * the actual conversion 
+   * Runs the validations.
    * 
-   * @return status code represeting success/failure/conversion impossible
+   * This runs the validations that were previously enabled via methods such as
+   * SBMLDocument::setConsistencyChecks(@if java int categ, boolean onoff@endif).
+   *
+   * @return the number of validation failures that occurred.  The objects
+   * describing the actual failures can be retrieved using getFailures().
    */
   virtual unsigned int validate();
 
+
 protected:
+  /** @cond doxygen-libsbml-internal */
 
   unsigned char mApplicableValidators;
   unsigned char mApplicableValidatorsForConversion;
 
+  /** @endcond */
+
 
 private:
   /** @cond doxygen-libsbml-internal */
