@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -94,6 +94,270 @@ using namespace std;
 
 /** @endcond */
 
+START_CONSTRAINT (20217, Model, x)
+{
+  // level 3
+  pre( m.getLevel() > 2);
+  pre( m.isSetTimeUnits());
+
+  const string&         units = m.getTimeUnits();
+  const UnitDefinition* defn  = m.getUnitDefinition(units);
+
+  inv_or( units == "second" );
+  inv_or( units == "dimensionless"  );
+  inv_or( defn  != NULL && defn->isVariantOfTime() );
+  inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+}
+END_CONSTRAINT
+
+
+START_CONSTRAINT (20218, Model, x)
+{
+  // level 3
+  pre( m.getLevel() > 2);
+  pre( m.isSetVolumeUnits());
+
+  const string&         units = m.getVolumeUnits();
+  const UnitDefinition* defn  = m.getUnitDefinition(units);
+
+  inv_or( units == "litre" );
+  inv_or( units == "dimensionless"  );
+  inv_or( defn  != NULL && defn->isVariantOfVolume() );
+  inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+}
+END_CONSTRAINT
+
+
+START_CONSTRAINT (20219, Model, x)
+{
+  // level 3
+  pre( m.getLevel() > 2);
+  pre( m.isSetAreaUnits());
+
+  const string&         units = m.getAreaUnits();
+  const UnitDefinition* defn  = m.getUnitDefinition(units);
+
+  inv_or( units == "dimensionless"  );
+  inv_or( defn  != NULL && defn->isVariantOfArea() );
+  inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+}
+END_CONSTRAINT
+
+
+START_CONSTRAINT (20220, Model, x)
+{
+  // level 3
+  pre( m.getLevel() > 2);
+  pre( m.isSetLengthUnits());
+
+  const string&         units = m.getLengthUnits();
+  const UnitDefinition* defn  = m.getUnitDefinition(units);
+
+  inv_or( units == "metre" );
+  inv_or( units == "dimensionless"  );
+  inv_or( defn  != NULL && defn->isVariantOfLength() );
+  inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+}
+END_CONSTRAINT
+
+
+START_CONSTRAINT (20221, Model, x)
+{
+  // level 3
+  pre( m.getLevel() > 2);
+  pre( m.isSetExtentUnits());
+
+  const string&         units = m.getExtentUnits();
+  const UnitDefinition* defn  = m.getUnitDefinition(units);
+
+  inv_or( units == "mole" );
+  inv_or( units == "item" );
+  inv_or( units == "dimensionless"  );
+  inv_or( units == "avogadro" );
+  inv_or( units == "kilogram" );
+  inv_or( units == "gram" );
+  inv_or( defn  != NULL && defn->isVariantOfSubstance() );
+  inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+}
+END_CONSTRAINT
+
+
+
+START_CONSTRAINT (20509, Compartment, c)
+{
+  pre( c.getSpatialDimensions() == 3 );
+  pre( c.isSetUnits()                );
+
+  if (  c.getLevel() == 1 
+    || (c.getLevel() == 2 &&  c.getVersion() == 1))
+  {
+    msg =
+      "The value of the 'units' attribute on a <compartment> having "
+      "'spatialDimensions' of '3' must be either 'volume', 'litre', or the "
+      "identifier of a <unitDefinition> based on either 'litre', 'metre' (with "
+      "'exponent' equal to '3').";
+  }
+  else
+  {
+    msg =
+      "The value of the 'units' attribute on a <compartment> having "
+      "'spatialDimensions' of '3' must be either 'volume', 'litre', or the "
+      "identifier of a <unitDefinition> based on either 'litre', 'metre' (with "
+      "'exponent' equal to '3'), or 'dimensionless'.";
+  }
+
+  const string&         units = c.getUnits();
+  const UnitDefinition* defn  = m.getUnitDefinition(units);
+
+  /* dimensionless is allowable in L2V2 */
+  if ( c.getLevel() == 1
+    || (c.getLevel() == 2 && c.getVersion() == 1))
+  {
+    inv_or( units == "volume" );
+    inv_or( units == "litre"  );
+    inv_or( units == "liter" && c.getLevel() == 1 );
+    inv_or( defn  != NULL && defn->isVariantOfVolume() );
+  }
+  else
+  {
+    inv_or( units == "volume" );
+    inv_or( units == "litre"  );
+    inv_or( units == "dimensionless"  );
+    inv_or( defn  != NULL && defn->isVariantOfVolume() );
+    inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+  }
+}
+END_CONSTRAINT
+
+
+START_CONSTRAINT (20608, Species, s)
+{
+  pre( s.isSetSubstanceUnits() );
+
+  if (s.getLevel() == 1)
+  {
+    msg =
+      "The value of a <species>'s 'units' attribute can only be one "
+      "of the following: 'substance', "
+      "or the identifier of a <unitDefinition> derived from "
+      "'mole' (with an 'exponent' of '1') or 'item' (with an 'exponent' "
+      "of '1').";
+  }
+  else if (s.getLevel() == 2)
+  {
+    if (s.getVersion() == 1)
+    {
+      msg =
+        "The value of a <species>'s 'substanceUnits' attribute can only be one "
+        "of the following: 'substance', 'mole' or 'item' "
+        "or the identifier of a <unitDefinition> derived from "
+        "'mole' (with an 'exponent' of '1') or 'item' (with an 'exponent' "
+        "of '1').";
+    }
+    else
+    {
+      msg =
+        "The value of a <species>'s 'substanceUnits' attribute can only be one "
+        "of the following: 'substance', 'mole', 'item', 'gram', 'kilogram', "
+        "'dimensionless', or the identifier of a <unitDefinition> derived from "
+        "'mole' (with an 'exponent' of '1'), 'item' (with an 'exponent' of '1')"
+        ", 'gram' (with an 'exponent' of '1'), 'kilogram' (with an 'exponent' "
+        "of '1'), or 'dimensionless'.";
+    }
+  }
+  else
+  {
+    msg =
+      "The value of a <species>'s 'substanceUnits' attribute can only be one "
+      "of the following: 'substance', 'mole', 'item', 'gram', 'kilogram', "
+      "'dimensionless', 'avogadro' or the identifier of a <unitDefinition> derived from "
+      "'mole' (with an 'exponent' of '1'), 'item' (with an 'exponent' of '1')"
+      ", 'gram' (with an 'exponent' of '1'), 'kilogram' (with an 'exponent' "
+      "of '1'), 'avogadro' (with an 'exponent' of '1') or 'dimensionless'.";
+  }
+
+  const string&         units = s.getSubstanceUnits();
+  const UnitDefinition* defn  = m.getUnitDefinition(units);
+
+  /* dimensionless/gram/kilogram are allowable in L2V2 */
+  if (s.getLevel() == 1 
+    || (s.getLevel() == 2 &&  s.getVersion() == 1))
+  {
+    inv_or( units == "substance" );
+    inv_or( units == "item"      );
+    inv_or( units == "mole"      );
+    inv_or( defn  != NULL && defn->isVariantOfSubstance() );
+  }
+  else if (s.getLevel() == 2)
+  {
+    inv_or( units == "substance"      );
+    inv_or( units == "item"           );
+    inv_or( units == "mole"           );
+    inv_or( units == "dimensionless"  );
+    inv_or( units == "gram"           );
+    inv_or( units == "kilogram"       );
+    inv_or( defn  != NULL && defn->isVariantOfSubstance()     );
+    inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+    inv_or( defn  != NULL && defn->isVariantOfMass()          );
+  }
+  else if (s.getLevel() == 3)
+  {
+    inv_or( units == "substance"      );
+    inv_or( units == "item"           );
+    inv_or( units == "mole"           );
+    inv_or( units == "dimensionless"  );
+    inv_or( units == "gram"           );
+    inv_or( units == "kilogram"       );
+    inv_or( units == "avogadro"       );
+    inv_or( defn  != NULL && defn->isVariantOfSubstance()     );
+    inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+    inv_or( defn  != NULL && defn->isVariantOfMass()          );
+  }
+}
+END_CONSTRAINT
+
+
+START_CONSTRAINT (20701, Parameter, p)
+{
+  pre( p.isSetUnits() );
+
+  //msg =
+  //  "The 'units' in a <parameter> definition must be a value chosen from "
+  //  "among the following: a value from the 'UnitKind' enumeration (e.g., "
+  //  "'litre', 'mole', 'metre', etc.), a built-in unit (e.g., 'substance', "
+  //  "'time', etc.), or the identifier of a <unitDefinition> in the model. "
+  //  "(References: L2V1 Section 4.7.3; L2V2 Section 4.9.3; L2V3 Section 4.9.3.)";
+
+  const string& units = p.getUnits();
+
+  inv_or( Unit::isUnitKind(units, p.getLevel(), p.getVersion())    );
+  inv_or( Unit::isBuiltIn(units, p.getLevel())     );
+  inv_or( m.getUnitDefinition(units) );
+}
+END_CONSTRAINT
+
+
+START_CONSTRAINT (99130, Model, x)
+{
+  // level 3
+  pre( m.getLevel() > 2);
+  pre( m.isSetSubstanceUnits());
+
+  const string&         units = m.getSubstanceUnits();
+  const UnitDefinition* defn  = m.getUnitDefinition(units);
+
+  inv_or( units == "mole" );
+  inv_or( units == "item" );
+  inv_or( units == "dimensionless"  );
+  inv_or( units == "avogadro" );
+  inv_or( units == "kilogram" );
+  inv_or( units == "gram" );
+  inv_or( defn  != NULL && defn->isVariantOfSubstance() );
+  inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+}
+END_CONSTRAINT
+
+
 /* check to report the use of parameters/numbers with undeclared units */
 START_CONSTRAINT (99505, AssignmentRule, ar)
 {
@@ -214,12 +478,21 @@ START_CONSTRAINT (99505, Priority, e)
 
   pre ( formulaUnits != NULL );
 
-  char * formula = SBML_formulaToString(e.getMath());
-  msg = "The units of the <event> <priority> expression '";
-  msg += formula;
-  msg += "' cannot be fully checked. Unit consistency reported as either no errors ";
-  msg += "or further unit errors related to this object may not be accurate.";
-  safe_free(formula);
+  if (e.isSetMath() == true)
+  {
+    char * formula = SBML_formulaToString(e.getMath());
+    msg = "The units of the <event> <priority> expression '";
+    msg += formula;
+    msg += "' cannot be fully checked. Unit consistency reported as either no errors ";
+    msg += "or further unit errors related to this object may not be accurate.";
+    safe_free(formula);
+  }
+  else
+  {
+    msg = "The <event> <priority> has no defined math expression. ";
+    msg += "Thus unit consistency reported as either no errors ";
+    msg += "or further unit errors related to this object may not be accurate.";
+  }
 
   inv( !formulaUnits->getContainsUndeclaredUnits());
 }
@@ -1464,6 +1737,5 @@ START_CONSTRAINT (20702, Parameter, p)
 END_CONSTRAINT
 
 
-
 /** @endcond */
 
