@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -32,6 +32,7 @@
 #include <sstream>
 
 #include <sbml/xml/XMLErrorLog.h>
+#include <sbml/xml/XMLConstructorException.h>
 #include <sbml/xml/XMLAttributes.h>
 /** @cond doxygen-libsbml-internal */
 #include <sbml/xml/XMLOutputStream.h>
@@ -45,16 +46,6 @@ using namespace std;
 /** @endcond */
 
 LIBSBML_CPP_NAMESPACE_BEGIN
-/** @cond doxygen-libsbml-internal */
-
-XMLConstructorException::XMLConstructorException(std::string message):
-      std::invalid_argument(message)
-{
-}
-
-
-/** @endcond */
-
 /*
  * @return s with whitespace removed from the beginning and end.
  */
@@ -335,7 +326,7 @@ XMLAttributes::getIndex (const XMLTriple& triple) const
 int
 XMLAttributes::getLength () const
 {
-  return mNames.size();
+  return (int)mNames.size();
 }
 
 
@@ -345,7 +336,7 @@ XMLAttributes::getLength () const
 int
 XMLAttributes::getNumAttributes () const
 {
-  return mNames.size();
+  return (int)mNames.size();
 }
 
 
@@ -529,7 +520,9 @@ XMLAttributes::readInto (  int          index
                          , const std::string& name
                          , bool&        value
                          , XMLErrorLog* log
-                         , bool         required ) const
+                         , bool         required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   bool assigned = false;
   bool missing  = true;
@@ -558,8 +551,8 @@ XMLAttributes::readInto (  int          index
 
   if ( log != NULL && !assigned && &name != NULL)
   {
-    if ( !missing ) attributeTypeError(name, Boolean, log);
-    else if ( required ) attributeRequiredError (name, log);
+    if ( !missing ) attributeTypeError(name, Boolean, log, line, column);
+    else if ( required ) attributeRequiredError (name, log, line, column);
   }
 
   return assigned;
@@ -585,9 +578,11 @@ bool
 XMLAttributes::readInto (  const std::string&   name
                          , bool&                value
                          , XMLErrorLog*         log
-                         , bool                 required ) const
+                         , bool                 required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
-   return readInto(getIndex(name), name, value, log, required);
+   return readInto(getIndex(name), name, value, log, required, line, column);
 }
 
 
@@ -608,10 +603,12 @@ bool
 XMLAttributes::readInto (  const XMLTriple& triple
                          , bool&            value
                          , XMLErrorLog*     log
-                         , bool             required ) const
+                         , bool             required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   if (&triple == NULL || &value == NULL) return (int)false;
-   return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required);
+   return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required, line, column);
 }
 
 
@@ -634,10 +631,12 @@ XMLAttributes::readInto (  const XMLTriple& triple
  */
 bool
 XMLAttributes::readInto (  int          index
-			 , const std::string& name
+                         , const std::string& name
                          , double&      value
                          , XMLErrorLog* log
-                         , bool         required ) const
+                         , bool         required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   bool assigned = false;
   bool missing  = true;
@@ -673,7 +672,7 @@ XMLAttributes::readInto (  int          index
         char*        endptr = NULL;
         const char*  nptr   = trimmed.c_str();
         double       result = strtod(nptr, &endptr);
-        unsigned int length = endptr - nptr;
+        unsigned int length = (unsigned int)(endptr - nptr);
 
         // Restore previous locale
         setlocale(LC_ALL, locale.empty() ? NULL : locale.c_str());
@@ -691,8 +690,8 @@ XMLAttributes::readInto (  int          index
 
   if ( log != NULL && !assigned && &name != NULL)
   {
-    if ( !missing ) attributeTypeError(name, Double, log);
-    else if ( required ) attributeRequiredError (name, log);
+    if ( !missing ) attributeTypeError(name, Double, log, line, column);
+    else if ( required ) attributeRequiredError (name, log, line, column);
   }
 
   return assigned;
@@ -720,9 +719,11 @@ bool
 XMLAttributes::readInto (  const XMLTriple& triple
                          , double&          value
                          , XMLErrorLog*     log
-                         , bool             required ) const
+                         , bool             required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
-  return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required);
+  return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required, line, column);
 }
 
 
@@ -745,9 +746,11 @@ bool
 XMLAttributes::readInto (  const std::string&   name
                          , double&          value
                          , XMLErrorLog*     log
-                         , bool             required ) const
+                         , bool             required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
-  return readInto(getIndex(name), name, value, log, required);
+  return readInto(getIndex(name), name, value, log, required, line, column);
 }
 
 
@@ -770,10 +773,12 @@ XMLAttributes::readInto (  const std::string&   name
  */
 bool
 XMLAttributes::readInto (  int          index
-			 , const std::string& name
+			                   , const std::string& name
                          , long&        value
                          , XMLErrorLog* log
-                         , bool         required ) const
+                         , bool         required 
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   bool assigned = false;
   bool missing  = true;
@@ -789,7 +794,7 @@ XMLAttributes::readInto (  int          index
       char*        endptr = NULL;
       const char*  nptr   = trimmed.c_str();
       long         result = strtol(nptr, &endptr, 10);
-      unsigned int length = endptr - nptr;
+      unsigned int length = (unsigned int)(endptr - nptr);
 
       if ((length == trimmed.size()) && (errno != ERANGE))
       {
@@ -803,8 +808,8 @@ XMLAttributes::readInto (  int          index
 
   if ( log != NULL && !assigned && &name != NULL )
   {
-    if ( !missing ) attributeTypeError(name, Integer, log);
-    else if ( required ) attributeRequiredError (name, log);
+    if ( !missing ) attributeTypeError(name, Integer, log, line, column);
+    else if ( required ) attributeRequiredError (name, log, line, column);
   }
 
   return assigned;
@@ -832,10 +837,12 @@ bool
 XMLAttributes::readInto (  const XMLTriple& triple
                          , long&            value
                          , XMLErrorLog*     log
-                         , bool             required ) const
+                         , bool             required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   if (&triple == NULL) return false;
-  return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required);
+  return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required, line, column);
 }
 
 /*
@@ -856,9 +863,11 @@ bool
 XMLAttributes::readInto (  const std::string& name
                          , long&              value
                          , XMLErrorLog*       log
-                         , bool               required ) const
+                         , bool               required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
-  return readInto(getIndex(name), name, value, log, required);
+  return readInto(getIndex(name), name, value, log, required, line, column);
 }
 
 
@@ -881,13 +890,15 @@ XMLAttributes::readInto (  const std::string& name
  */
 bool
 XMLAttributes::readInto (  int          index
-			 , const std::string& name
+			                   , const std::string& name
                          , int&         value
                          , XMLErrorLog* log
-                         , bool         required ) const
+                         , bool         required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   long  temp;
-  bool  assigned = readInto(index, name, temp, log, required);
+  bool  assigned = readInto(index, name, temp, log, required, line, column);
 
   if (assigned) value = temp;
   return assigned;
@@ -915,10 +926,12 @@ bool
 XMLAttributes::readInto (  const XMLTriple& triple
                          , int&             value
                          , XMLErrorLog*     log
-                         , bool             required ) const
+                         , bool             required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   if (&triple == NULL) return false;
-   return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required);    
+   return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required, line, column);    
 }
 
 
@@ -940,9 +953,11 @@ bool
 XMLAttributes::readInto (  const std::string&  name
                          , int&                value
                          , XMLErrorLog*        log
-                         , bool                required ) const
+                         , bool                required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
-  return readInto(getIndex(name), name, value, log, required);
+  return readInto(getIndex(name), name, value, log, required, line, column);
 }
 
 
@@ -965,13 +980,15 @@ XMLAttributes::readInto (  const std::string&  name
  */
 bool
 XMLAttributes::readInto (  int           index
-			 , const std::string& name
+			                   , const std::string& name
                          , unsigned int& value
                          , XMLErrorLog*  log
-                         , bool          required ) const
+                         , bool          required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   long  temp;
-  bool  assigned = readInto(index, name, temp, log, required);
+  bool  assigned = readInto(index, name, temp, log, required, line, column);
 
   if (assigned && temp >= 0) value = temp;
   else assigned = false;
@@ -1001,10 +1018,12 @@ bool
 XMLAttributes::readInto (  const XMLTriple& triple
                          , unsigned int&    value
                          , XMLErrorLog*     log
-                         , bool             required ) const
+                         , bool             required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   if (&triple == NULL) return false;
-  return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required);
+  return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required, line, column);
 }
 
 
@@ -1027,9 +1046,11 @@ bool
 XMLAttributes::readInto (  const std::string&  name
                          , unsigned int&       value
                          , XMLErrorLog*        log
-                         , bool                required ) const
+                         , bool                required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
-  return readInto(getIndex(name), name, value, log, required);
+  return readInto(getIndex(name), name, value, log, required, line, column);
 }
 
 
@@ -1049,7 +1070,9 @@ XMLAttributes::readInto (  int          index
                          , const std::string& name
                          , std::string& value
                          , XMLErrorLog* log
-                         , bool         required ) const
+                         , bool         required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   bool assigned = false;
 
@@ -1063,7 +1086,7 @@ XMLAttributes::readInto (  int          index
 
   if ( log != NULL && !assigned && required && &name != NULL )
   {
-    attributeRequiredError(name, log);
+    attributeRequiredError(name, log, line, column);
   }
 
   return assigned;
@@ -1083,12 +1106,14 @@ XMLAttributes::readInto (  int          index
  */
 bool
 XMLAttributes::readInto (  const XMLTriple& triple
-			 , std::string&     value
+			                   , std::string&     value
                          , XMLErrorLog*     log
-                         , bool             required ) const
+                         , bool             required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
   if (&triple == NULL) return false;
-  return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required);
+  return readInto(getIndex(triple), triple.getPrefixedName(), value, log, required, line, column);
 }
 
 
@@ -1103,11 +1128,13 @@ XMLAttributes::readInto (  const XMLTriple& triple
  */
 bool
 XMLAttributes::readInto (  const std::string& name
-			 , std::string&       value
+                         , std::string&       value
                          , XMLErrorLog*       log
-                         , bool               required ) const
+                         , bool               required
+                         , const unsigned int line     
+                         , const unsigned int column   ) const
 {
-  return readInto(getIndex(name), name, value, log, required);
+  return readInto(getIndex(name), name, value, log, required, line, column);
 }
 
 
@@ -1144,7 +1171,9 @@ XMLAttributes::write (XMLOutputStream& stream) const
 void
 XMLAttributes::attributeTypeError (  const std::string& name
 				   , DataType           type
-				   , XMLErrorLog*       log ) const
+				   , XMLErrorLog*       log
+           , const unsigned int line     
+           , const unsigned int column   ) const
 {
   ostringstream message;
 
@@ -1180,7 +1209,7 @@ XMLAttributes::attributeTypeError (  const std::string& name
       break;
   }
 
-  log->add( XMLError(XMLAttributeTypeMismatch, message.str()) );
+  log->add( XMLError(XMLAttributeTypeMismatch, message.str(), line, column) );
 }
 
 
@@ -1190,8 +1219,10 @@ XMLAttributes::attributeTypeError (  const std::string& name
  * @param  name  Name of the attribute
  */
 void
-XMLAttributes::attributeRequiredError (const std::string&  name,
-				       XMLErrorLog*        log) const
+XMLAttributes::attributeRequiredError (const std::string&  name
+				       , XMLErrorLog*        log
+               , const unsigned int line     
+               , const unsigned int column   ) const
 {
   ostringstream message;
 
@@ -1202,7 +1233,7 @@ XMLAttributes::attributeRequiredError (const std::string&  name,
   if ( !mElementName.empty() ) message << mElementName << ' ';
   message << "attribute '" << name << "' is required.";
 
-  log->add( XMLError(MissingXMLRequiredAttribute, message.str()) );
+  log->add( XMLError(MissingXMLRequiredAttribute, message.str(), line, column) );
 }
 
 /** @endcond */
