@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -30,6 +30,28 @@
  *
  * @htmlinclude not-sbml-warning.html
  *
+ * <!-- leave this next break as-is to work around some doxygen bug -->
+ */ 
+/**
+ * @class XMLConstructorException
+ * @brief Class of exceptions thrown by constructors of some libSBML objects.
+ *
+ * @htmlinclude not-sbml-warning.html
+ *
+ * In some situations, constructors for SBML objects may need to indicate
+ * to callers that the creation of the object failed.  The failure may be
+ * for different reasons, such as an attempt to use invalid parameters or a
+ * system condition such as a memory error.  To communicate this to
+ * callers, those classes will throw an XMLConstructorException.  @if cpp
+ * Callers can use the standard C++ <code>std::exception</code> method
+ * <code>what()</code> to extract the diagnostic message stored with the
+ * exception.@endif
+ * <p>
+ * In languages that don't have an exception mechanism (e.g., C), the
+ * constructors generally try to return an error code instead of throwing
+ * an exception.
+ *
+ * @see SBMLConstructorException
  */
 
 
@@ -57,17 +79,6 @@ class XMLErrorLog;
 class XMLOutputStream;
 /** @endcond */
 
-/** @cond doxygen-libsbml-internal */
-class LIBSBML_EXTERN XMLConstructorException : public std::invalid_argument
-{
-public:
-
-  /* constructor */
-  XMLConstructorException (std::string 
-                    message="NULL reference in XML constructor");
-};
-/** @endcond */
-
 class LIBLAX_EXTERN XMLAttributes
 {
 public:
@@ -86,12 +97,23 @@ public:
 
   /**
    * Copy constructor; creates a copy of this XMLAttributes set.
+   *
+   * @p orig the XMLAttributes object to copy.
+   *
+   * @throws @if python ValueError @else XMLConstructorException @endif
+   * Thrown if the argument @p orig is @c NULL.
    */
   XMLAttributes(const XMLAttributes& orig);
 
 
   /**
    * Assignment operator for XMLAttributes.
+   *
+   * @param rhs The XMLAttributes object whose values are used as the basis
+   * of the assignment.
+   *
+   * @throws @if python ValueError @else XMLConstructorException @endif
+   * Thrown if the argument @p rhs is @c NULL.
    */
   XMLAttributes& operator=(const XMLAttributes& rhs);
 
@@ -490,6 +512,8 @@ public:
    * @param value a boolean, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -503,7 +527,9 @@ public:
   bool readInto (  const std::string&  name
                  , bool&               value
                  , XMLErrorLog*        log      = NULL
-                 , bool                required = false ) const;
+                 , bool                required = false 
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -523,6 +549,8 @@ public:
    * @param value a boolean, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -531,7 +559,9 @@ public:
   bool readInto (  const XMLTriple& triple
                  , bool&        value
                  , XMLErrorLog* log      = NULL
-                 , bool         required = false ) const;
+                 , bool         required = false 
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
 
@@ -552,6 +582,8 @@ public:
    * @param value a double, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -565,7 +597,9 @@ public:
   bool readInto (  const std::string&  name
                  , double&             value
                  , XMLErrorLog*        log      = NULL
-                 , bool                required = false ) const;
+                 , bool                required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -585,6 +619,8 @@ public:
    * @param value a double, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -593,7 +629,9 @@ public:
   bool readInto (  const XMLTriple&  triple
                  , double&           value
                  , XMLErrorLog*      log      = NULL
-                 , bool              required = false ) const;
+                 , bool              required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -613,6 +651,8 @@ public:
    * @param value a long, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -626,7 +666,9 @@ public:
   bool readInto (  const std::string&  name
                  , long&               value
                  , XMLErrorLog*        log      = NULL
-                 , bool                required = false ) const;
+                 , bool                required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -646,6 +688,8 @@ public:
    * @param value a long, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -654,7 +698,9 @@ public:
   bool readInto (  const XMLTriple& triple
                  , long&            value
                  , XMLErrorLog*     log      = NULL
-                 , bool             required = false ) const;
+                 , bool             required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -674,6 +720,8 @@ public:
    * @param value an integer, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -687,7 +735,9 @@ public:
   bool readInto (  const std::string&  name
                  , int&                value
                  , XMLErrorLog*        log      = NULL
-                 , bool                required = false ) const;
+                 , bool                required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -707,6 +757,8 @@ public:
    * @param value an integer, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -715,7 +767,9 @@ public:
   bool readInto (  const XMLTriple& triple
                  , int&             value
                  , XMLErrorLog*     log      = NULL
-                 , bool             required = false ) const;
+                 , bool             required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -735,6 +789,8 @@ public:
    * @param value an unsigned integer, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -748,7 +804,9 @@ public:
   bool readInto (  const std::string&  name
                  , unsigned int&       value
                  , XMLErrorLog*        log      = NULL
-                 , bool                required = false ) const;
+                 , bool                required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -768,6 +826,8 @@ public:
    * @param value an unsigned integer, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -776,7 +836,9 @@ public:
   bool readInto (  const XMLTriple& triple
                  , unsigned int&    value
                  , XMLErrorLog*     log      = NULL
-                 , bool             required = false ) const;
+                 , bool             required = false 
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -790,6 +852,8 @@ public:
    * @param value a string, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -803,7 +867,9 @@ public:
   bool readInto (  const std::string&  name
                  , std::string&        value
                  , XMLErrorLog*        log      = NULL
-                 , bool                required = false ) const;
+                 , bool                required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -817,6 +883,8 @@ public:
    * @param value a string, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    *
@@ -825,7 +893,9 @@ public:
   bool readInto (  const XMLTriple& triple
                  , std::string&     value
                  , XMLErrorLog*     log       = NULL
-                 , bool              required = false ) const;
+                 , bool              required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /** @cond doxygen-libsbml-internal */
@@ -891,10 +961,14 @@ protected:
    * @param name  name of the attribute
    * @param type  the datatype of the attribute value.
    * @param log   the XMLErrorLog where the error should be logged
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    */
   void attributeTypeError (  const std::string& name
 			   , DataType           type
-			   , XMLErrorLog*       log ) const;
+			   , XMLErrorLog*       log
+         , const unsigned int line     = 0
+         , const unsigned int column   = 0) const;
 
 
   /**
@@ -903,8 +977,13 @@ protected:
    * 
    * @param name  name of the attribute
    * @param log   the XMLErrorLog where the error should be logged
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    */
-  void attributeRequiredError ( const std::string& name, XMLErrorLog* log ) const;
+  void attributeRequiredError ( const std::string& name
+        , XMLErrorLog* log 
+        , const unsigned int line     = 0
+        , const unsigned int column   = 0) const;
 
 
   /**
@@ -926,14 +1005,18 @@ protected:
    * @param value a boolean, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    */
   bool readInto (  int          index
-	         , const std::string&  name
+                 , const std::string&  name
                  , bool&        value
                  , XMLErrorLog* log      = NULL
-                 , bool         required = false ) const;
+                 , bool         required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -955,14 +1038,18 @@ protected:
    * @param value a double, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    */
   bool readInto (  int          index
-	         , const std::string&  name
+                 , const std::string&  name
                  , double&      value
                  , XMLErrorLog*  log      = NULL
-                 , bool          required = false ) const;
+                 , bool          required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -984,14 +1071,18 @@ protected:
    * @param value a long, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    */
   bool readInto (  int          index
-	         , const std::string&  name
+                 , const std::string&  name
                  , long&         value
                  , XMLErrorLog*  log      = NULL
-                 , bool          required = false ) const;
+                 , bool          required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -1013,14 +1104,18 @@ protected:
    * @param value an integer, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    */
   bool readInto (  int          index
-	         , const std::string&  name
+                 , const std::string&  name
                  , int&         value
                  , XMLErrorLog*  log      = NULL
-                 , bool          required = false ) const;
+                 , bool          required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -1042,14 +1137,18 @@ protected:
    * @param value an unsigned integer, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    */
   bool readInto (  int           index
-	         , const std::string&  name
+                 , const std::string&  name
                  , unsigned int& value
                  , XMLErrorLog*  log      = NULL
-                 , bool          required = false ) const;
+                 , bool          required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
   /**
@@ -1065,14 +1164,18 @@ protected:
    * @param value a string, the value of the attribute.
    * @param log an XMLErrorLog, the error log.
    * @param required a boolean, indicating whether the attribute is required.
+   * @param line an unsigned int, the line number at which the error occured.
+   * @param column an unsigned int, the column number at which the error occured.
    *
    * @returns @c true if the attribute was read into value, @c false otherwise.
    */
   bool readInto (  int          index
-	         , const std::string&  name
+                 , const std::string&  name
                  , std::string& value
                  , XMLErrorLog* log      = NULL
-                 , bool         required = false ) const;
+                 , bool         required = false
+                 , const unsigned int line     = 0
+                 , const unsigned int column   = 0) const;
 
 
 
