@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -30,6 +30,7 @@
 /** @cond doxygen-libsbml-internal */
 #include <sbml/xml/XMLOutputStream.h>
 #include <sbml/util/util.h>
+#include <sbml/xml/XMLConstructorException.h>
 /** @endcond */
 #include <sbml/xml/XMLToken.h>
 
@@ -124,7 +125,7 @@ XMLToken::XMLToken (  const std::string&  chars
 {
   if (&chars == NULL)
   {
-    throw XMLConstructorException();
+    throw XMLConstructorException("NULL reference in XML constructor");
   }
   else
   {
