@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2009-2011 jointly by the following organizations: 
+ * Copyright (C) 2009-2012 jointly by the following organizations: 
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EBML-EBI), Hinxton, UK
  *  
@@ -46,6 +46,8 @@
 #include <sbml/compress/CompressCommon.h>
 #include <sbml/compress/InputDecompressor.h>
 
+#include <sbml/common/common.h>
+
 
 /** @cond doxygen-ignored */
 
@@ -614,6 +616,8 @@ XercesParser::parse (const char* content, bool isFile)
 bool
 XercesParser::parseFirst (const char* content, bool isFile)
 {
+  if (content == NULL) return false;
+
   return parse(content, isFile, true);
 }
 
